// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: comfile.cpp
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/20/1999
// Date Last Modified: 06/21/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbsCommFileCache class is used to move data to and from disk
files, serial ports, and stream sockets using the vbs device cache.
*/
// ----------------------------------------------------------- // 
#include "comfile.h"

vbsCommFileCache::vbsCommFileCache(int CacheSize) : cache(CacheSize) 
{ 
  is_ok = 1; 
  ready_for_writing = 1; 
  ready_for_reading = 1;
  cache.Connect(this); 
}

void vbsCommFileCache::Read(void *buf, unsigned Bytes, vbDeviceTypes dev) 
{
  switch(dev) {
    case vbDEVICE_DISK_FILE:
      if(!infile) { is_ok = 0; ready_for_reading = 0; return; }
      else { is_ok = 1; ready_for_reading = 1; }
      infile.read((char *)buf, Bytes);
      break;
            
    case  vbDEVICE_STREAM_SOCKET:
      if(!stream_sock) { is_ok = 0; ready_for_reading = 0; return; }
      else { is_ok = 1; ready_for_reading = 1; }
      if(stream_sock->ReadRemoteBlock((char *)buf, curr_stream_hdr) != 0)
	DeviceErrorHandler(stream_sock->SocketExceptionMessage());
      break;

    default:
      break;
  }
}
  
void vbsCommFileCache::Write(const void *buf, unsigned Bytes,
			     vbDeviceTypes dev) 
{
  switch(dev) {
    case vbDEVICE_CONSOLE:
      cout.write((char *)buf, Bytes);
      break;

    case vbDEVICE_DISK_FILE:
      if(!outfile) { is_ok = 0; ready_for_writing = 0; return; }
      else { is_ok = 1; ready_for_writing = 1; }
      outfile.write((char *)buf, Bytes);
      break;
      
    case  vbDEVICE_STREAM_SOCKET:
      if(!stream_sock) { is_ok = 0; ready_for_writing = 0; return; }
      else { is_ok = 1; ready_for_writing = 1; }
      if(stream_sock->WriteBlock((char *)buf, Bytes) != 0)
	DeviceErrorHandler(stream_sock->SocketExceptionMessage());
      break;

    case vbDEVICE_SERIAL_PORT: 
      if(!serial_port) { is_ok = 0; ready_for_writing = 0; return; }
      else { is_ok = 1; ready_for_writing = 1; }
      if(serial_port->Send((char *)buf, Bytes) < 0)
	DeviceErrorHandler(serial_port->SerialCommExceptionMessage());
      break;

    default:
      break;
  }
}

long vbsCommFileCache::StaticFileSize(const char *fname)
// Returns the file size of fname. 
{
  struct stat buf;
  int result = stat(fname, &buf);
  if(result != 0) return 0;
  return buf.st_size;
}

int vbsCommFileCache::CopyFileToFile(const char *in, const char *out, int *bc)
// Copy a a disk file to another disk file using the device cache.
// Returns false if an error occurred.
{
  if(!OpenInputFile(in)) return 0;
  if(!OpenOutputFile(out)) return 0;
  long end_of_file = StaticFileSize(in);

  vbDeviceTypes o_device = vbDEVICE_DISK_FILE; // Output device
  vbDeviceTypes i_device = vbDEVICE_NULL; // No input buffering

  // Setup a pointer to the cache buckets
  devCachePointer p(cache, o_device, i_device); 
  *bc = LoadFile(p, end_of_file); // Load the file into the cache 
  cache.Flush(); // Ensure all the buckets a written to the output device
  infile.close();
  outfile.close();
  return 1;
}

unsigned vbsCommFileCache::LoadFile(devCachePointer p, long end_of_file)
// Load a file from disk into the device cache. Returns the
// number bytes read from the file. The __PACKET_SIZE__ will
// determine the length of each block. NOTE: If fix memory
// blocks are used the __PACKET_SIZE__ cannot exceed the
// size of the fixed block. Variable length memory blocks
// are use by default.
{
  unsigned i, byte_count = 0;
  char sbuf[__PACKET_SIZE__];
  for(i = 0; i < __PACKET_SIZE__; i++) sbuf[i] = 0;

  if(end_of_file <= __PACKET_SIZE__) { // The file is smaller then one block
    infile.read(sbuf, end_of_file);
    byte_count = end_of_file;
    p.Alloc();
    p->Load(sbuf, end_of_file);
  }
  else { // The file is larger then one block
    while(end_of_file > __PACKET_SIZE__) {
      for(i = 0; i < __PACKET_SIZE__; i++) sbuf[i] = 0; 
      byte_count += __PACKET_SIZE__;
      infile.read(sbuf, __PACKET_SIZE__);
      p.Alloc();
      p->Load(sbuf, __PACKET_SIZE__);
      end_of_file -= __PACKET_SIZE__;
      if(end_of_file <= __PACKET_SIZE__) {
	for(i = 0; i < __PACKET_SIZE__; i++) sbuf[i] = 0; 
	byte_count += end_of_file;
	infile.read(sbuf, end_of_file);
	p.Alloc();
	p->Load(sbuf, end_of_file);
	break;
      }
      else
	continue;
    }
  }
  return  byte_count;
}

int vbsCommFileCache::OpenInputFile(const char *in)
// Open the file if it exists. Returns false
// it the file cannot be opened or if it doee
// not exist.
{
#if defined (__WIN32__) || defined(__DOS__)
  // In MS-DOS/Windows there are two file types, text and binary
  infile.open(in, ios::in | ios::binary | ios::nocreate);
#elif defined(__UNIX__) 
  // In UNIX there is only one file type
  infile.open(in, ios::in | ios::nocreate);
#else
#error You must define a file system: __WIN32__ __DOS__ or __UNIX__
#endif

  if(!infile) return 0;
  infile_len = StaticFileSize(in);
  
  return 1;
}

int vbsCommFileCache::OpenOutputFile(const char *out)
// Open the specifed file for writing and truncate
// it. Returns false if the file cannot be opened.
{
#if defined (__WIN32__) || defined(__DOS__)
  // In MS-DOS/Windows there are two file types, text and binary
  outfile.open(out, ios::out | ios::binary | ios::trunc);
#elif defined(__UNIX__) 
  // In UNIX there is only one file type
  outfile.open(out, ios::out|ios::trunc);
#else
#error You must define a file system: __WIN32__ __DOS__ or __UNIX__
#endif

  if(!outfile) return 0;
  outfile_len = StaticFileSize(out);
  return 1;
}

int vbsCommFileCache::CopyFileToStreamSocket(vbStream *s, unsigned &byte_count)
// Copy a disk file to a stream socket. Returns true if no errors
// occurred. NOTE: This functions assumes that file has been open
// by the calling OpenInputFile/OpenOutputFile function and the
// socket has already been initialized. The calling function is
// responsible for flushing the cache buffers and closing any
// open disk files.
{
  if(!infile) return 0;
  if(!s) return 0;
  stream_sock = s;

  vbDeviceTypes o_device = vbDEVICE_STREAM_SOCKET; // Output device
  vbDeviceTypes i_device = vbDEVICE_DISK_FILE;     // Input device

  // Setup a pointer to the cache buckets
  devCachePointer p(cache, o_device, i_device); 
  byte_count = LoadFile(p, infile_len); // Load the file into the cache 

  return 1;
}

int vbsCommFileCache::CopyStreamSocketToSerialPort(vbStream *s,
						   vbSerialComm *p,
						   vbBlockHeader &vbs)
// Copy a stream socket to a serial port. Returns true if no errors
// occurred. NOTE: This functions assumes that socket and serial
// port has already been initialized. The calling function is
// responsible for flushing the cache buffers and closing any
// open disk files.
{
  if((!s) || (!p)) return 0;
  serial_port = p;
  stream_sock = s;
  curr_stream_hdr = vbs;

  vbDeviceTypes o_device = vbDEVICE_SERIAL_PORT;   // Output device
  vbDeviceTypes i_device = vbDEVICE_STREAM_SOCKET; // Input device

  // Setup a pointer to the cache buckets
  devCachePointer cp(cache, o_device, i_device); 

  // Load the cache buckets
  cp.Alloc((__ULWORD__)vbs.Length);

  return 1;
}

void vbsCommFileCache::DeviceErrorHandler(const char *mesg)
{
  // Display the device error and exit the program
  cout << mesg << endl;
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
