// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Header File Name: comfile.h
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/20/1999
// Date Last Modified: 06/21/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbsCommFileCache class is used to move data to and from disk
files, serial ports, and stream sockets using the vbs device cache.
*/
// ----------------------------------------------------------- // 
#ifndef __VBSCOMFILE_HPP__
#define __VBSCOMFILE_HPP__

// --------------------------------------------------------------
// Constants 
// --------------------------------------------------------------
const int __DEFAULT_CACHE_SIZE__ = 1024;  // Default cache size  
const int __DEFAULT_PACKET_SIZE__ = 1024; // Maximum length of each block

// Adjust these values for maximum performance
const int __CACHE_SIZE__ = 1024; // Number of cache buckets that will be used

const int __PACKET_SIZE__ = 32767; // Maximum packet size that will be used
// NOTE: If fix memory blocks are used as cache buckets the
// __PACKET_SIZE__ cannot exceed the size of the fixed block.
// Variable length memory blocks are use by default.
// --------------------------------------------------------------

// --------------------------------------------------------------
// Include files
// --------------------------------------------------------------
#include <fstream.h>
#include "devcache.h"
#include "vbstream.h"
#include "vbscomm.h"
#include "membuf.h"
#include "memblock.h"

// Define the __DOS_INCLUDES__ macro to use DOS path separators
#ifdef __DOS_INCLUDES__
#include <sys\types.h>
#include <sys\stat.h>
#else
#include <sys/types.h>
#include <sys/stat.h>
#endif
// --------------------------------------------------------------

// --------------------------------------------------------------
// Type definitons
// --------------------------------------------------------------
#if defined(__SMEMORY_BLOCK__) // Fixed length memory (static) blocks
typedef MemoryBlock MEMTYPE;

#elif defined(__VMEMORY_BUFFER__) // Variable length memory buffers
typedef MemoryBuffer MEMTYPE;

#else // Default to variable length memory buffers
typedef MemoryBuffer MEMTYPE;
#endif

typedef vbDeviceCachePtr devCachePointer; // Pointer to cache
// --------------------------------------------------------------

// Class used to create the device cache
class vbsCommFileCache : public vbDeviceCache
{
public:
  vbsCommFileCache(int CacheSize = __DEFAULT_CACHE_SIZE__);
  ~vbsCommFileCache() { }

public:
  void DeviceErrorHandler(const char *mesg);
  void Flush() { cache.Flush(); } // Flush the cache buckets
  unsigned BucketsInUse() { return cache.BucketsInUse(); }
  void CloseOutputFile() { infile.close(); }
  void CloseInputFile() { outfile.close(); }
  int OpenInputFile(const char *in);
  int OpenOutputFile(const char *out);
  long StaticFileSize(const char *fname);
  unsigned LoadFile(devCachePointer p, long end_of_file);
  int CopyFileToFile(const char *in, const char *out, int *bc = 0);
  int CopyStreamSocketToSerialPort(vbStream *s, vbSerialComm *p,
				   vbBlockHeader &vbs);
  int CopyFileToStreamSocket(vbStream *s, unsigned &byte_count);
  
private: // Base class interface
  void Read(void *buf, unsigned Bytes, vbDeviceTypes dev);
  void Write(const void *buf, unsigned Bytes, vbDeviceTypes dev);

private: // Device objects
  vbStream *stream_sock;     // Stream socket (network device) 
  vbSerialComm *serial_port; // Serial port (RS232 device)
  fstream outfile;           // File used to output data
  fstream infile;            // File used to input data

private: // Temporary variables used during read/write operations
  long infile_len;
  long outfile_len;
  vbBlockHeader curr_stream_hdr;

private: // Device cache
  vbDeviceBucketCache cache;

public: // Functions used to get the current device cache
  vbDeviceBucketCache *GetCache() { return &cache; }
};

#endif //__VBSCOMFILE_HPP__
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
