// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: vbsfile.cpp
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/20/1999
// Date Last Modified: 06/21/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbsFile class is used to move data to and from disk
files using the vbs device cache.
*/
// ----------------------------------------------------------- // 

// Define the __DOS_INCLUDES__ macro to use DOS path separators
#ifdef __DOS_INCLUDES__
#include <sys\types.h>
#include <sys\stat.h>
#else
#include <sys/types.h>
#include <sys/stat.h>
#endif

#include "vbsfile.h"

vbsFile::vbsFile(int CacheSize) : cache(CacheSize) 
{ 
  is_ok = ready_for_writing = ready_for_reading = 1;
  infile_len = outfile_len = 0;
  cache.Connect(this); 
  stream_sock = 0;
  datagram_sock = 0;
  serial_port = 0;
}

void vbsFile::Read(void *buf, unsigned Bytes, vbDeviceTypes dev) 
{
  switch(dev) {
    case vbDEVICE_DISK_FILE:
      if(!infile) { is_ok = 0; ready_for_reading = 0; return; }
      else { is_ok = 1; ready_for_reading = 1; }
      infile.read((char *)buf, Bytes);
      break;
      
    case  vbDEVICE_STREAM_SOCKET:
      if(!stream_sock) { is_ok = 0; ready_for_reading = 0; return; }
      else { is_ok = 1; ready_for_reading = 1; }
      if(stream_sock->ReadRemoteBlock((char *)buf, curr_stream_hdr) != 0)
	DeviceErrorHandler(stream_sock->SocketExceptionMessage());
      break;

    case vbDEVICE_DATAGRAM_SOCKET:
      if(!datagram_sock) { is_ok = 0; ready_for_reading = 0; return; }
      else { is_ok = 1; ready_for_reading = 1; }
      if(datagram_sock->ReadRemoteBlock((char *)buf, curr_datagram_hdr) != 0)
	DeviceErrorHandler(datagram_sock->SocketExceptionMessage());
      break;
      
    case vbDEVICE_SERIAL_PORT:
      if(!serial_port) { is_ok = 0; ready_for_reading = 0; return; }
      else { is_ok = 1; ready_for_reading = 1; }
      if(serial_port->ReadBlock((char *)buf, curr_serial_hdr) != 0)
	DeviceErrorHandler(serial_port->SerialCommExceptionMessage());
      break;

    case vbDEVICE_NULL:
      break;
      
    default:
      break;
  }
}

void vbsFile::Write(const void *buf, unsigned Bytes, vbDeviceTypes dev) 
{
  switch(dev) {
    case vbDEVICE_CONSOLE:
      cout.write((char *)buf, Bytes);
      break;

    case vbDEVICE_DISK_FILE:
      if(!outfile) { is_ok = 0; ready_for_writing = 0; return; }
      else { is_ok = 1; ready_for_writing = 1; }
      outfile.write((char *)buf, Bytes);
      break;

    case  vbDEVICE_STREAM_SOCKET: 
      if(!stream_sock) { is_ok = 0; ready_for_writing = 0; return; }
      else { is_ok = 1; ready_for_writing = 1; }
      if(stream_sock->WriteBlock((char *)buf, Bytes) != 0)
	DeviceErrorHandler(stream_sock->SocketExceptionMessage());
      break;
      
    case vbDEVICE_DATAGRAM_SOCKET:
      if(!datagram_sock) { is_ok = 0; ready_for_writing = 0; return; }
      else { is_ok = 1; ready_for_writing = 1; }
      if(datagram_sock->WriteBlock((char *)buf, Bytes) != 0)
	DeviceErrorHandler(datagram_sock->SocketExceptionMessage());
      break;

    case vbDEVICE_SERIAL_PORT: 
      if(!serial_port) { is_ok = 0; ready_for_writing = 0; return; }
      else { is_ok = 1; ready_for_writing = 1; }
      if(serial_port->WriteBlock((char *)buf, Bytes) != 0)
	DeviceErrorHandler(serial_port->SerialCommExceptionMessage());
      break;

    case vbDEVICE_NULL:
      break;
      
    default:
      break;
  }
}

long vbsFile::StaticFileSize(const char *fname)
// Returns the file size of fname. 
{
  struct stat buf;
  int result = stat(fname, &buf);
  if(result != 0) return 0;
  return buf.st_size;
}

int vbsFile::CopyFileToStreamSocket(vbStream *s, unsigned &byte_count)
// Copy a disk file to a stream socket. Returns true if no errors
// occurred. NOTE: This functions assumes that file has been open
// by the calling OpenInputFile/OpenOutputFile function and the
// socket has already been initialized. The calling function is
// responsible for flushing the cache buffers and closing any
// open disk files.
{
  if(!infile) return 0;
  if(!s) return 0;
  stream_sock = s;

  vbDeviceTypes o_device = vbDEVICE_STREAM_SOCKET; // Output device
  vbDeviceTypes i_device = vbDEVICE_DISK_FILE;     // Input device

  // Setup a pointer to the cache buckets
  devCachePointer p(cache, o_device, i_device); 
  byte_count = LoadFile(p, infile_len); // Load the file into the cache 

  return 1;
}

int vbsFile::CopyFileToDatagramSocket(vbDatagram *s, unsigned &byte_count) 
// Copy a disk file to a datagram socket. Returns true if no errors
// occurred. NOTE: This functions assumes that file has been open
// by the calling OpenInputFile/OpenOutputFile function and the
// socket has already been initialized. The calling function is
// responsible for flushing the cache buffers and closing any
// open disk files.
{
  if(!infile) return 0;
  if(!s) return 0;
  datagram_sock = s;

  vbDeviceTypes o_device = vbDEVICE_DATAGRAM_SOCKET; // Output device
  vbDeviceTypes i_device = vbDEVICE_DISK_FILE; 

  // Setup a pointer to the cache buckets
  devCachePointer p(cache, o_device, i_device); 
  byte_count = LoadFile(p, infile_len); // Load the file into the cache 

  return 1;
}

int vbsFile::CopyFileToSerialPort(vbSerialCommServer *s, unsigned &byte_count)
// Copy a disk file to a serial port. Returns true if no errors
// occurred. NOTE: This functions assumes that file has been open
// by the calling OpenInputFile/OpenOutputFile function and the
// port has already been initialized. The calling function is
// responsible for flushing the cache buffers and closing any
// open disk files.
{
  if(!outfile) return 0;
  if(!s) return 0;
  serial_port = s;

  vbDeviceTypes o_device = vbDEVICE_SERIAL_PORT; // Output device
  vbDeviceTypes i_device = vbDEVICE_DISK_FILE; 

  // Setup a pointer to the cache buckets
  devCachePointer p(cache, o_device, i_device); 
  byte_count = LoadFile(p, infile_len); // Load the file into the cache 

  return 1;
}

int vbsFile::CopyFileToFile(unsigned &byte_count)
// Copy a disk file to another disk file . Returns true if no errors
// occurred. NOTE: This functions assumes that files have been open
// by the calling OpenInputFile/OpenOutputFile function. The calling
// function is responsible for flushing the cache buffers and closing
// any open disk files.

{
  if(!infile) return 0;
  if(!outfile) return 0;
  
  vbDeviceTypes o_device = vbDEVICE_DISK_FILE; // Output device
  vbDeviceTypes i_device = vbDEVICE_DISK_FILE; 

  // Setup a pointer to the cache buckets
  devCachePointer p(cache, o_device, i_device); 
  byte_count = LoadFile(p, infile_len); // Load the file into the cache 
  return 1;
}

unsigned vbsFile::LoadFile(devCachePointer p, long end_of_file)
// Load a file from disk into the device cache. Returns the
// number bytes read from the file.
{
  unsigned i, byte_count = 0;
  char sbuf[MEMORY_BLOCK_SIZE];
  for(i = 0; i < MEMORY_BLOCK_SIZE; i++) sbuf[i] = 0;

  // The file is smaller then one block
  if(end_of_file <= (long)MEMORY_BLOCK_SIZE) { 
    byte_count = end_of_file;
    p.Alloc(end_of_file); // Load the cache bucket
  }
  else { // The file is larger then one block
    while(end_of_file > (long)MEMORY_BLOCK_SIZE) {
      for(i = 0; i < MEMORY_BLOCK_SIZE; i++) sbuf[i] = 0; 
      byte_count += MEMORY_BLOCK_SIZE;
      p.Alloc(MEMORY_BLOCK_SIZE); // Load the cache bucket
      end_of_file -= MEMORY_BLOCK_SIZE;
      if(end_of_file <= (long)MEMORY_BLOCK_SIZE) {
	for(i = 0; i < MEMORY_BLOCK_SIZE; i++) sbuf[i] = 0; 
	byte_count += end_of_file;
	p.Alloc(end_of_file); // Load the cache bucket
	break;
      }
      else
	continue;
    }
  }
  return  byte_count;
}

int vbsFile::OpenInputFile(const char *in)
// Open the file if it exists. Returns false
// it the file cannot be opened or if it does
// not exist.
{
#if defined (__WIN32__) || defined(__DOS__) 
  // In MS-DOS/Windows there are two file types, text and binary
  infile.open(in, ios::in | ios::binary | ios::nocreate);
#elif defined(__UNIX__) 
  // In UNIX there is only one file type
  infile.open(in, ios::in | ios::nocreate);
#else
#error You must define a file system: __WIN32__  __DOS__ or __UNIX__
#endif

  if(!infile) return 0;
  infile_len = StaticFileSize(in);
  
  return 1;
}

int vbsFile::OpenOutputFile(const char *out)
// Open the specifed file for writing and truncate
// it. Returns false if the file cannot be opened.
{
#if defined (__WIN32__) || defined(__DOS__)
  // In MS-DOS/Windows there are two file types, text and binary
  outfile.open(out, ios::out | ios::binary | ios::trunc);
#elif defined(__UNIX__) 
  // In UNIX there is only one file type
  outfile.open(out, ios::out|ios::trunc);
#else
#error You must define a file system: __WIN32__  __DOS__ or __UNIX__
#endif

  if(!outfile) return 0;
  outfile_len = StaticFileSize(out);
  return 1;
}

int vbsFile::CopyStreamSocketToFile(vbStream *s, vbBlockHeader &vbs)
{
  if(!s) return 0; 
  stream_sock = s;
  curr_stream_hdr = vbs;

  vbDeviceTypes o_device = vbDEVICE_DISK_FILE;     // Output device
  vbDeviceTypes i_device = vbDEVICE_STREAM_SOCKET; // Input device

  // Setup a pointer to the cache buckets
  devCachePointer p(cache, o_device, i_device); 

  // Load the cache buckets
  p.Alloc((__ULWORD__)vbs.Length);

  return 1;
}

int vbsFile::CopyDatagramSocketToFile(vbDatagram *s, vbBlockHeader &vbs)
{
  if(!s) return 0; 
  datagram_sock = s;
  curr_datagram_hdr = vbs;

  vbDeviceTypes o_device = vbDEVICE_DISK_FILE;       // Output device
  vbDeviceTypes i_device = vbDEVICE_DATAGRAM_SOCKET; // Input device

  // Setup a pointer to the cache buckets
  devCachePointer p(cache, o_device, i_device); 

  // Load the cache buckets
  p.Alloc((__ULWORD__)vbs.Length);

  return 1;
}

void vbsFile::CloseOutputFile()
{
  outfile_len = 0;
  outfile.close();
}

void vbsFile::CloseInputFile()
{
  infile_len = 0;
  infile.close();
}

void vbsFile::Flush()
{
  cache.Flush();
}

int vbsFile::CopySerialPortToFile(vbSerialCommServer *s, vbBlockHeader &vbs)
{
  if(!outfile) return 0;
  if(!s) return 0;
  serial_port = s;
  curr_serial_hdr = vbs;

  vbDeviceTypes o_device = vbDEVICE_DISK_FILE;   // Output device
  vbDeviceTypes i_device = vbDEVICE_SERIAL_PORT; // Input device 

  // Setup a pointer to the cache buckets
  devCachePointer p(cache, o_device, i_device); 

  // Load the cache buckets
  p.Alloc((__ULWORD__)vbs.Length);

  return 1;
}

void vbsFile::DeviceErrorHandler(const char *mesg)
{
  // Display the device error and exit the program
  cout << mesg << endl;
  exit(0);
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
