// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Header File Name: vbsfile.h
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/20/1999
// Date Last Modified: 06/21/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbsFile class is used to move data to and from disk
files using the vbs device cache.
*/
// ----------------------------------------------------------- // 
#ifndef __VBSFILE_HPP__
#define __VBSFILE_HPP__

#include <fstream.h>
#include <stdlib.h>
#include "devcache.h"
#include "membuf.h"
#include "memblock.h"
#include "vbstream.h"
#include "vbdatagm.h"
#include "scomserv.h"

#if defined(__SMEMORY_BLOCK__) // Fixed length memory (static) blocks
typedef MemoryBlock MEMTYPE;

#elif defined(__VMEMORY_BUFFER__) // Variable length memory buffers
typedef MemoryBuffer MEMTYPE;

#else // Default to variable length memory buffers
typedef MemoryBuffer MEMTYPE;
#endif

typedef vbDeviceCachePtr devCachePointer; // Pointer to cache


class vbsFile : public vbDeviceCache
{
public:
  vbsFile(int CacheSize = 1024);
  ~vbsFile() { }

public:
  void DeviceErrorHandler(const char *mesg);
  void Flush(); // Flush the cache buckets to the output device
  unsigned BucketsInUse() { return cache.BucketsInUse(); }
  void CloseOutputFile();
  void CloseInputFile();
  int OpenInputFile(const char *in);
  int OpenOutputFile(const char *out);
  long StaticFileSize(const char *fname);
  unsigned LoadFile(devCachePointer p, long end_of_file);
  int CopyFileToFile(unsigned &byte_count);
  int CopyFileToStreamSocket(vbStream *s, unsigned &byte_count);
  int CopyFileToDatagramSocket(vbDatagram *s, unsigned &byte_count);
  int CopyFileToSerialPort(vbSerialCommServer *s, unsigned &byte_count);
  int CopyStreamSocketToFile(vbStream *s, vbBlockHeader &vbs);
  int CopyDatagramSocketToFile(vbDatagram *s, vbBlockHeader &vbs);
  int CopySerialPortToFile(vbSerialCommServer *s, vbBlockHeader &vbs);

private: // Base class interface
  void Read(void *buf, unsigned Bytes, vbDeviceTypes dev);
  void Write(const void *buf, unsigned Bytes, vbDeviceTypes dev);

private: // Device objects
  fstream outfile;                 // File used to output data
  fstream infile;                  // File used to input data
  vbStream *stream_sock;           // Stream socket (network device) 
  vbDatagram *datagram_sock;       // Datagram socket (network device)
  vbSerialCommServer *serial_port; // Serial port (RS232 device)

private: // Temporary variables used during read/write operations
  long infile_len;
  long outfile_len;
  vbBlockHeader curr_stream_hdr;
  vbBlockHeader curr_datagram_hdr;
  vbBlockHeader curr_serial_hdr;

private: // Device cache
  vbDeviceBucketCache cache;

public: // Functions used to get the current device cache
  vbDeviceBucketCache *GetCache() { return &cache; }
};

#endif // __VBSFILE_HPP__
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
