// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: vbsema.cpp
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 03/25/2000
// Date Last Modified: 06/21/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD and VBS C++ classes are copyright (c) 1997 and 2000 by 
Douglas M. Gaer. All those who put this code or its derivatives 
in a commercial product MUST mention this copyright in their
documentation for users of the products in which this code or its 
derivative classes are used. Otherwise, you have the freedom to 
redistribute verbatim copies of this source code, adapt it to your 
specific needs, or improve the code and release your improvements 
to the public provided that the modified files carry prominent 
notices stating that you changed the files and the date of any
change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

Semaphore synchronization class implemented using condition variables
and a mutually exclusive integer variable. Typically, an application
uses a semaphore to limit the number of threads using a resource by
causing a thread to stop and wait until another thread signals that it
has arrived at a certain place in the entry function.
*/
// ----------------------------------------------------------- //   
#include "vbsema.h"

vbSemaphore::vbSemaphore(vbProcessType type)
{
  SemaphoreInit(type);
}

vbSemaphore::~vbSemaphore()
{
  SemaphoreDestroy();
}

int vbSemaphore::SemaphoreInit(vbProcessType type)
// Initialize the semaphore. Returns a non-zero value if
// any errors occur.
{
  if(vbThreadSemaphoreInit(&semaphore, type) != 0) return 1;
  return 0;
}

int vbSemaphore::SemaphoreDestroy()
// Destroy the semaphore. Returns a non-zero value if
// any errors occur.
{
  return vbThreadSemaphoreDestroy(&semaphore);
}

int vbSemaphore::SemaphoreWait()
// Decrement the semaphore and block if the semaphore
// value is zero until another thread signals a change.
// Returns a non-zero value if any errors occur.
{
  return vbThreadSemaphoreWait(&semaphore);
}

int vbSemaphore::SemaphorePost()
// Increments the semaphore and signals any threads that are blocked.
// Returns a non-zero value if any errors occur.
{
  return vbThreadSemaphorePost(&semaphore);
}

int vbSemaphore::SemaphoreDecrement()
// vbSemaphore::SemaphoreDecrement() is a non-blocking function
// that decrements the value of the semaphore. It allows threads
// to decrement the semaphore to some negative value as part of
// an initialization process. Decrements allow multiple threads
// to move up on a semaphore before another thread can go down.
// Returns a non-zero value if any errors occur.
{
  return vbThreadSemaphoreIncrement(&semaphore);
}

int vbSemaphore::SemaphoreIncrement()
// vbSemaphore::SemaphoreIncrement() is a non-blocking function
// that increments the value of the semaphore. Returns a non-zero
// value if any errors occur.
{
  return vbThreadSemaphoreIncrement(&semaphore);
}

int vbSemaphore::SemaphoreValue()
// Returns the value of the semaphore at the time the
// critical section is accessed. NOTE: The value may
// change after the function unlocks the critical
// section.
{
  return vbThreadSemaphoreValue(&semaphore);
}

const char *vbSemaphore::SemaphoreExceptionMessage()
// Returns a null termainated string that can
// be use to log or print a semaphore execption.
{
  return vbThreadSemaphoreExceptionMessage(&semaphore);
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
