// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: vbsema.h
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 03/25/2000
// Date Last Modified: 06/21/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBD and VBS C++ classes are copyright (c) 1997 and 2000 by 
Douglas M. Gaer. All those who put this code or its derivatives 
in a commercial product MUST mention this copyright in their
documentation for users of the products in which this code or its 
derivative classes are used. Otherwise, you have the freedom to 
redistribute verbatim copies of this source code, adapt it to your 
specific needs, or improve the code and release your improvements 
to the public provided that the modified files carry prominent 
notices stating that you changed the files and the date of any
change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

Semaphore synchronization class implemented using condition variables
and a mutually exclusive integer variable. Typically, an application
uses a semaphore to limit the number of threads using a resource by
causing a thread to stop and wait until another thread signals that it
has arrived at a certain place in the entry function.
*/
// ----------------------------------------------------------- //   
#ifndef __VBTHREAD_SEMAPHORE_HPP__
#define __VBTHREAD_SEMAPHORE_HPP__

#include "thrtypes.h"
#include "thelpers.h"

class vbSemaphore
{
public:
  vbSemaphore(vbProcessType type = vbPROCESS_PRIVATE);
  ~vbSemaphore();

private: // Disallow copying and assignment
  vbSemaphore(const vbSemaphore &ob) { }
  void operator=(const vbSemaphore &ob) { }

public:
  int SemaphoreInit(vbProcessType type = vbPROCESS_PRIVATE);
  int SemaphoreDestroy();
  int SemaphoreValue();
  int SemaphoreWait();
  int SemaphorePost();
  int SemaphoreDecrement();
  int SemaphoreIncrement();
  vbSemaphore_t *GetSemaphore() { return &semaphore; }
  vbSemaphoreError GetSemaphoreError() { return semaphore.semaphore_error; }
  vbProcessType GetSemaphoreProcessType() { return semaphore.process_type; }
  
public: // Semaphore status/debug message functions
  const char *SemaphoreExceptionMessage();
  
public: // Overloaded operators
  void operator++(int) { SemaphoreIncrement(); }
  void operator--(int) { SemaphoreDecrement(); }

public: // Conversion functions
  operator int() { return SemaphoreValue(); }

private:
  vbSemaphore_t semaphore;
};

#endif // __VBTHREAD_SEMAPHORE_HPP__
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
