// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: server.cpp
// C++ Compiler Used: MSVC40, HP CPP 10.24, egcs-2.90.29 
// Produced By: Doug Gaer   
// File Creation Date: 09/20/1999
// Date Last Modified: 06/21/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The program is used to test vbSerialCommServer class.
*/
// ----------------------------------------------------------- // 
#include <iostream.h>
#include <stdlib.h>
#include "scomserv.h"

int ReadBlock(vbSerialCommServer *server, vbBlockHeader &vb)
{
  cout << "Reading a raw block of data" << endl;
  
  char *buf = new char[(__ULWORD__)vb.Length];
  if(server->ReadBlock(buf, vb) != 0) return 1;

  buf[(__ULWORD__)vb.Length] = 0;
  cout << buf;
  cout << server->BytesRead() << " bytes received." << endl;
  
  delete buf;
  return 0;
}

int AnswerRequest(vbSerialCommServer *server, vbBlockHeader &vb)
{
  cout << "Client has requested a block" << endl;
 
  char *test_block = "The quick brown fox jumps over the lazy dog \
0123456789\n";

  char *buf = new char[(__ULWORD__)vb.Length];
  if(server->ReadBlock(buf, vb) != 0) return 1;

  buf[(__ULWORD__)vb.Length] = 0;
  cout << "Client has requested block: " << buf << endl;
  delete buf;

  cout << "Answering request" << endl;
  
  if(server->WriteBlock((char *)test_block, strlen(test_block)) != 0)
    return 1;

  return 0;
}

int ChangeBlock(vbSerialCommServer *server, vbBlockHeader &vb)
{
  cout << "Received a change block request" << endl;
 
  char *buf = new char[(__ULWORD__)vb.Length];
  if(server->ReadBlock(buf, vb) != 0) return 1;

  buf[(__ULWORD__)vb.Length] = 0;
  cout << "Client requested that block \"" << buf << "\" be changed to:" 
       << endl << flush;
  delete buf;

  vbBlockHeader block_header;
  if(server->ReadHeader(block_header) != 0) return 1;

  char *block = new char[(__ULWORD__)block_header.Length];
  if(server->ReadBlock((char *)block, block_header) != 0) return 1;

  cout << "Bytes = " << block_header.Length << endl;
  block[(__ULWORD__)block_header.Length] = 0;
  cout << block << flush;

  delete block;
  return 0;
}

int AddBlock(vbSerialCommServer *server, vbBlockHeader &vb)
{
  cout << "Received an add block request" << endl;
  
  char *block = new char[(__ULWORD__)vb.Length];
  if(server->ReadBlock(block, vb) != 0) return 1;

  block[(__ULWORD__)vb.Length] = 0;
  cout << "Client has requested the following block be added:" << endl;
  cout << block << flush;
  delete block;
  
  return 0;
}

int DeleteBlock(vbSerialCommServer *server, vbBlockHeader &vb)
{
  cout << "Received a delete block request" << endl;
  
  char *buf = new char[(__ULWORD__)vb.Length];
  if(server->ReadBlock(buf, vb) != 0) return 1;

  buf[(__ULWORD__)vb.Length] = 0;
  cout << "Client has requested block \"" << buf << "\" be deleted" << endl;
  delete buf;
  
  return 0;
}

int main(int argc, char **argv)
{
  if(argc < 2) {
    cout << "VB Serial port server" << endl;
    cout << "Usage: " << argv[0] << " device name [baud rate]" << endl;
    return 1;
  }

  vbSerialCommServer server;

  int baud_rate = 9600;
  if(argc > 2 && atoi(argv[2])) baud_rate = atoi(argv[2]);

  cout << "Initializing the VB serial port server..." << endl;
  int rv = server.InitCommServer(argv[1], baud_rate);

  if(rv != vbSerialComm::scomm_NO_ERROR) {
    cout << server.SerialCommExceptionMessage() << endl;
    return 1;
  }

  cout << "Opening VB serial port server on " << argv[1] << endl;
    
  while(1) { // Block until the next read. 
    // Wait for a block header
    vbBlockHeader vb;
    if(server.ReadHeader(vb) != 0) {
      cout << server.SerialCommExceptionMessage() << endl;
      return 1;
    }

    cout << "Client connecting on " << argv[1] << endl;

    // Read the status byte to determine what to do with this block
    __ULWORD__ block_status = vb.Status;
    __SBYTE__ status = (__SBYTE__)((block_status & 0xFF00)>>8);

    switch(status) { 
      // Process each block of data
      case vbAcknowledgeBlock:
	cout << "Received an acknowledge block command" << endl;
	cout << "Sending acknowledgment" << endl;
	if(server.WriteAckBlock() != 0)
	  cout << server.SerialCommExceptionMessage() << endl;	  
	break;

      case vbAddRemoteBlock:
	if(AddBlock(&server, vb) != 0) 
	  cout << server.SerialCommExceptionMessage() << endl;
	break;

      case vbChangeRemoteBlock:
	if(ChangeBlock(&server, vb) != 0) 
	  cout << server.SerialCommExceptionMessage() << endl;
	break;

      case vbRequestBlock:
	if(AnswerRequest(&server, vb) != 0) 
	  cout << server.SerialCommExceptionMessage() << endl;
	break;

      case vbDeleteRemoteBlock:
	if(DeleteBlock(&server, vb) != 0) 
	  cout << server.SerialCommExceptionMessage() << endl;
	break;

      case vbSendBlock :  
	if(ReadBlock(&server, vb) != 0) 
	  cout << server.SerialCommExceptionMessage() << endl;
	break;

      case vbCloseConnection : 
	cout << "Client sent a close connection command" << endl;
	break;

      case vbKillServer:
	cout << "Client shutdown the server" << endl;
	server.Close();
	return 0;
	
      default:
	cout << "Received bad block command from client" << endl;
	break;
    }
  }

  server.Close();
  cout << "Exiting..." << endl;
  return 0;
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
