// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: r_unix.cpp
// C++ Compiler Used: MSVC40, HP CPP 10.24, egcs-2.90.29 
// Produced By: Doug Gaer   
// File Creation Date: 09/20/1999
// Date Last Modified: 06/19/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

Code used by UNIX systems to read data from a serial port using
a signal handler. 
*/
// ----------------------------------------------------------- // 
#include <fcntl.h>

// #ifndef __USE_SIGNAL_IO__
// #define __USE_SIGNAL_IO__
// #endif 

#ifndef __USE_SIGNAL_IO__ // Do not interrupt driven I/O 
void SerialRead(vbSerialComm *dev, char *dumpfile = 0)
{
  fstream outfile;
  if(dumpfile) {
    cout << "Writing output to the " << dumpfile << " file" << endl; 
    outfile.open(dumpfile, ios::out|ios::trunc);
    if(!outfile) {
      cout << endl;
      cout << "Cannot open the: " << dumpfile << " capture file"
	   << endl;
      cout << "Exiting..." << endl;
      cout << endl;
      return;
    }
  }

  cout << "Waiting to receive..." << endl;
  cout << "Press Ctrl C to terminate." << endl;

  const int rxBufferLen = 255;
  char buf[rxBufferLen];

  // Loop while waiting for input
  while(1) {
    usleep(100000); // Allow for I/O recovery time 
    dev->RawRead((char *)buf, rxBufferLen);
    if(dev->BytesRead() > 0) {
      buf[dev->BytesRead()] = 0; // Null terminate the string
      if(dumpfile) { // Capture data to a file
	outfile.write((char *)buf, dev->BytesRead());
	outfile.flush();
      }
      else { // Write the data to the console
	cout << buf << flush;
      }
    }
  }
}

#else // Using interrupt driven I/O
#include <sys/signal.h>

int comSTOP=0;
void signal_handler_IO(int status);  // Signal handler
int wait_flag=1;  // true while no signal received 

void SerialRead(vbSerialComm *dev, char *dumpfile = 0)
{
  fstream outfile;
  if(dumpfile) {
    cout << "Writing output to the " << dumpfile << " file" << endl; 
    outfile.open(dumpfile, ios::out|ios::trunc);
    if(!outfile) {
      cout << endl;
      cout << "Cannot open the: " << dumpfile << " capture file"
	   << endl;
      cout << "Exiting..." << endl;
      cout << endl;
      return;
    }
  }

  struct sigaction saio; // Signal action
  const int rxBufferLen = 255;
  char buf[rxBufferLen];
    
  // Install the signal handler before making the device asynchronous
  saio.sa_handler = signal_handler_IO;
  saio.sa_flags = 0;
  sigaction(SIGIO, &saio, 0);
  
  // Allow this process to receive SIGIO
#if defined (FASYNC) // ASYNC I/O is a BDS feature
  // A SIGIO signal will be sent whenever input or
  // output becomes possible on this file descriptor.
  fcntl(dev->DeviceHandle(), F_SETOWN, getpid());
  fcntl(dev->DeviceHandle(), F_SETFL, FASYNC);

#elif defined (FIOSSAIOSTAT) // HPUX specific
  // Set the process ID that will receive the SIGIO signals
  ioctl(dev->DeviceHandle(), FIOSSAIOOWN, dev->GetTermIOS());
  ioctl(dev->DeviceHandle(), FIOSSAIOSTAT, dev->GetTermIOS());  

#else
#error Cannot receive SIGIO signals using ASYNC IO
#endif

  cout << "Waiting to receive..." << endl;
  cout << "Press Ctrl C to terminate." << endl;
  
  // Loop while waiting for input
  while(comSTOP == 0) {
    // After receiving SIGIO check the wait_flag to see if input
    // is available and can be read
    if(wait_flag == 0) {
      if(dev->RawRead((char *)buf, 255) < 0) {
	// Stop loop if an error occurs
	cerr << endl;
	cerr << "Read error!" << endl;
	cerr << "Exiting..." << endl;
	comSTOP = 1; 
      }
      buf[dev->BytesRead()] = 0; // Null terminate the string
      if(dumpfile) { // Capture data to a file
	outfile.write((char *)buf, dev->BytesRead());
	outfile.flush();
      }
      else { // Write the data to the console
	cout << buf << flush;
      }
      wait_flag = 1; // Wait for new input 
    }
  }
}

void signal_handler_IO(int status)
// Signal handler. Sets wait_flag to 0, to indicate above loop that  
// characters have been received. 
{
  cout.flush();
  wait_flag = 0;
}

#endif // __USE_SIGNAL_IO__
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
