// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: r_win32.cpp
// C++ Compiler Used: MSVC40, HP CPP 10.24, egcs-2.90.29 
// Produced By: Doug Gaer   
// File Creation Date: 09/20/1999
// Date Last Modified: 06/19/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

Code used by Win32 applications to read and write  data to and
from a serial port using overlapped I/O.
*/
// ----------------------------------------------------------- // 
void PrintError(LPCSTR str)
{
  LPVOID lpMessageBuffer;
  int error = GetLastError();
  FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER |
		FORMAT_MESSAGE_FROM_SYSTEM,
		NULL,
		error,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMessageBuffer,
		0,
		NULL
		);
  cout << str << ": " << error << " " << lpMessageBuffer << endl;
  LocalFree(lpMessageBuffer);
}

DWORD CALLBACK ConInThread(HANDLE h)
{
  // Starts a user input thread using the console.
  // Takes console characters and sends them to the com port.
  
  OVERLAPPED ov;
  HANDLE hconn = GetStdHandle(STD_INPUT_HANDLE);
  BOOL quit = FALSE;
  
  ZeroMemory(&ov, sizeof(ov));
  ov.hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
  if(ov.hEvent == INVALID_HANDLE_VALUE) {
    PrintError("CreateEvent failed");
    SetCommMask(h, 0);
    return 0;
  }
  
  SetConsoleMode(hconn, 0);
  cout << "Waiting to receive..." << endl;
  cout << "Press Esc to terminate." << endl;
  do {
    char buf[10];
    DWORD bytes_read = 0;
    
    // Wait for user to type something
    WaitForSingleObject(hconn, INFINITE);
    
    // Read the console buffer
    if(!ReadConsole(hconn, buf, sizeof(buf), &bytes_read, NULL)) {
      PrintError("ReadConsole failed");
      quit  = TRUE;
    }
    if(bytes_read) {
      DWORD write;
      // Check for Esc key
      for(write=0; write < bytes_read; write++) {
	// Terminate when Esc key is pressed
	if(buf[write] == 0x1b) {
	  quit = TRUE;
	  break;
	}
      }
      // Send it to the com port
      if(write) {
	if(!WriteFile(h, buf, write, &write, &ov)) {
	  if(GetLastError() == ERROR_IO_PENDING) {
	    if(!GetOverlappedResult(h, &ov, &write, TRUE)) {
	      PrintError("GetOverlappedResult failed");
	      quit = TRUE;
	    }
	  }
	} else {
	  PrintError("WriteFile failed");
	  quit = TRUE;
	}
      }
    }
    
  } while(!quit);
  
  // Tell terminal thread to quit
  SetCommMask(h, 0);
  
  return 0;
}

void SerialRead(vbSerialComm *dev, char *dumpfile = 0)
{
  fstream outfile;
  if(dumpfile) {
    cout << "Writing output to the " << dumpfile << " file" << endl;
    outfile.open(dumpfile, ios::out|ios::binary|ios::trunc);
    if(!outfile) {
      cout << endl;
      cout << "Cannot open the: " << dumpfile << " capture file"
	   << endl;
      cout << "Exiting..." << endl;
      cout << endl;
      return;
    }
  }

  HANDLE hconn = GetStdHandle(STD_OUTPUT_HANDLE);
  HANDLE h = dev->DeviceHandle();
  DWORD mask;
  DWORD id;
  OVERLAPPED ov;
    
  HANDLE hconin = CreateThread(NULL, 0, ConInThread, h, 0, &id);
  if(hconin == INVALID_HANDLE_VALUE) {
    PrintError("CreateThread failed");
    return;
  }
  CloseHandle(hconin);  // Don't need this handle
  
  ZeroMemory(&ov, sizeof(ov));

  // Create event for overlapped I/O
  ov.hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
  if(ov.hEvent == INVALID_HANDLE_VALUE)
    PrintError("CreateEvent failed");
  
  // Wait for received characters
  if(!SetCommMask(h, EV_RXCHAR))
    PrintError("SetCommMask failed");
  
  while(1) {
    // Get the event mask
    if(!WaitCommEvent(h, &mask, &ov)) {
      DWORD e = GetLastError();
      if(e == ERROR_IO_PENDING) {
	DWORD r;
	if(!GetOverlappedResult(h, &ov, &r, TRUE)) {
	  PrintError("GetOverlappedResult failed");
	  break;
	}
      } else {
	PrintError("WaitCommEvent failed");
	break;
      }
    }
    
    // If no event, then UI thread terminated with SetCommMask(h, 0)
    if(mask == 0) {
      break;
    }

    if(mask & EV_RXCHAR) {
      char buf[10];
      DWORD bytes_read;
      do {
	bytes_read = 0;
	if(!ReadFile(h, buf, sizeof(buf), &bytes_read, &ov)) {
	  if(GetLastError() == ERROR_IO_PENDING) {
	    if(!GetOverlappedResult(h, &ov, &bytes_read, TRUE)) {
	      PrintError("GetOverlappedResult failed");
	      break;
	    }
	  } else {
	    PrintError("ReadFile failed");
	    break;
	  }
	}
	if(bytes_read) {
	  if(dumpfile) { // Capture the data to a file
	    outfile.write((char *)buf, bytes_read);
	    outfile.flush();
	  }
	  else // Write the data to the console
	    WriteFile(hconn, buf, bytes_read, &bytes_read, NULL);
	}
      } while(bytes_read);
    }
    mask = 0;
  }
  outfile.close();
  CloseHandle(ov.hEvent);  // Close the event
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
