// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: ustring.h 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer 
// File Creation Date: 11/29/1996
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The UString class is a user-defined string class used to 
create and manipulate non null-terminated resizable, variable-
length strings of binary data. UString objects are implemented 
as a concrete data type, just like the built-in data types: char,
int, long, float, and double. 
*/
// ----------------------------------------------------------- //   
#ifndef __USER_DEFINED_STRING_HPP__
#define __USER_DEFINED_STRING_HPP__

#include <iostream.h>
#include <iomanip.h>
#include <string.h>


// The StrData class is used to hold the string data and its associated 
// reference count. Reference counting is a technique used to ensure the 
// safe deletion or modification of an object. It works by storing a 
// reference count with each object. Each time the object is used (referenced)
// by some other entity the counter is incremented. When the entity is 
// finished with the object the counter is decremented. When the counter 
// goes to zero, it means that no entities are referencing the object, so it 
// can be safely deleted or modified. StrData objects are always allocated 
// dynamically. When allocated, enough additional memory will be reserved to 
// handle the maximum string length and the reference count will be set to 
// one. StrData objects with a reference count of one indicate that a string 
// is unique and can be modified during assignment, sub-string creation, 
// concatenation, fill, replacement, or insertion operations performed by 
// the UString class. If the string is not unique the UString class will make 
// a copy of the string data. 
class StrData 
{
private:
  friend class UString;
  StrData() { RefCount = 1; }

private:
  void *operator new(size_t StrSize, unsigned Bytes = 0);
  void operator delete(void *ptr);
  
private:
  unsigned int RefCount;   // Reference count for string text
  char Data[1];            // Start of the string text
  static StrData Null_Ptr; // For null strings
};

// The UString class is responsible for constructing StrData objects. 
// After the overloaded new operator in the StrData class has allocated 
// the memory, the UString class will copy the actual string data into 
// memory. The allocated length of the sting data is referred to as the 
// dimensioned length of the string. The number of bytes that the string 
// data occupies is referred to as its logical length. Strings will be 
// allowed to grow by a "grow by" value specified when a UString object 
// is constructed. A "grow by" value of zero will not allow the string 
// to grow or be reallocated. 
class UString
{
public:
  enum { // UString enumerated constants
    NoMatch = -1, // Value returned if a matching element is not found
    DefSize = 16, // Default string size used when a string is constructed
    DefInc  = 16  // Default increment size used when a string grows
  };

public:
  UString(unsigned Bytes = DefSize, int GB = DefInc) { Alloc(Bytes, GB); }
  UString(const char *s, int GB = DefInc);
  UString(const char *s, unsigned Bytes, int GB = DefInc);
  
  // Constructor used to create substrings
  UString(const UString &s,
	 unsigned Offset, unsigned Bytes, int GB = DefInc);

  ~UString();
  UString(const UString &s);
  UString &operator=(const UString &s);
  UString &operator=(const char *s) { Copy(s); return *this; }
  
public: // Append and copy functions
  void Cat(const char *s, unsigned Bytes) { InsReplAt(Len, s, Bytes); }
  void Cat(const char *s) { InsReplAt(Len, s, strlen(s)); }
  void Copy(const char *s);    // Copy data from s into this object  
  void Copy(const UString &s); // Copy data from s into this object

public: // String search function
  unsigned Find(char *s, unsigned Offset = 0); 
  unsigned Find(char *s, unsigned Bytes, unsigned Offset = 0) const;
  unsigned Find(const UString &s, unsigned Offset = 0) const {
    return Find(s.TextPtr, s.Len, Offset);
  }

  // Case insensitive pattern searches
  unsigned IFind(char *s, unsigned Offset = 0); 
  unsigned IFind(char *s, unsigned Bytes, unsigned Offset = 0) const;
  unsigned IFind(const UString &s, unsigned Offset = 0) const {
    return IFind(s.TextPtr, s.Len, Offset);
  }

public: // Insert, delete, and remove functions
  unsigned DeleteAt(unsigned Pos, unsigned Bytes);
  unsigned ReplaceAt(unsigned Pos, const char *s, unsigned Bytes) {
    return InsReplAt(Pos, s, Bytes, 0);
  }
  unsigned ReplaceAt(unsigned Pos, const char *s) {
    return InsReplAt(Pos, s, strlen(s), 0);
  }
  unsigned ReplaceAt(unsigned Pos, const UString &s) {
    return InsReplAt(Pos, s.TextPtr, s.Len, 0);
  }
  unsigned InsertAt(unsigned Pos, const char *s, unsigned Bytes) {
    return InsReplAt(Pos, s, Bytes);
  }
  unsigned InsertAt(unsigned Pos, const char *s) {
    return InsReplAt(Pos, s, strlen(s));
  }
  unsigned InsertAt(unsigned Pos, const UString &s) {
    return InsReplAt(Pos, s.TextPtr, s.Len);
  }
  
public: // Fill functions
  void Fill(const char *p, unsigned Bytes, unsigned Offset=0, unsigned Ln=0);
  void Fill(const char c, unsigned Offset=0, unsigned Ln=0) {
    Fill(&c, 1, Offset, Ln); }
  void Fill(const char *s, unsigned Offset=0, unsigned Ln=0) {
    Fill(s, strlen(s), Offset, Ln); }
  void Fill(const UString &s, unsigned Offset=0, unsigned Ln=0) {
    Fill(s.TextPtr, s.Len, Offset, Ln); }

public: // Sub-string functions
  UString Mid(unsigned Index, unsigned Bytes) const; 
  UString Left(unsigned Ln) const;  // Returns left substring of length Ln
  UString Right(unsigned Ln) const; // Returns right substring

public: // C String, pointer, and length functions
  const char *NullTermStr(); // Null-terminate the string
  unsigned GetLength() const { return Len; }    // Returns logical length
  unsigned GetDimLen() const { return DimLen; } // Returns allocated length
  char *GetTextPtr() { return TextPtr; } // Returns pointer to start of text
  const char *GetTextPtr() const { return TextPtr; } 
  int IsNull() const { return DataPtr == &StrData::Null_Ptr; }
  
  char *str_ptr() { return TextPtr; } // Returns pointer the string
  const char *str_ptr() const { return TextPtr; } 
  unsigned length() const { return Len; } // Returns logical length
  char *c_str(); // Returns NULL terminated C string 
  const char *c_str() const;
  int is_null() const { return DataPtr == &StrData::Null_Ptr; }

public: // Formatting and filtering functions
  int ToUpper();
  int ToLower();
  unsigned FindLast(const char *s);
  unsigned FindLast(char *s);
  unsigned ReplaceChar(const char c, const char replacement,
		       unsigned Offset = 0);
  unsigned ReplaceString(const char *s, const char *replacement,
			 unsigned Offset = 0);
  unsigned ReplaceString(char *s, char *replacement, unsigned Offset = 0);
  unsigned FilterString(const char *s, unsigned Offset = 0);
  unsigned FilterString(char *s, unsigned Offset = 0);
  unsigned FilterChar(const char c, unsigned Offset = 0);
  unsigned TrimLeadingSpaces();
  unsigned TrimTrailingSpaces();
  int DeleteAfter(const char *s);
  int DeleteBefore(const char *s);
  int DeleteAfterIncluding(const char *s);
  int DeleteBeforeIncluding(const char *s);
  int DeleteAfterLast(const char *s);
  int DeleteAfterLastIncluding(const char *s);
  int DeleteBeforeLast(const char *s);
  int DeleteBeforeLastIncluding(const char *s);
  
public: // Allocation functions
  UString *StrDup();
  UString *StrDup() const;
  int Grow() { return Realloc(DimLen + GrowBy); } // Grow by default amount
  int Resize(unsigned Bytes) { return Realloc(Bytes); } // Resize the string
  int resize(unsigned Bytes , int keep = 0); // Adjust the logical length
  int FreeExtra() { return Realloc(Len); }   // Frees unused space in string
  void ChgGrowByInc(int GB) { GrowBy = GB; } // Change GrowBy increment
  void SetLength(unsigned Bytes);
  
private: // Internal processing functions 
  void CopyN(const char *s, unsigned Bytes); 
  unsigned InsReplAt(unsigned Pos, const char *s, unsigned Bytes, int Ins=1);
  int Grow_By(unsigned Bytes) { return Realloc(DimLen + Bytes); }
  int EnsureUnique(); // Ensures that this string uniquely owns its data
  int IsUnique() const { return DataPtr->RefCount == 1 || IsNull(); }
  
private: // Internal memory allocation routines
  int Alloc(unsigned Bytes, int GB);
  int Realloc(unsigned NewDimLen, int Keep = 1);
  void Bind(const UString &s);
  void UnBind(); // Unbinds this object from the shared string data
  void NewBinding(const UString &s);

private: // Data members
  StrData *DataPtr;
  char *TextPtr;   // Pointer to start of string text
  unsigned DimLen; // Maximum length without reallocation
  unsigned Len;    // Logical length of text
  int GrowBy;      // Minimum resizing increment

public: // Class type to ostream type conversions
  friend ostream &operator<<(ostream &os, const UString &s);
  friend istream &operator>>(istream &os, UString &s);

public: // Overloaded operators
  int operator!() { return ((Len == 0) || (*TextPtr == 0)); } 
  int operator!() const { return ((Len == 0) || (*TextPtr == 0)); }
  operator const int () const { return ((Len > 0) || (*TextPtr)); }
  operator int () { return ((Len > 0) || (*TextPtr)); }
  char &operator[](unsigned i);
  char &operator[](unsigned i) const;
  void operator+=(const UString &s) { Cat(s.TextPtr, s.Len); }
  void operator+=(const char *s) { Cat(s, strlen(s)); }
  void operator+=(const char c);
  friend UString operator+(const UString &a, const UString &b);
  friend UString operator+(const char *s, const UString &b);
  friend UString operator+(const UString &a, const char *s);
  friend int operator==(const UString &a, const UString &b);
  friend int operator==(const char *s, const UString &b);
  friend int operator==(const UString &a, const char *s);
  friend int operator!=(const UString &a, const UString &b);
  friend int operator!=(const char *s, const UString &b);
  friend int operator!=(const UString &a, const char *s);
  friend int operator>(const UString &a, const UString &b);
  friend int operator>(const char *s , const UString &b);
  friend int operator>(const UString &a, const char *s);
  friend int operator>=(const UString &a, const UString &b);
  friend int operator>=(const char *s, const UString &b);
  friend int operator>=(const UString &a, const char *s);
  friend int operator<(const UString &a, const UString &b);
  friend int operator<(const char *s, const UString &b);
  friend int operator<(const UString &a, const char *s);
  friend int operator<=(const UString &a, const UString &b);
  friend int operator<=(const char *s, const UString &b);
  friend int operator<=(const UString &a, const char *s);

public: // Friend functions
  // Returns -1 if a < b, 0 if a == b, and 1 if a > b
  friend int StringCompare(const UString &a, const UString &b);  
  friend int StringCompare(const char *s, const UString &b) {
    UString a(s); return StringCompare(a, b);
  }
  friend int StringCompare(const UString &a, const char *s) {
    UString b(s); return StringCompare(a, b);
  }
};

// General-purpose string routines that need to be ported from UNIX to DOS
int CaseICmp(const UString &s1, const UString &s2);
int CaseICmp(UString &s1, UString &s2);
int CaseICmp(const UString &s1, const char *s);
int CaseICmp(const char *s, const UString &s2);

#endif  // __USER_DEFINED_STRING_HPP__
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //

