// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Header File Name: vbconfig.h
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer  
// File Creation Date: 02/19/1996 
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbConfig class is a character string based singly linked list
that is used to load program parameters from a ASCII text based
configuration file.
*/
// ----------------------------------------------------------- // 
#ifndef __VBCONFIG_HPP__
#define __VBCONFIG_HPP__

#include "vblistb.h"
#include "vbstring.h"

// Configuration data nodes 
class vbConfigListNode: public vbDNodeBase
{
public:
  vbConfigListNode() { } 
  vbConfigListNode(const vbString &X) : Data(X) { } 

public:
  vbConfigListNode *GetNext() { return (vbConfigListNode *)Next; }
  const vbConfigListNode *GetNext() const {
    return (vbConfigListNode *)Next;
  }

public:
  vbString Data;
};

// Singly linked list used to store configuration data
class vbConfigList : public vbDLListBase
{
public:
  vbConfigList() { }
  virtual ~vbConfigList();
  vbConfigList(const vbConfigList &X) { Copy(X); }
  void operator=(const vbConfigList &X) { Copy(X); }

public:
  int Copy(const vbConfigList &List);
  const vbConfigListNode *Find(const vbString &X,
			      const vbConfigListNode *ptr=0) const;
  vbConfigListNode *Find(const vbString &X, vbConfigListNode *ptr=0);
  vbConfigListNode *FindAny(const vbString &X, vbConfigListNode *ptr=0);
  vbConfigListNode *Store(const vbString &X);
  vbConfigListNode *GetFront();
  const vbConfigListNode *GetFront() const;
  int IsHeader(const vbConfigListNode *Node) const;
  
protected:
  virtual vbConfigListNode *AllocNode(const vbString &X);
  virtual vbDNodeBase *DupNode(const vbDNodeBase *Node);
  virtual void FreeNode(vbDNodeBase *Node);
};

// Configuration manager class
class vbConfig : public vbConfigList
{
public:
  vbConfig() {
    FileName = "\0"; parm_ID = "="; comment_char = '#';
    filter_comments = 1;
  }
  vbConfig(const vbString &fname) {
    FileName = fname; parm_ID = "="; comment_char = '#';
    filter_comments = 1;
  }
  vbConfig(char *fname) {
    FileName = fname; parm_ID = "="; comment_char = '#';
    filter_comments = 1;
  }
  vbConfig(const char *fname) {
    FileName = fname; parm_ID = "="; comment_char = '#';
    filter_comments = 1;
  }

  // Constructors add to set the parameter ID string: 09/17/1999
  vbConfig(const vbString &fname, const vbString &p_id) {
    FileName = fname; parm_ID = p_id;
  }
  vbConfig(const vbString &fname, char *p_id) {
    FileName = fname; parm_ID = p_id;
  }
  vbConfig(const vbString &fname, const char *p_id) {
    FileName = fname; parm_ID = p_id;
  }
  ~vbConfig();

private:
  vbConfig(const vbConfig &ob) { }       // Disallow copying
  void operator=(const vbConfig &ob) { } // Disallow assignment

public: // Functions used to load and unload config file
  int Load();
  int Load(char *fname); 
  int Load(const char *fname);
  int Load(const vbString &fname);
  int ReLoad();
  int ReLoad(char *fname); 
  int ReLoad(const char *fname);
  int ReLoad(const vbString &fname);
  void UnLoad();
  char *GetFileName() { return FileName.c_str(); }
  void SetFileName(const vbString &s) { FileName = s; }
  void SetFileName(const char *s) { FileName = s; }
  void SetFileName(char *s) { FileName = s; }
  
public: // Formatting functions
  // Treat all characters after the ID string as a parameter value
  void SetParmID(const vbString &s) { parm_ID = s; }
  void SetParmID(char *s) { parm_ID = s; }
  void SetParmID(const char *s) { parm_ID = s; }
  char *GetParmID() { return parm_ID.c_str(); }
  char GetCommentChar() { return comment_char; }
  void SetCommentChar(char c) { comment_char = c; }
  void FilterComments() { filter_comments = 1; }
  void ReadComments() { filter_comments = 0; }
  
public: // Functions used to read config file values
  double GetFloatValue(char *Name, int fn=1); // Floating point 
  double GetFloatValue(const char *Name, int fn=1);    
  double GetFloatValue(const vbString &Name, int fn=1); 
  double GetDFPValue(char *Name, int fn=1);         // Floating point values
  double GetDFPValue(const char *Name, int fn=1);              
  double GetDFPValue(const vbString &Name, int fn=1);           
  char* GetStrValue(char *Name, int fn=1);          // Character string values
  char* GetStrValue(const char *Name, int fn=1);    
  char* GetStrValue(const vbString &Name, int fn=1); 
  int GetIntValue(char *Name, int fn=1);            // Integer values
  int GetIntValue(const char *Name, int fn=1);      
  int GetIntValue(const vbString &Name, int fn=1);   
  long GetLongValue(char *Name, int fn=1);          // Long integer values
  long GetLongValue(const char *Name, int fn=1);    
  long GetLongValue(const vbString &Name, int fn=1); 

public: // Functions used to write config values
  int WriteConfigLine(const vbString &parm, const vbString &value);
  int WriteConfigLine(const char *parm, const char *value);
  int WriteConfigLine(char *parm, char *value);
  int WriteCommentLine(const vbString &s);
  int WriteCommentLine(const char *s);
  int WriteCommentLine(char *s);
  int WriteLine(const vbString &s);
  int WriteLine(const char *s);
  int WriteLine(char *s);
  int ChangeConfigValue(const vbString &parm, const vbString &value);
  int ChangeConfigValue(const char *parm, const vbString &value);
  int ChangeConfigValue(const vbString &parm, const char *value);
  int ChangeConfigValue(char *parm, const vbString &value);
  int ChangeConfigValue(const vbString &parm, char *value);
  int ChangeConfigValue(const char *parm, const char *value);
  int ChangeConfigValue(char *parm, char *value);
  int ChangeConfigValue(const vbString &parm, int value);
  int ChangeConfigValue(const char *parm, int value);
  int ChangeConfigValue(char *parm, int value);
  int ChangeConfigValue(const vbString &parm, unsigned value);
  int ChangeConfigValue(const char *parm, unsigned value);
  int ChangeConfigValue(char *parm, unsigned value);
  int ChangeConfigValue(const vbString &parm, long value);
  int ChangeConfigValue(const char *parm, long value);
  int ChangeConfigValue(char *parm, long value);
  int ChangeConfigValue(const vbString &parm, float value);
  int ChangeConfigValue(const char *parm, float value);
  int ChangeConfigValue(char *parm, float value);
  int ChangeConfigValue(const vbString &parm, double value);
  int ChangeConfigValue(const char *parm, double value);
  int ChangeConfigValue(char *parm, double value);
  
private: // Internal proccessing functions
  void FilterLineFeed(char &c);
  int StoreCfgData(const vbString &str);
  int ChangeConfigLine(const vbString &string_to_replace,
		       const vbString &string_to_insert,
		       int check_case, int append_lines);
  
private:
  vbString FileName;  // Name of the configuration file
  vbString parm_ID;   // String used indentify parameters in the file
  char comment_char;   // Char used to ID comments in the config file
  int filter_comments; // If ture all comments will be ignored
};

#endif // __VBCONFIG_HPP__
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //

