// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: vbheader.h 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 02/04/1997
// Date Last Modified: 07/17/2000
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

This include file contains type definitions, constants, and data 
structures needed to create variable block headers. 
*/
// ----------------------------------------------------------- //   
#ifndef __VARIABLE_BLOCK_HEADER_HPP__
#define __VARIABLE_BLOCK_HEADER_HPP__

// --------------------------------------------------------------
// Include files
// --------------------------------------------------------------
#include "vbdtypes.h"
#include "vbint32.h"
#include "vbuint32.h"
#include "vbd_ver.h"
// --------------------------------------------------------------

// --------------------------------------------------------------
// Type definitions 
// --------------------------------------------------------------
// typedefs for file operations and stream position
typedef vbINT32 FAU; // (F)ile (A)ddress (U)nit, physical file address type
typedef vbINT32 vbStreamPos; // Stream position

// CRC-32 checksum used to detect bit errors
typedef vbUINT32 vbChecksum;
// --------------------------------------------------------------

// --------------------------------------------------------------
// Constants
// --------------------------------------------------------------
// Constants for dynamic data attributes used by the first byte of the
// block header Status member.
const __SBYTE__ vbBadBlock     = 'B'; // Bad Variable Block
const __SBYTE__ vbDeletedBlock = 'D'; // Deleted Variable Block 
const __SBYTE__ vbNormalBlock  = 'N'; // Normal Read/Write attribute 
const __SBYTE__ vbRemovedBlock = 'R'; // Removed Variable Block

// This block was received from a remote device. Added in VBD revision C
const __SBYTE__ vbRemoteDeviceBlock = 'V'; 

// Constants for block control characters used by the second byte of the
// block header Status member. Added in VBD revision C to handle device
// control commands.
const __SBYTE__ vbAddRemoteBlock    = 'A'; // Add a remote block
const __SBYTE__ vbChangeRemoteBlock = 'C'; // Change a remote block
const __SBYTE__ vbDeleteRemoteBlock = 'D'; // Delete a remote block
const __SBYTE__ vbRequestFailed     = 'F'; // The client request failed
const __SBYTE__ vbCloseConnection   = 'L'; // Close Client/Server connection
const __SBYTE__ vbKillServer        = 'K'; // Shutdown Client/Server
const __SBYTE__ vbRequestBlock      = 'R'; // Requesting a block
const __SBYTE__ vbSendBlock         = 'S'; // Sending raw data block
const __SBYTE__ vbAcknowledgeBlock  = 'W'; // Acknowledge a data block

// Constants for file operations and stream position 
const int vbMaxNameLength     = 255;        // Max len of file names
const FAU vbStartOfFile       = 0;          // First byte in the file
const FAU vbCurrAddress       = -1;         // Indicates current location
const vbUINT32 vbCheckWord    = 0x0000fefe; // 32-bit Check Word
const vbINT32 vbFSListCorrupt = -1;       // Free space list is corrupt
// --------------------------------------------------------------

// --------------------------------------------------------------
// Data Sturctures
// --------------------------------------------------------------
// Variable Block Database File Header
struct vbFileHeader // VBD file header information
{ 
  vbFileHeader() { 
    FreeSpace = EndOfFile = HeapStart = HighestVB = (FAU)0;
    Version = 0; Signature[0] = 0;
  }
  ~vbFileHeader() { }

  FAU FreeSpace;          // Address to first block of free heap space
  FAU EndOfFile;          // Address of byte after end of file
  FAU HeapStart;          // Address of the start of the heap space
  FAU HighestVB;          // Highest allocated (V)ariable (B)lock
  __SBYTE__ Signature[8]; // Signature used for every VBD file 
  vbINT32 Version;        // VBD Version number
  
private:
  vbFileHeader(const vbFileHeader &ob) { }
  void operator=(const vbFileHeader &ob) { }
};

// Variable Block Header 
struct vbBlockHeader // Marks each variable data block (16 bytes total)
{
  vbBlockHeader() {
    CkWord = vbCheckWord;
    Status = vbNormalBlock;
    Length = (vbUINT32)0;
    NextDeletedVB = (FAU)0;
  }
  ~vbBlockHeader() { }
  vbBlockHeader(const vbBlockHeader &ob) {
    CkWord = ob.CkWord; Length = ob.Length; Status = ob.Status;
    NextDeletedVB = ob.NextDeletedVB;
  }
  vbBlockHeader& operator=(const vbBlockHeader &ob) {
    CkWord = ob.CkWord; Length = ob.Length; Status = ob.Status;
    NextDeletedVB = ob.NextDeletedVB;
    return *this;
  }

  vbUINT32 CkWord;   // Check-word used to mark each block (4 bytes)
  vbUINT32 Length;   // Block Length (4 bytes) 
  vbUINT32 Status;   // First byte  = status of the block's dynamic data 
                     // Second byte = block control commands
                     // Third byte  = reserved for future use
                     // Fourth byte = reserved for future use
  FAU NextDeletedVB; // Pointer to next deleted block (4 bytes) 
};

// 05/31/2000: File lock header added to allow applications to lock the
// entire file during a multi-thread/multi-machine read or write operation.
struct vbFileLockHeader
{
  vbFileLockHeader() { file_lock_predicate = file_lock = 0; }
  ~vbFileLockHeader() { }
  vbFileLockHeader(const vbFileLockHeader &ob) {
    file_lock_predicate = ob.file_lock_predicate;
    file_lock = ob.file_lock;
  }
  vbFileLockHeader& operator=(const vbFileLockHeader &ob) {
    file_lock_predicate = ob.file_lock_predicate;
    file_lock = ob.file_lock;
    return *this;
  }

  // Lock predicates are used to serialize access to the file lock
  // and set the file lock access mode to shared or exclusive. The
  // file lock will allow up to 255 threads per generation to lock the
  // file.
  __UBYTE__ file_lock_predicate; // Serialize access to the file lock
  __UBYTE__ file_lock;           // Member used to lock the entire file
};

// 05/31/2000: Record lock header added to all application to lock a
// specific block during a multi-thread/multi-machine read or write
// operation.
struct vbRecordLockHeader
{
  vbRecordLockHeader() {
    write_lock_predicate = write_lock = read_lock_predicate = read_lock = 0;
  }
  ~vbRecordLockHeader() { }
  vbRecordLockHeader(const vbRecordLockHeader &ob) {
    write_lock_predicate = ob.write_lock_predicate; write_lock = ob.write_lock;
    read_lock_predicate = ob.read_lock_predicate; read_lock = ob.read_lock;
  }
  vbRecordLockHeader& operator=(const vbRecordLockHeader &ob) {
    write_lock_predicate = ob.write_lock_predicate; write_lock = ob.write_lock;
    read_lock_predicate = ob.read_lock_predicate; read_lock = ob.read_lock;
    return *this;
  }

  // Lock predicates are used to serialize access to the read and
  // write locks and set the lock access mode to shared or exclusive.
  // The read and write lock will allow up to 255 thread per generation
  // lock the record during a read or write operation
  __UBYTE__ write_lock_predicate; // Serialize access to the write lock
  __UBYTE__ write_lock;           // Lock record during a write operation
  __UBYTE__ read_lock_predicate;  // Serialize access to the read lock
  __UBYTE__ read_lock;            // Lock record during a read operation
};
// --------------------------------------------------------------

#endif // __VARIABLE_BLOCK_HEADER_HPP__
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
