// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Header File Name: vbshtml.h
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 01/25/2000
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbsHTML class is a base class used to parse html documents. 
*/
// ----------------------------------------------------------- // 
#ifndef __VBSTHML_HPP__
#define __VBSTHML_HPP__

#include <fstream.h>
#include "vbstring.h"
#include "vblist.h"
#include "dfileb.h"
#include "membuf.h"

// Data structure used to store the file position of an html tag,
// the tag itself, its attributes and instructions.
struct vbsHTMLTagInfo
{
  vbsHTMLTagInfo();
  ~vbsHTMLTagInfo();
  vbsHTMLTagInfo(const vbsHTMLTagInfo &ob) { Copy(ob); }
  vbsHTMLTagInfo& operator=(const vbsHTMLTagInfo &ob) { 
    Copy(ob); return *this; 
  }
  void Copy(const vbsHTMLTagInfo &ob);
  friend int operator==(const vbsHTMLTagInfo &a, const vbsHTMLTagInfo &b);

  // File information
  df_StreamPos start_tag; // This tag's starting position in the file
  df_StreamPos end_tag;   // This tag's ending position in the file
  unsigned tag_length;    // The complete length of this tag "< ---- >"
  
  // Tag information
  int tag_id;        // Numerical value used to identify supported tags
  vbString tag_info; // The complete tag from opening to closing bracket
  vbString tag;      // HTML tag
  vbString attr;     // HTML tag attributes 

  // Tag instructions
  int start_instruction; // True if start of tag instruction "<"
  int end_instruction;   // True if end of a tag instruction "/x>"
  int has_attributes;    // True if this tag has attributes associated with it
};

// The total number of tags defined in the HTML tags and modifiers
// enumeration. The number must match the number of strings defined 
// in the vbs_SUPPROTED_TAGS array.  
const int vbsMAX_SUPPORTED_TAGS = 117;

class vbsHTML : public virtual DiskFileB
{
public:
  // The following list of HTML tags is a combination of HTML
  // 2.0, 3.0, 3.2 tags supported by Netscape's Navigator
  // web browser, Microsoft's Internet Explorer web browser, 
  // and standards defined by the World Wide Web Consortium.
  // This list was taken from the Willcam's Comprehensive HTML 
  // Cross Reference at: 
  // http://www.willcam.com/cmat/html/crossref.html 
  enum { // HTML tags and modifiers
    vbs_invalid_tag = 0, // Invalid tag specified
    vbs_unknown_tag,     // Unknown tag specified
    vbs_special_tag,     // Unknown special tags starting with an 
                         // ampersand ending in a semicolon &xxxx;

    // Tags and format specifiers with special meaning
    vbs_comment_tag,     // comment
    vbs_less_then,       // Less than sign "&lt;" 
    vbs_greater_then,    // Greater then sign "&gt;"
    vbs_ampersand,       // Ampersand "&amp;"
    vbs_nb_space,        // Non-breaking space "&nbsp;"
    vbs_quote,           // Quotation mark "&quot;"
    vbs_ex_acsii_set,    // Extended ASCII character set

    // HTML tag codes
    vbs_a_tag,           // anchor
    vbs_abbrev_tag,      // abbreviation
    vbs_acronym_tag,     // acronym
    vbs_address_tag,     // address
    vbs_applet_tag,      // java applet
    vbs_area_tag,        // area
    vbs_au_tag,          // author
    vbs_author_tag,      // author
    vbs_b_tag,           // bold
    vbs_banner_tag,      // banner
    vbs_base_tag,        // base
    vbs_basefont_tag,    // base font
    vbs_bgsound_tag,     // background sound
    vbs_big_tag,         // big text
    vbs_blink_tag,       // blink
    vbs_blockquote_tag,  // block quote
    vbs_bq_tag,          // block quote
    vbs_body_tag,        // body
    vbs_br_tag,          // line break
    vbs_caption_tag,     // caption
    vbs_center_tag,      // center
    vbs_cite_tag,        // citation
    vbs_code_tag,        // code
    vbs_col_tag,         // table column
    vbs_colgroup_tag,    // table column group
    vbs_credit_tag,      // credit
    vbs_del_tag,         // deleted text
    vbs_dfn_tag,         // definition
    vbs_dir_tag,         // directory list
    vbs_div_tag,         // division
    vbs_dl_tag,          // definition list
    vbs_dt_tag,          // definition term
    vbs_dd_tag,          // definition definition
    vbs_em_tag,          // emphasized
    vbs_embed_tag,       // embed
    vbs_fig_tag,         // figure
    vbs_fn_tag,          // footnote
    vbs_font_tag,        // font
    vbs_form_tag,        // form
    vbs_frame_tag,       // frame
    vbs_frameset_tag,    // frame set
    vbs_h1_tag,          // heading 1
    vbs_h2_tag,          // heading 2
    vbs_h3_tag,          // heading 3
    vbs_h4_tag,          // heading 4
    vbs_h5_tag,          // heading 5
    vbs_h6_tag,          // heading 6
    vbs_head_tag,        // head
    vbs_hr_tag,          // horizontal rule
    vbs_html_tag,        // html
    vbs_i_tag,           // italic
    vbs_iframe_tag,      // frame - floating
    vbs_img_tag,         // inline image
    vbs_input_tag,       // form input
    vbs_ins_tag,         // inserted text
    vbs_isindex_tag,     // is index
    vbs_kbd_tag,         // keyboard
    vbs_lang_tag,        // language
    vbs_lh_tag,          // list heading
    vbs_li_tag,          // list item
    vbs_link_tag,        // link
    vbs_listing_tag,     // listing
    vbs_map_tag,         // map
    vbs_marquee_tag,     // marquee
    vbs_math_tag,        // math
    vbs_menu_tag,        // menu list
    vbs_meta_tag,        // meta
    vbs_multicol_tag,    // multi column text
    vbs_nobr_tag,        // no break
    vbs_noframes_tag,    // no frames
    vbs_note_tag,        // note
    vbs_ol_tag,          // ordered list
    vbs_overlay_tag,     // overlay
    vbs_p_tag,           // paragraph
    vbs_param_tag,       // parameters
    vbs_person_tag,      // person
    vbs_plaintext_tag,   // plain text
    vbs_pre_tag,         // preformatted text
    vbs_q_tag,           // quote
    vbs_range_tag,       // range
    vbs_samp_tag,        // sample
    vbs_script_tag,      // script
    vbs_select_tag,      // form select
    vbs_small_tag,       // small text
    vbs_spacer_tag,      // white space
    vbs_spot_tag,        // spot
    vbs_strike_tag,      // strikethrough
    vbs_strong_tag,      // strong
    vbs_sub_tag,         // subscript
    vbs_sup_tag,         // superscript
    vbs_tab_tag,         // horizontal tab
    vbs_table_tag,       // table
    vbs_tbody_tag,       // table body
    vbs_td_tag,          // table data
    vbs_textarea_tag,    // form text area
    vbs_textflow_tag,    // java applet textflow
    vbs_tfoot_tag,       // table footer
    vbs_th_tag,          // table header
    vbs_thead_tag,       // table head
    vbs_title_tag,       // title
    vbs_tr_tag,          // table row
    vbs_tt_tag,          // teletype
    vbs_u_tag,           // underlined
    vbs_ul_tag,          // unordered list
    vbs_var_tag,         // variable
    vbs_wbr_tag,         // word break
    vbs_xmp_tag          // example
  };

public:
  vbsHTML();
  virtual ~vbsHTML();
  vbsHTML(const vbsHTML &ob) { Copy(ob); }
  vbsHTML &operator=(const vbsHTML &ob) { Copy(ob); return *this; }

public:
  int LoadHTMLFile(const char *fname);
  void CloseFile();
  int LoadMemoryBuffer(const MemoryBuffer &membuf);
  unsigned NumTags() { return num_tags; } 
  unsigned NumProcessed() { return num_processed; }
  void ParseHTMLTagInfo(vbsHTMLTagInfo &t);
  vbDLList<vbsHTMLTagInfo> *GetTagList() { return &tag_list; }
  void ClearTagList();
  char *GetTag(int tag_id);
  const char *GetTag(int tag_id) const;
  int GetTagID(const vbString &tag);
  void HandleHTMLTag(int tag_id);
  
public: // Derived class interface used to process tags
  virtual void Handle_INVALID_Tag();         // Invalid tag handler
  virtual void Handle_UNKNOWN_Tag();         // Unknown tag handler
  virtual void Handle_UNKNOWN_SPECIAL_Tag(); // Unknown special tags 
  virtual void Handle_COMMENT_Tag();         // Comment tag handler

  // Most common special meaning tags
  virtual void Handle_LESS_THEN_Tag();    // Less than sign "&lt;" 
  virtual void Handle_GREATER_THEN_Tag(); // Greater then sign "&gt;"
  virtual void Handle_AMPERSAND_Tag();    // Ampersand "&amp;"
  virtual void Handle_NB_SPACE_Tag();     // Non-breaking space "&nbsp;"
  virtual void Handle_QUOTE_Tag();        // Quotation mark "&quot;" 
  virtual void Handle_EX_ASCII_Tag();     // Extended ASCII character set

  // Known HTML tags
  virtual void Handle_A_Tag();            // Anchor tag handler
  virtual void Handle_ABBREV_Tag();       // Abbreviation tag handler
  virtual void Handle_ACRONYM_Tag();      // Acronym tag handler
  virtual void Handle_ADDRESS_Tag();      // Address tag handler
  virtual void Handle_APPLET_Tag();       // Java Applet tag handler
  virtual void Handle_AREA_Tag();         // Area tag handler
  virtual void Handle_AU_Tag();           // Author tag handler
  virtual void Handle_AUTHOR_Tag();       // Author tag handler
  virtual void Handle_B_Tag();            // Bold tag handler
  virtual void Handle_BANNER_Tag();       // Banner tag handler
  virtual void Handle_BASE_Tag();         // Base tag handler
  virtual void Handle_BASEFONT_Tag();     // Base Font tag handler
  virtual void Handle_BGSOUND_Tag();      // Background Sound tag handler
  virtual void Handle_BIG_Tag();          // Big Text tag handler
  virtual void Handle_BLINK_Tag();        // Blink tag handler
  virtual void Handle_BLOCKQUOTE_Tag();   // Block Quote tag handler
  virtual void Handle_BQ_Tag();           // Block Quote tag handler
  virtual void Handle_BODY_Tag();         // Body tag handler
  virtual void Handle_BR_Tag();           // Line Break tag handler
  virtual void Handle_CAPTION_Tag();      // Caption tag handler
  virtual void Handle_CENTER_Tag();       // Center tag handler
  virtual void Handle_CITE_Tag();         // Citation tag handler
  virtual void Handle_CODE_Tag();         // Code tag handler
  virtual void Handle_COL_Tag();          // Table Column tag handler
  virtual void Handle_COLGROUP_Tag();     // Table Column Group tag handler
  virtual void Handle_CREDIT_Tag();       // Credit tag handler
  virtual void Handle_DEL_Tag();          // Deleted Text tag handler
  virtual void Handle_DFN_Tag();          // Definition tag handler
  virtual void Handle_DIR_Tag();          // Directory List tag handler
  virtual void Handle_DIV_Tag();          // Division tag handler
  virtual void Handle_DL_Tag();           // Definition List tag handler
  virtual void Handle_DT_Tag();           // Definition Term tag handler
  virtual void Handle_DD_Tag();           // Definition Definition tag handler
  virtual void Handle_EM_Tag();           // Emphasized tag handler
  virtual void Handle_EMBED_Tag();        // Embed tag handler
  virtual void Handle_FIG_Tag();          // Figure tag handler
  virtual void Handle_FN_Tag();           // Footnote tag handler
  virtual void Handle_FONT_Tag();         // Font tag handler
  virtual void Handle_FORM_Tag();         // Form tag handler
  virtual void Handle_FRAME_Tag();        // Frame tag handler
  virtual void Handle_FRAMESET_Tag();     // Frame Set tag handler
  virtual void Handle_H1_Tag();           // Heading 1 tag handler
  virtual void Handle_H2_Tag();           // Heading 2 tag handler
  virtual void Handle_H3_Tag();           // Heading 3 tag handler
  virtual void Handle_H4_Tag();           // Heading 4 tag handler
  virtual void Handle_H5_Tag();           // Heading 5 tag handler
  virtual void Handle_H6_Tag();           // Heading 6 tag handler
  virtual void Handle_HEAD_Tag();         // Head tag handler
  virtual void Handle_HR_Tag();           // Horizontal Rule tag handler
  virtual void Handle_HTML_Tag();         // HTML tag handler
  virtual void Handle_I_Tag();            // Italic tag handler
  virtual void Handle_IFRAME_Tag();       // Frame - Floating tag handler
  virtual void Handle_IMG_Tag();          // Inline Image tag handler
  virtual void Handle_INPUT_Tag();        // Form Input tag handler
  virtual void Handle_INS_Tag();          // Inserted Text tag handler
  virtual void Handle_ISINDEX_Tag();      // Is Index tag handler
  virtual void Handle_KBD_Tag();          // Keyboard tag handler
  virtual void Handle_LANG_Tag();         // Language tag handler
  virtual void Handle_LH_Tag();           // List Heading tag handler
  virtual void Handle_LI_Tag();           // List Item tag handler
  virtual void Handle_LINK_Tag();         // Link tag handler
  virtual void Handle_LISTING_Tag();      // Listing tag handler
  virtual void Handle_MAP_Tag();          // Map tag handler
  virtual void Handle_MARQUEE_Tag();      // Marquee tag handler
  virtual void Handle_MATH_Tag();         // Math tag handler
  virtual void Handle_MENU_Tag();         // Menu List tag handler
  virtual void Handle_META_Tag();         // Meta tag handler
  virtual void Handle_MULTICOL_Tag();     // Multi Column Text tag handler
  virtual void Handle_NOBR_Tag();         // No Break tag handler
  virtual void Handle_NOFRAMES_Tag();     // No Frames tag handler
  virtual void Handle_NOTE_Tag();         // Note tag handler
  virtual void Handle_OL_Tag();           // Ordered List tag handler
  virtual void Handle_OVERLAY_Tag();      // Overlay tag handler
  virtual void Handle_P_Tag();            // Paragraph tag handler
  virtual void Handle_PARAM_Tag();        // Parameters tag handler
  virtual void Handle_PERSON_Tag();       // Person tag handler
  virtual void Handle_PLAINTEXT_Tag();    // Plain Text tag handler
  virtual void Handle_PRE_Tag();          // Preformatted Text tag handler
  virtual void Handle_Q_Tag();            // Quote tag handler
  virtual void Handle_RANGE_Tag();        // Range tag handler
  virtual void Handle_SAMP_Tag();         // Sample tag handler
  virtual void Handle_SCRIPT_Tag();       // Script tag handler
  virtual void Handle_SELECT_Tag();       // Form Select tag handler
  virtual void Handle_SMALL_Tag();        // Small Text tag handler
  virtual void Handle_SPACER_Tag();       // White Space tag handler
  virtual void Handle_SPOT_Tag();         // Spot tag handler
  virtual void Handle_STRIKE_Tag();       // Strikethrough tag handler
  virtual void Handle_STRONG_Tag();       // Strong tag handler
  virtual void Handle_SUB_Tag();          // Subscript tag handler
  virtual void Handle_SUP_Tag();          // Superscript tag handler
  virtual void Handle_TAB_Tag();          // Horizontal Tab tag handler
  virtual void Handle_TABLE_Tag();        // Table tag handler
  virtual void Handle_TBODY_Tag();        // Table Body tag handler
  virtual void Handle_TD_Tag();           // Table Data tag handler
  virtual void Handle_TEXTAREA_Tag();     // Form Text Area tag handler
  virtual void Handle_TEXTFLOW_Tag();     // Java Applet Textflow tag handler
  virtual void Handle_TFOOT_Tag();        // Table Footer tag handler
  virtual void Handle_TH_Tag();           // Table Header tag handler
  virtual void Handle_THEAD_Tag();        // Table Head tag handler
  virtual void Handle_TITLE_Tag();        // Title tag handler
  virtual void Handle_TR_Tag();           // Table Row tag handler
  virtual void Handle_TT_Tag();           // Teletype tag handler
  virtual void Handle_U_Tag();            // Underlined tag handler
  virtual void Handle_UL_Tag();           // Unordered List tag handler
  virtual void Handle_VAR_Tag();          // Variable tag handler
  virtual void Handle_WBR_Tag();          // Word Break tag handler
  virtual void Handle_XMP_Tag();          // Example tag handler
  
protected: // Internal processing functions
  void Copy(const vbsHTML &ob);
  int CollectHTMLTags();
  int ProcessHTMLTags();
  int CollectHTMLTags(const MemoryBuffer &membuf);
  int ProcessHTMLTags(const MemoryBuffer &membuf);
  
protected:
  vbDLList<vbsHTMLTagInfo> tag_list; // List of HTML tags found in a file
  vbDNode<vbsHTMLTagInfo> *list_ptr; // Current list node
  unsigned num_tags;      // Number of tags collected from an html file
  unsigned num_processed; // Number of valid tags read from the file
};

#endif // __VBSTHML_HPP__
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //

