// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Header File Name: vbshttpc.h
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 01/25/2000
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

HTTP client class.
*/
// ----------------------------------------------------------- // 
#ifndef __VBSOCKET_HTTP_CLIENT__HPP__
#define __VBSOCKET_HTTP_CLIENT__HPP__

#include <fstream.h>
#include "devcache.h"
#include "vbsocket.h"
#include "vbsurl.h"
#include "vbshttp.h"

// --------------------------------------------------------------
// Type definitions
// --------------------------------------------------------------
typedef MemoryBuffer MEMTYPE; // Variable length memory buffers
typedef vbDeviceCachePtr httpCachePointer; // Pointer to cache
// --------------------------------------------------------------

// --------------------------------------------------------------
// Constants 
// --------------------------------------------------------------
const int __HTTP_DEFAULT_CACHE_SIZE__  = 1024;  // Default cache size  
const int __HTTP_DEFAULT_PACKET_SIZE__ = 1024;  // Maximum length of each block

// Adjust these values for maximum performance
const int __HTTP_CACHE_SIZE__  = 1024; // Number of cache buckets used
const int __HTTP_PACKET_SIZE__ = 32767; // Maximum packet size
// --------------------------------------------------------------

class vbsHTTPClient : public vbSocket, public vbDeviceCache
{
public:
  vbsHTTPClient(int CacheSize = __HTTP_DEFAULT_CACHE_SIZE__);
  ~vbsHTTPClient();

public:
  void Flush() { cache.Flush(); } // Flush the cache buckets
  unsigned BucketsInUse() { return cache.BucketsInUse(); }
  int ConnectClient(const char *host, int port = vbSOCKET_HTTP_PORT);
  int RequestHeader(const vbsURLInfo &u, vbsHTTPHeader &hdr);
  int SimpleRequest(const vbsURLInfo &u, fstream *stream);
  int SimpleRequest(const vbsURLInfo &u, MemoryBuffer &mbuf);
  int RequestFile(const vbsURLInfo &u, const vbsHTTPHeader &hdr,
		  fstream *stream);
  int RequestFile(const vbsURLInfo &u, const vbsHTTPHeader &hdr,
		  MemoryBuffer &mbuf);
  char *GetMethod(int method);
  char *GetProtocolString(int protocol);
  void BuildRequestString(const vbsURLInfo &u, int method, vbString &request);
  void BuildSimpleRequestString(const vbsURLInfo &u, int method,
				vbString &request);
  int ParseHTTPHeader(const vbsURLInfo &u, const vbString &header,
		      vbsHTTPHeader &hdr);

private: // Device cache base class interface
  void Read(void *buf, unsigned Bytes, vbDeviceTypes dev);
  void Write(const void *buf, unsigned Bytes, vbDeviceTypes dev);

public: // Client header information that can or must be modified
        // according to the application. These values will be used
        // when a request string is built.
  vbString HTTP_VERSION_STRING; // HTTP requested version string
  vbString HTTP_AGENT_STRING;   // Application name;
  vbString HTTP_ACCEPT_STRING;  // File types that will be accepted
  vbString HTTP_PRAGMA_STRING;  // Implementation-specific directives 

  // Used to provide information left earlier by some server.
  // Usage: "Cookie: persistent data\r\n"
  vbString HTTP_COOKIE_STRING; 
  
  // Default file name used if no file is specified. NOTE: defaults to " /"
  vbString HTTP_DEFAULT_FILE_REQUEST_STRING;

  // This value will be set to null by default. The application must set
  // this value in order to include it in the request string. NOTE: The
  // following syntax must be used when setting this value:
  // "Referer: http://www.w3.org/hypertext/DataSources/Overview.html\r\n"
  vbString HTTP_REFERER_STRING; // The Referer request-header field allows
                                // the client to specify, for the server's
                                // benefit, the address (URI) of the resource
                                // from which the Request-URI was obtained. 
  
  // This value will be set to null by default. The application must set
  // this value in order to include it in the request string. NOTE: The
  // following syntax must be used when setting this value:
  // "If-Modified-Since: Sat, 29 Oct 1999 19:43:31 GMT\r\n"
  // There are three accepted date formats: RFC 822, RFC 850, asctime()
  vbString HTTP_IF_MOD_STRING; // Used with the GET method to make it
                               // conditional: if the requested resource has
                               // not been modified since the time specified
                               // in this field, a copy of the resource will
                               // not be returned from the server; instead, a
                               // 304 (not modified) response will be returned
                               // without any Entity-Body. 
  
private:
  vbsURL url;              // Used to process URL infomation
  int write_error;         // Holds the last write error, if any
  int read_error;          // Holds the last read error, if any
  unsigned bytes_received; // Number of bytes read from an open socket

private: // I/O device members
  fstream *file_ptr;                   // Pointer to an open file
  char rxBuffer[__HTTP_PACKET_SIZE__]; // Receiver memory buffer

private: // Device cache
  vbDeviceBucketCache cache;

public: // Functions used to get the current device cache
  vbDeviceBucketCache *GetCache() { return &cache; }
};

#endif // __VBSOCKET_HTTP_CLIENT__HPP__
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
