// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: vbstack.h 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 01/21/1997 
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

A generic singly linked list based stack. This is a last-in,
first-out (LIFO) resizable data structure. Heap space is
allocated and de-allocated with every push and pop operation.
*/
// ----------------------------------------------------------- //   
#ifndef __VBSTACK_HPP__
#define __VBSTACK_HPP__

#include "vblist.h"

template<class TYPE>
class vbStack : public vbDLListBase
{
public:
  vbStack() { }
  ~vbStack();
  vbStack(const vbStack<TYPE> &X) { Copy(X); }
  void operator=(const vbStack<TYPE> &X) { Copy(X); }

public:
  int Push(const TYPE &X);
  int Pop(TYPE &X);
  int Pop();
  void Rewind(unsigned Index=0);
  TYPE *Top() { return IsEmpty() ? 0 : &(((vbDNode<TYPE> *)GetFront())->Data); }
  const TYPE *Top() const { return ((vbStack<TYPE> *)this)->Top(); }
  TYPE *Look(unsigned Index);
  const TYPE *Look(unsigned Index) const {
    return ((vbStack<TYPE> *)this)->Look(Index); }

protected:
  virtual vbDNode<TYPE> *AllocNode(const TYPE &X);
  virtual vbDNodeBase *DupNode(const vbDNodeBase *Node);
  virtual void FreeNode(vbDNodeBase *Node);
};

template<class TYPE>
vbStack<TYPE>::~vbStack()
{
  Rewind();
}

template<class TYPE>
vbDNode<TYPE> *vbStack<TYPE>::AllocNode(const TYPE &X)
{
  return new vbDNode<TYPE>(X);
}

template<class TYPE>
vbDNodeBase *vbStack<TYPE>::DupNode(const vbDNodeBase *Node)
{
  return new vbDNode<TYPE>(((vbDNode<TYPE> *)Node)->Data);
}

template<class TYPE>
void vbStack<TYPE>::FreeNode(vbDNodeBase *Node)
{
  delete (vbDNode<TYPE> *)Node;
}

template<class TYPE>
int vbStack<TYPE>::Push(const TYPE &X)
{
  vbDNode<TYPE> *ptr = AllocNode(X);
  if (ptr == 0) return 0; // Allocation failed
  AttachToFront(ptr);
  return 1;
}

template<class TYPE>
int vbStack<TYPE>::Pop(TYPE &X)
{
  vbDNode<TYPE> *ptr = (vbDNode<TYPE> *)RmvFront();
  if (ptr == 0) return 0; // Stack is empty
  X = ptr->Data;
  FreeNode(ptr);
  return 1;
}

template<class TYPE>
int vbStack<TYPE>::Pop()
{
  if (IsEmpty()) return 0; // Stack is empty
  FreeNode(RmvFront());
  return 1;
}

template<class TYPE>
TYPE *vbStack<TYPE>::Look(unsigned Index)
{
  vbDNode<TYPE> *ptr = (vbDNode<TYPE> *)GetFront();
  while(!IsHeader(ptr)) {
    if (Index-- == 0) return &ptr->Data; // Return data in the top elememnt
    ptr = ptr->GetNext();
  }
  return 0; // Out of range
}

template<class TYPE>
void vbStack<TYPE>::Rewind(unsigned Index)
{
  if (Index == 0) Index = (unsigned)(-1); // Biggest number that can be used
  vbDNode<TYPE> *ptr = (vbDNode<TYPE> *)Next;
  while(!IsHeader(ptr) && Index--) {
    vbDNode<TYPE> *NextPtr = ptr->GetNext();
    FreeNode(ptr);
    ptr = NextPtr;
    Next = NextPtr;
  }
}

#endif // __VBSTACK_HPP__
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
