// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Header File Name: vbsurl.h
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 01/25/2000
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbsURL class is used to parse uniform resource locators.
*/
// ----------------------------------------------------------- // 
#ifndef __VBSURL_HPP__
#define __VBSURL_HPP__

#include "vbstring.h"

// Structure containing info on a URL. 
struct vbsURLInfo
{
  vbsURLInfo();
  ~vbsURLInfo();
  vbsURLInfo(const vbsURLInfo &ob) { Copy(ob); }
  vbsURLInfo& operator=(const vbsURLInfo &ob) { Copy(ob); return *this; }
  void Copy(const vbsURLInfo &ob);

  // Overloaded operators
  friend int operator==(const vbsURLInfo &a, const vbsURLInfo &b);
  friend int operator!=(const vbsURLInfo &a, const vbsURLInfo &b);

  // URL information
  vbString url;	            // Unchanged URL 
  vbString proto;	    // URL protocol 
  vbString host;            // Extracted hostname 
  vbString path, dir, file; // Path, as well as dir and file 
  vbString user, passwd;    // Username and password 
  vbsURLInfo *proxy;        // The exact string to pass to proxy server 
  vbString local;           // The local filename of the URL document
  vbString referer;	    // Source that requested URI was obtained
  int port;                 // Port number
  int proto_type;           // Enumerated value representing a protocol type 
  char ftp_type;            // FTP type

  // Members used by HTTP clients
  vbString parent_directory; // This resource's parent directory
  vbString local_file;       // This resource's parent directory
};

// Maximum number of reconized URL protocols.
// Adjust this value to match the number of
// protocols defined in the vbsPROTOCOL_STRINGS
// arrary and enumeration.
const int vbsMAX_PROTOCOL_STRINGS = 56;

class vbsURL
{
public:
  // This following list of URL protocols is a combination
  // standard and non-standard URI schemes taken from:
  // http://www.w3.org/pub/WWW/Addressing/schemes.html
  enum { // Recognized URL protocols
    vbs_Unknown_URL_protocol = 0, // The URL protocol specified is not known
    vbs_about,       // Client-Side JavaScript Reference
    vbs_acap,        // ACAP -- Application Configuration Access Protocol
    vbs_afp,         // Definition of afp: URLs for use with Service Location
    vbs_afs,         // Reserved, per Internet Standard list of url schemes 
    vbs_callto,      // NetMeeting Hyperlink on a Web Page
    vbs_chttp,       // Caching Protocol in Authoring Content for RealPlayer 
    vbs_cid,         // Content-ID and Message-ID Uniform Resource Locators 
    vbs_clsid,       // Identifies OLE/COM classes. related to OBJECT spec 
    vbs_data,        // Data: URL scheme
    vbs_file,        // Host-specific file names URL RFC  
    vbs_finger,      // Finger protocol URL
    vbs_ftp,         // File Transfer protocol URL
    vbs_gopher,      // Gopher protocol URL
    vbs_hdl,         // CNRI handle system 
    vbs_http,        // Hypertext Transfer Protocol URL 
    vbs_https,       // HTTP over SSL (Secure Socket Layer)
    vbs_iioploc,     // Interoperable Naming Joint Revised Submission
    vbs_ilu,         // ILU types, string binding handles 
    vbs_imap,        // IMAP URL scheme 
    vbs_ior,         // CORBA interoperable object reference 
    vbs_java,        // Identifies java classes 
    vbs_javascript,  // Client-Side JavaScript Reference
    vbs_jdbc,        // Used in Java SQL API 
    vbs_ldap,        // An LDAP URL Format
    vbs_lifn,        // BFD -- Bulk File distribution
    vbs_mailto,      // Electronic mail address
    vbs_mid,         // Content-ID and Message-ID Uniform Resource Locators
    vbs_news,        // USENET news
    vbs_nfs,         // NFS URL Scheme
    vbs_nntp,        // USENET news using NNTP access URL
    vbs_path,        // Path spec 
    vbs_pop,         // POP URL Scheme
    vbs_pop3,        // A POP3 URL Interface
    vbs_printer,     // Definition of printer: URLs with Service Location
    vbs_prospero,    // Prospero Directory Service URL
    vbs_res,         // Res Protocol
    vbs_rtsp,        // Real Time Streaming Protocol (RTSP)
    vbs_rvp,         // Rendezvous Protocol
    vbs_rlogin,      // Remote login
    vbs_rwhois,      // The RWhois Uniform Resource Locator
    vbs_rx,          // Remote Execution, per Mobile GUI On The Web
    vbs_sdp,         // SDP URL Scheme
    vbs_service,     // Service Templates and service
    vbs_sip,         // SIP URL Scheme
    vbs_shttp,       // Secure http
    vbs_snews,       // NNTP over SSL
    vbs_stanf,       // Stable Network Filenames 
    vbs_telnet,      // Reference to interactive sessions URL RFC 
    vbs_tip,         // Transaction Internet Protocol Version 3.0
    vbs_tn3270,      // Reserved, per Internet Standard list of url schemes 
    vbs_tv,          // Uniform Resource Locators for Television Broadcasts
    vbs_uuid,        // The UUID addressing scheme
    vbs_wais,        // Wide Area Information Servers URL 
    vbs_whois,       // Distributed directory service
    vbs_whodp        // WhoDP: Widely Hosted Object Data Protocol
  };

public:
  vbsURL() { }
  ~vbsURL() { }
  
public: // Functions used to identify URL protocols and port numbers
  char *GetProtocolString(int protocol);
  const char *GetProtocolString(int protocol) const;
  int GetProtocolType(const vbString &protocol);
  int GetPortNumber(const vbString &url, int &port);
  
public: // Functions used to format and test URL strings
  void ProcessFTPType(const vbString &url, char &ftp_type);
  int CleanUserName(const vbString &url, vbString &clean_url);
  int HasProtocol(const vbString &url);
  int HasFile(const vbString &path, vbString &dir, vbString &file);
  int HasFile(const vbString &path);
  
public: // Functions used to parse uniform resource locators
  int ParseURL(const vbString &url, vbsURLInfo &u, int strict = 0);
  int ParseURL(const char *url, vbsURLInfo &u, int strict = 0); 
  int ParseDirectory(vbsURLInfo &u);
  int ParseProtocol(const vbString &url, vbString &proto_name, 
		    int &proto_type);
  int ParseUserName(const vbString &url, vbString &user, vbString &passwd,
		    vbString &clean_url);
  int ParsePortNumber(const vbString &url, int &port);
  int ParseHostName(const vbString &url, vbString &host,
		    int remove_port_number = 1);
  int ParseDirectory(const vbString &url, vbString &path, vbString &dir,
		     vbString &file);
};

#endif // __VBSURL_HPP__
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
