// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: vbuint32.h 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/05/1997  
// Date Last Modified: 07/17/2000
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbUINT32 class is used to represent 32 bit unsigned integers
independently of the operating system or hardware platform used.
It works by separating a 32-bit value into four separate byte
values and reordering the bytes lowest-order to highest-order.
An vbUINT32 type has a base 10 limit of 4,294,967,295.

Changes:
================================================================
04/21/1999: Added Watcom 10 patch 1031 #1. This patch provides
type casting from an int type to a vbINT32, vbUINT32, vbINT16,
vbUINT16 or vbFLOAT64 type as required by the Watcom compiler.
Added by: Heiko Weber

10/11/1999: Moved the corresponding include files above the
__USE_NATIVE_INT_TYPES__ directive to make the compile time
directives visible.
Changed by: Oleg Smolsky

03/14/2000: All divide by zero errors will no longer use the
Ehandler class to throw an exception. This change was implemented
to allow the vbUINT32 class to be compiled independently of the
Ehandler class in applications that do not require the use of
the Ehandler class. A divide by zero exception will return to
the caller without performing any operations that would other-
wise cause the program to crash.
changed by: Doug Gaer

05/31/2000: Changed the name of the UINT32 class to vbUINT32
to elminate the name conflicts with the UINT32 type defined
in BCC32 5.5 and MFC.
Changed by: Advanced Open SQL Project

06/04/2000: Added a type cast to the correct type for all member
functions that accept types other then vbUINT32 and __ULWORD__ types.
This will elminate the MSVC and BCC32 compiler warnings as integer
and floating point types are promoted and demoted to vbUINT32 and
__ULWORD__ types.
Added by: Advanced Open SQL Project

06/04/2000: Added an __USBYTE__ type cast to the vbUINT32::UnPackBits()
function to elminate BCC32 5.5 compiler warnings.
Added by: Advanced Open SQL Project
*/
// ----------------------------------------------------------- //   
#ifndef __vbUINT32_HPP
#define __vbUINT32_HPP

#include "vbdtypes.h"

#if defined (__USE_NATIVE_INT_TYPES__)
typedef unsigned long vbUINT32;
#else
// Data structure for unsigned 32 bit integer values.
class vbUINT32
{
public:
  vbUINT32(__ULWORD__ val = 0);
  vbUINT32(const vbUINT32& ob);
  vbUINT32& operator=(const vbUINT32& ob);
  vbUINT32& operator=(const __ULWORD__ ob);

public:
  void UnPackBits(__ULWORD__ val);
  __ULWORD__ PackBits() const;

public:
  operator __ULWORD__() const;
  
public: // Arithmetic operators that modify their operand
  vbUINT32 operator++(int);  // Postfix
  vbUINT32 operator--(int);  // Postfix
  vbUINT32 &operator++() { operator=(*this + 1); return *this; } // Prefix
  vbUINT32 &operator--() { operator=(*this - 1); return *this; } // Prefix
  void operator+=(const vbUINT32 &i) { operator=(*this + i); }
  void operator-=(const vbUINT32 &i) { operator=(*this - i); }
  void operator*=(const vbUINT32 &i) { operator=(*this * i); }
  void operator/=(const vbUINT32 &i);

  void operator+=(const __LWORD__ &i) { operator=(*this + (__ULWORD__)i); }
  void operator-=(const __LWORD__ &i) { operator=(*this - (__ULWORD__)i); }
  void operator*=(const __LWORD__ &i) { operator=(*this * (__ULWORD__)i); }
  void operator/=(const __LWORD__ &i);

  void operator+=(const __ULWORD__ &i) { operator=(*this + (__ULWORD__)i); }
  void operator-=(const __ULWORD__ &i) { operator=(*this - (__ULWORD__)i); }
  void operator*=(const __ULWORD__ &i) { operator=(*this * (__ULWORD__)i); }
  void operator/=(const __ULWORD__ &i);

  void operator+=(const __WORD__ &i) { operator=(*this + (__ULWORD__)i); }
  void operator-=(const __WORD__ &i) { operator=(*this - (__ULWORD__)i); }
  void operator*=(const __WORD__ &i) { operator=(*this * (__ULWORD__)i); }
  void operator/=(const __WORD__ &i);

  void operator+=(const __SWORD__ &i) { operator=(*this + (__ULWORD__)i); }
  void operator-=(const __SWORD__ &i) { operator=(*this - (__ULWORD__)i); }
  void operator*=(const __SWORD__ &i) { operator=(*this * (__ULWORD__)i); }
  void operator/=(const __SWORD__ &i);

  void operator+=(const __UWORD__ &i) { operator=(*this + (__ULWORD__)i); }
  void operator-=(const __UWORD__ &i) { operator=(*this - (__ULWORD__)i); }
  void operator*=(const __UWORD__ &i) { operator=(*this * (__ULWORD__)i); }
  void operator/=(const __UWORD__ &i);

  void operator+=(const __USWORD__ &i) { operator=(*this + (__ULWORD__)i); }
  void operator-=(const __USWORD__ &i) { operator=(*this - (__ULWORD__)i); }
  void operator*=(const __USWORD__ &i) { operator=(*this * (__ULWORD__)i); }
  void operator/=(const __USWORD__ &i);

  void operator+=(const __SBYTE__ &i) { operator=(*this + (__ULWORD__)i); }
  void operator-=(const __SBYTE__ &i) { operator=(*this - (__ULWORD__)i); }
  void operator*=(const __SBYTE__ &i) { operator=(*this * (__ULWORD__)i); }
  void operator/=(const __SBYTE__ &i);

  void operator+=(const __UBYTE__ &i) { operator=(*this + (__ULWORD__)i); }
  void operator-=(const __UBYTE__ &i) { operator=(*this - (__ULWORD__)i); }
  void operator*=(const __UBYTE__ &i) { operator=(*this * (__ULWORD__)i); }
  void operator/=(const __UBYTE__ &i);

public: // Comparison operators
  friend int operator==(const vbUINT32 &a, const vbUINT32 &b);
  friend int operator==(const vbUINT32 &a, const __LWORD__ &bl);
  friend int operator==(const __LWORD__ &al, const vbUINT32 &b);
  friend int operator==(const vbUINT32 &a, const __ULWORD__ &bl);
  friend int operator==(const __ULWORD__ &al, const vbUINT32 &b);
  friend int operator==(const vbUINT32 &a, const __WORD__ &bl);
  friend int operator==(const __WORD__ &al, const vbUINT32 &b);
  friend int operator==(const vbUINT32 &a, const __SWORD__ &bl);
  friend int operator==(const __SWORD__ &al, const vbUINT32 &b);
  friend int operator==(const vbUINT32 &a, const __UWORD__ &bl);
  friend int operator==(const __UWORD__ &al, const vbUINT32 &b);
  friend int operator==(const vbUINT32 &a, const __USWORD__ &bl);
  friend int operator==(const __USWORD__ &al, const vbUINT32 &b);
  friend int operator==(const vbUINT32 &a, const __SBYTE__ &bl);
  friend int operator==(const __SBYTE__ &al, const vbUINT32 &b);
  friend int operator==(const vbUINT32 &a, const __UBYTE__ &bl);
  friend int operator==(const __UBYTE__ &al, const vbUINT32 &b);

  friend int operator!=(const vbUINT32 &a, const vbUINT32 &b);
  friend int operator!=(const vbUINT32 &a, const __LWORD__ &bl);
  friend int operator!=(const __LWORD__ &al, const vbUINT32 &b);
  friend int operator!=(const vbUINT32 &a, const __ULWORD__ &bl);
  friend int operator!=(const __ULWORD__ &al, const vbUINT32 &b);
  friend int operator!=(const vbUINT32 &a, const __WORD__ &bl);
  friend int operator!=(const __WORD__ &al, const vbUINT32 &b);
  friend int operator!=(const vbUINT32 &a, const __SWORD__ &bl);
  friend int operator!=(const __SWORD__ &al, const vbUINT32 &b);
  friend int operator!=(const vbUINT32 &a, const __UWORD__ &bl);
  friend int operator!=(const __UWORD__ &al, const vbUINT32 &b);
  friend int operator!=(const vbUINT32 &a, const __USWORD__ &bl);
  friend int operator!=(const __USWORD__ &al, const vbUINT32 &b);
  friend int operator!=(const vbUINT32 &a, const __SBYTE__ &bl);
  friend int operator!=(const __SBYTE__ &al, const vbUINT32 &b);
  friend int operator!=(const vbUINT32 &a, const __UBYTE__ &bl);
  friend int operator!=(const __UBYTE__ &al, const vbUINT32 &b);

  friend int operator<(const vbUINT32 &a, const vbUINT32 &b);
  friend int operator<(const vbUINT32 &a, const __LWORD__ &bl);
  friend int operator<(const __LWORD__ &al, const vbUINT32 &b);
  friend int operator<(const vbUINT32 &a, const __ULWORD__ &bl);
  friend int operator<(const __ULWORD__ &al, const vbUINT32 &b);
  friend int operator<(const vbUINT32 &a, const __WORD__ &bl);
  friend int operator<(const __WORD__ &al, const vbUINT32 &b);
  friend int operator<(const vbUINT32 &a, const __SWORD__ &bl);
  friend int operator<(const __SWORD__ &al, const vbUINT32 &b);
  friend int operator<(const vbUINT32 &a, const __UWORD__ &bl);
  friend int operator<(const __UWORD__ &al, const vbUINT32 &b);
  friend int operator<(const vbUINT32 &a, const __USWORD__ &bl);
  friend int operator<(const __USWORD__ &al, const vbUINT32 &b);
  friend int operator<(const vbUINT32 &a, const __SBYTE__ &bl);
  friend int operator<(const __SBYTE__ &al, const vbUINT32 &b);
  friend int operator<(const vbUINT32 &a, const __UBYTE__ &bl);
  friend int operator<(const __UBYTE__ &al, const vbUINT32 &b);

  friend int operator>(const vbUINT32 &a, const vbUINT32 &b);
  friend int operator>(const vbUINT32 &a, const __LWORD__ &bl);
  friend int operator>(const __LWORD__ &al, const vbUINT32 &b);
  friend int operator>(const vbUINT32 &a, const __ULWORD__ &bl);
  friend int operator>(const __ULWORD__ &al, const vbUINT32 &b);
  friend int operator>(const vbUINT32 &a, const __WORD__ &bl);
  friend int operator>(const __WORD__ &al, const vbUINT32 &b);
  friend int operator>(const vbUINT32 &a, const __SWORD__ &bl);
  friend int operator>(const __SWORD__ &al, const vbUINT32 &b);
  friend int operator>(const vbUINT32 &a, const __UWORD__ &bl);
  friend int operator>(const __UWORD__ &al, const vbUINT32 &b);
  friend int operator>(const vbUINT32 &a, const __USWORD__ &bl);
  friend int operator>(const __USWORD__ &al, const vbUINT32 &b);
  friend int operator>(const vbUINT32 &a, const __SBYTE__ &bl);
  friend int operator>(const __SBYTE__ &al, const vbUINT32 &b);
  friend int operator>(const vbUINT32 &a, const __UBYTE__ &bl);
  friend int operator>(const __UBYTE__ &al, const vbUINT32 &b);

  friend int operator<=(const vbUINT32 &a, const vbUINT32 &b);
  friend int operator<=(const vbUINT32 &a, const __LWORD__ &bl);
  friend int operator<=(const __LWORD__ &al, const vbUINT32 &b);
  friend int operator<=(const vbUINT32 &a, const __ULWORD__ &bl);
  friend int operator<=(const __ULWORD__ &al, const vbUINT32 &b);
  friend int operator<=(const vbUINT32 &a, const __WORD__ &bl);
  friend int operator<=(const __WORD__ &al, const vbUINT32 &b);
  friend int operator<=(const vbUINT32 &a, const __SWORD__ &bl);
  friend int operator<=(const __SWORD__ &al, const vbUINT32 &b);
  friend int operator<=(const vbUINT32 &a, const __UWORD__ &bl);
  friend int operator<=(const __UWORD__ &al, const vbUINT32 &b);
  friend int operator<=(const vbUINT32 &a, const __USWORD__ &bl);
  friend int operator<=(const __USWORD__ &al, const vbUINT32 &b);
  friend int operator<=(const vbUINT32 &a, const __SBYTE__ &bl);
  friend int operator<=(const __SBYTE__ &al, const vbUINT32 &b);
  friend int operator<=(const vbUINT32 &a, const __UBYTE__ &bl);
  friend int operator<=(const __UBYTE__ &al, const vbUINT32 &b);

  friend int operator>=(const vbUINT32 &a, const vbUINT32 &b);
  friend int operator>=(const vbUINT32 &a, const __LWORD__ &bl);
  friend int operator>=(const __LWORD__ &al, const vbUINT32 &b);
  friend int operator>=(const vbUINT32 &a, const __ULWORD__ &bl);
  friend int operator>=(const __ULWORD__ &al, const vbUINT32 &b);
  friend int operator>=(const vbUINT32 &a, const __WORD__ &bl);
  friend int operator>=(const __WORD__ &al, const vbUINT32 &b);
  friend int operator>=(const vbUINT32 &a, const __SWORD__ &bl);
  friend int operator>=(const __SWORD__ &al, const vbUINT32 &b);
  friend int operator>=(const vbUINT32 &a, const __UWORD__ &bl);
  friend int operator>=(const __UWORD__ &al, const vbUINT32 &b);
  friend int operator>=(const vbUINT32 &a, const __USWORD__ &bl);
  friend int operator>=(const __USWORD__ &al, const vbUINT32 &b);
  friend int operator>=(const vbUINT32 &a, const __SBYTE__ &bl);
  friend int operator>=(const __SBYTE__ &al, const vbUINT32 &b);
  friend int operator>=(const vbUINT32 &a, const __UBYTE__ &bl);
  friend int operator>=(const __UBYTE__ &al, const vbUINT32 &b);
  
private:
  __UBYTE__ byte[4];
};
#endif // __USE_NATIVE_INT_TYPES__

#endif // __vbUINT32_HPP
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
