// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: devcache.cpp 
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer  
// File Creation Date: 09/20/1999
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The DeviceCache class is an abstract base class used to define
the methods needed to interface a device with memory cache.
The derived class determines the type of device the cache will
be linked to: a file, TCP/IP socket, serial port, etc. The derived
class is also responsible for defining the methods used to read
and write to and from the device as well as keep track of the
device status. The device cache will buffer a stream of binary
data as is it is written to or read from the device the derived
class is using.
*/
// ----------------------------------------------------------- //   
#include "devcache.h"

vbDeviceCache::~vbDeviceCache()
{
  // Destructor provided for virtuality
}

vbDeviceBucketCacheb::vbDeviceBucketCacheb(vbDeviceBucketb *b, int n,
					   unsigned bkt_size)
// Assumes n buckets of size bkt_size have already been
// allocated. Sets up their Prior and Next pointers to form
// a circular list.
: devptr(0)
{
  vbDeviceBucketb *p = b;
  for(int i = 0; i < n; i++) {
      p->MakeNull();
      p->Prior = (vbDeviceBucketb *)((char *)p - bkt_size);
      p->Next = (vbDeviceBucketb *)((char *)p + bkt_size);
      p = p->Next;
  }

  // Make the list circular
  tail = (vbDeviceBucketb *)((char *)b + (n-1)*bkt_size);
  tail->Next = b;
  b->Prior = tail;

  // Set up other cache variables
  nbuckets = n; head = b; 
}

vbDeviceBucketCacheb::~vbDeviceBucketCacheb()
{
  // Destructor provided for virtuality
}

void vbDeviceBucketCacheb::MoveToFront(vbDeviceBucketb *b)
// Logically move a bucket to front of list (if needed),
// so that it is treated as the most recently reserved
// bucket. NOTE: A call to the this function will place
// the cache buckets out of sequence.
{
  if(b == head->Prior) { // b is tail
     head = head->Prior;
  }
  else if(b != head) {
    // Unhook bucket from where it is
    b->Prior->Next = b->Next;
    b->Next->Prior = b->Prior;

    // Put it before head
    b->Next = head;
    b->Prior = head->Prior;
    b->Prior->Next = b;
    head->Prior = b;
    head = b;
  }
}

int vbDeviceBucketCacheb::Flush(int empty_bkts)
// Flushes all buckets in the cache. Makes them empty if
// empty_bkts is true. Will return a non-zero value if a
// bucket is being referenced by more then one cache
// pointer. Returns zero to indicate a successful flush
// operation.
{
  // Flush, starting at the tail so that cache will be flushed
  // in a first in, first out sequence (FIFO)
  vbDeviceBucketb *b = head->Prior; 

  while(1) {
    if(empty_bkts && b->IsLocked()) {
      // The bucket is being referenced by a cache pointer
      return 1;
    }

    if(b->IsDirty()) { // Flush if bucket is in use
      if(devptr->ReadyForWriting()) b->Flush(*devptr);
    }
    
    if(empty_bkts) b->MakeNull();    
    
    b = b->Prior;
    if(b == head->Prior) break; // Reached the end of the list
  }
  return 0;
}

int vbDeviceBucketCacheb::rFlush(int empty_bkts)
// Flushes all buckets in reverse order. Makes them empty
// if empty_bkts is true. Will return a non-zero value if a
// bucket is being referenced by more then one cache
// pointer. Returns zero to indicate a successful flush
// operation.
{
  // Flush, starting at the front so that cache will be flushed
  // in a first in, last out sequence
  vbDeviceBucketb *b = head;
  do {
    if(empty_bkts && b->IsLocked()) {
      return 1;
    }

    if(b->IsDirty()) { // Flush if bucket is used
      if(devptr->ReadyForWriting()) b->Flush(*devptr);
    }
    
    if(empty_bkts) b->MakeNull();
    b = b->Next;
  } while(b != head);

  return 0;
}

void vbDeviceBucketCacheb::Clear()
// Flush all buckets in the cache, and then null them out.
{
  if(devptr) Flush(1);
}

vbDeviceBucketb *vbDeviceBucketCacheb::AcquireBkt()
// Finds least recently reserved bucket that isn't dirty,
// or locked and flushes the bucket. Passes back pointer
// to bucket or returns zero if no bucket available. If
// the cache is full the least-recently reserved bucket
// will be flushed to make room for the new bucket. This
// will prevent the cache from filling up.
{
  vbDeviceBucketb *b = head->Prior; // Least-recently reserved bucket
  while(1) {
 
    // Found an empty bucket in the cache 
    if((!b->IsDirty()) && (!b->IsLocked())) break;

    // Keep searching for the frist empty bucket
    b = b->Prior;
    
    // The cache is full, so use the last bucket
    if(b == head->Prior) {
      b->Flush(*devptr); // Flush the last bucket and move it to the top
      MoveToFront(b);    // Now this becomes the most recently used bucket
      return b;          
    }  
  }

  // Flush any data that might be in the bucket.
  b->Flush(*devptr);
  
  return b;
}

vbDeviceBucketb *vbDeviceBucketCacheb::ReserveBkt(vbDeviceTypes o_dev,
						  vbDeviceTypes i_dev,
						  int ensure_loaded,
						  unsigned bytes)
// Reserves a cache bucket and pass back a pointer to bucket or
// zero if an error occurred. If ensure_loaded == 1, then the 
// data will be read from the input device and loaded in the
// bucket. The "bytes" variable is used to set the length of
// the memory buffer the bucket is connected to during read
// operations.
{
  vbDeviceBucketb *b;
  b = AcquireBkt();
  if(b) {
    b->o_device = o_dev;
    b->i_device = i_dev;
    if(ensure_loaded && bytes) { // Load data from the device
      b->Fetch(*devptr, bytes);
      // If failure fetching data, return null bucket to
      // signal failure, else lock the bucket.
      if(!devptr->IsOK()) {
	b = 0;
      }
      else {
	b->Lock();     // This bucket is currently referenced
	b->SetDirty(); // The bucket is now contains data
      }
    }
    else {
      b->Lock();     // Lock the bucket without fetching any data
                     // Set the dirty flag during data acquisition
    }
  }
  return b;
}

unsigned vbDeviceBucketCacheb::BucketsInUse()
// Returns the number of buckets currently in use.
{
  vbDeviceBucketb *b = head; // Start search from the front
  unsigned num_dirty = 0;
  do {
    if (b->IsDirty()) num_dirty++;
    b = b->Next;
  } while(b != head);
  return num_dirty;
}


vbDeviceCachePtrb::vbDeviceCachePtrb(vbDeviceBucketCacheb &c,
				     vbDeviceTypes o_dev,
				     vbDeviceTypes i_dev)
// General cache pointer constructor. Note that a cache
// is required. The cache pointer stays unbound untill needed.
{
  o_device = o_dev;
  i_device = i_dev;
  bkt = 0;
  cache = &c;
  bound = 0;
}

vbDeviceCachePtrb::vbDeviceCachePtrb(const vbDeviceCachePtrb &c)
// Copy constructor. Note that destination cache pointer
// will not be bound until needed, even if source is bound.
{
  o_device = c.o_device;
  i_device = c.i_device;
  bkt = c.bkt;
  cache = c.cache;
  bound = 0;
}

vbDeviceBucketb *vbDeviceCachePtrb::AllocBucket(vbDeviceTypes o_dev,
						vbDeviceTypes i_dev)
// Allocates a cache bucket and returns a pointer to the bucket,
// or zero if an error occurred. NOTE: This version of the Alloc
// function will not load any data into the bucket. The bucket
// must be loaded must be loaded after the bucket is allocated. 
{
  Release(); // Let go of any currently bound data
  bkt = cache->ReserveBkt(o_dev, i_dev, 0, 0); // Do not load the bucket
  if(bkt) {
     bkt->SetDirty(); // Data modified by constructor
     o_device = o_dev;
     i_device = i_dev;
     bound = 1;
  }
  else { 
    o_device = vbDEVICE_VOID; // bound will = 0 here as well
    i_device = vbDEVICE_VOID;
  }
  return bkt;
}

vbDeviceBucketb *vbDeviceCachePtrb::AllocBucket(vbDeviceTypes o_dev,
						vbDeviceTypes i_dev, 
						unsigned buffer_size)
// Allocates a cache bucket during a read operation and returns 
// a pointer to the bucket, or zero if an error occurred. NOTE: 
// This version of the Alloc function will load data into the 
// bucket. The "buffer_size" variable is used to determine the 
// size of memory buffer needed to store the bucket's data.
{
  Release(); // Let go of any currently bound data
  bkt = cache->ReserveBkt(o_dev, i_dev, 1, buffer_size);
  if(bkt) {
    bkt->SetDirty(); // Data modified by constructor
    o_device = o_dev;
    i_device = i_dev;
    bound = 1;
  }
  else { 
    o_device = vbDEVICE_VOID; // bound will = 0 here as well
    i_device = vbDEVICE_VOID;
  }
  return bkt;
}

void vbDeviceCachePtrb::Copy(const vbDeviceCachePtrb &c)
// Copies one cache pointer into another. Note that the destination
// cache pointer isn't bound, even if the source cache pointer is.
{
  Release();  
  o_device = c.o_device;
  i_device = c.i_device;
  bkt = c.bkt;
  bound = 0;
  cache = c.cache;
}

int vbDeviceCachePtrb::Bind()
// Binds the cache pointer to a bucket containing data.
// Assumes bucket is not already bound. Will return a
// non-zero value if memory for the bucket cannot be
// allocated or if the bucket is not pointing to any
// valid device.
{
  if(bkt) {
    if((bkt->o_device == o_device) && (bkt->i_device == i_device)) { 
      if((o_device != vbDEVICE_VOID) && (i_device != vbDEVICE_VOID)) {
	bkt->Lock();
	bound = 1;
      }
      else {
	// Not pointing to any valid device
	return 1;
      }
    }
  }
  else {
    bkt = cache->ReserveBkt(o_device, i_device, 0, 0);
    if(bkt)
      bound = 1;
    else {
      return 1; // Referencing a null pointer
    }
  }
  return 0;
}

void vbDeviceCachePtrb::Release()
// Signal to the bucket that it is through with it.
{
   if(bound) {
     bkt->Unlock();
     bound = 0;
   }
}

void vbDeviceBucket::Fetch(vbDeviceCache &dev, unsigned bytes)
{
  // Fetch only the MBTYPE portion of the bucket
  MBTYPE *t = (MBTYPE *)this;
  if(t->length() != bytes) t->resize(bytes);
  dev.Read((char *)t->m_buf(), t->length(), i_device);
  Dirty = 0;
}

void vbDeviceBucket::Store(vbDeviceCache &dev)
{
  // Store only the MBTYPE portion of the bucket
  MBTYPE *t = (MBTYPE *)this;
  dev.Write((char *)t->m_buf(), t->length(), o_device);
  Dirty = 0;
}


vbDeviceBucketCache::vbDeviceBucketCache(int n)
  : vbDeviceBucketCacheb(buckets = new vbDeviceBucket[n], n,
			 sizeof(vbDeviceBucket))
// Buckets are stored in an array and organized as a circular doubly-linked
// list by the vbDeviceBucketCacheb class.
{

}

vbDeviceBucketCache::~vbDeviceBucketCache()
{
  if (devptr) Flush(); Disconnect();
  delete[] buckets;
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
