// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: membuf.cpp 
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer 
// File Creation Date: 09/20/1999
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The MemoryBuffer class is used to create variable length memory
buffers. 
*/
// ----------------------------------------------------------- // 
#include <string.h>
#include "membuf.h"

MemoryRep MemoryRep::Null_Ptr;

void *MemoryRep::operator new(size_t BufSize, unsigned Bytes)
// Overloaded new operator used to allocate memory for the size
// of a MemoryRep object plus number of bytes that represent the 
// length of the buffer's data. The memory for the buffer is
// allocated as an array of characters. It is the responsibility
// of the MemoryBuffer class to keep track of the buffer's length.
// The MemoryBuffer class must store the buffer data immediately following
// the reference count. A null pointer object will be statically 
// allocated when a new MemoryRep object is constructed. Null buffers
// will always point to the null pointer object.
{
  // StrSize already accounts for one character
  void *ptr = new char[BufSize + Bytes - 1];
  if(!ptr) { // If allocation fails reference Null_Ptr
    ptr = &Null_Ptr;
    ((MemoryRep *)ptr)->RefCount++;
  }
  return ptr;
}

void MemoryRep::operator delete(void *ptr)
// The delete operator is overloaded so that
// both the buffer data the reference count
// are deleted.
{
  delete[] (char *)ptr;
}

int MemoryBuffer::Alloc(unsigned Bytes, int GB)
{
  DataPtr = new(Bytes) MemoryRep;
  BufferPtr = DataPtr->Data;
  Len = 0;
  if (IsNull()) { // Could not allocate memory for buffer
    DimLen = 1;
    GrowBy = 0;
    return 0;
  }
  else {
    DimLen = Bytes;
    GrowBy = GB;
    return 1;
  }
}
      
void MemoryBuffer::Bind(const MemoryBuffer &buf)
{
  // Bind DataPtr to the same that s is bound to
  DataPtr = buf.DataPtr;
  DataPtr->RefCount++;
}

void MemoryBuffer::UnBind()
{
  DataPtr->RefCount--;

  // Make sure RefCount is a non-zero value
  if(DataPtr == &MemoryRep::Null_Ptr && DataPtr->RefCount == 0)
    DataPtr->RefCount = 1;

  if(DataPtr->RefCount == 0) delete DataPtr;
}

void MemoryBuffer::NewBinding(const MemoryBuffer &buf)
{
  // Unbind buffer from DataPtr that holds it, and
  // bind it to the DataPtr shared by s
  if(DataPtr != buf.DataPtr) {
    UnBind();
    Bind(buf);
  }
}

MemoryBuffer::MemoryBuffer(const void *buf, unsigned Bytes, int GB)
{
  unsigned DimensionLen = Bytes;
  if(GB) { // Compute next highest multiple of GB
    unsigned AddValue = (DimensionLen % GB) ? GB : 0;
    DimensionLen = (DimensionLen / GB) * GB + AddValue;
  }
  if(Alloc(DimensionLen, GB)) CopyN(buf, Bytes);
}

MemoryBuffer::MemoryBuffer(const MemoryBuffer &buf)
{
  Bind(buf);
  Len = buf.Len;
  DimLen = buf.DimLen;
  GrowBy = buf.GrowBy;
  BufferPtr = buf.BufferPtr;
}

MemoryBuffer::~MemoryBuffer()
{
  UnBind();
}

MemoryBuffer &MemoryBuffer::operator=(const MemoryBuffer &buf)
{
  if(this != &buf) {
    NewBinding(buf);
    Len = buf.Len;
    DimLen = buf.DimLen;
    GrowBy = buf.GrowBy;
    BufferPtr = buf.BufferPtr;
  }
  return *this;
}

int MemoryBuffer::Realloc(unsigned NewDimLen, int Keep)
{
  if(GrowBy == 0) return 0;

  MemoryRep *NewMemoryRep = new(NewDimLen) MemoryRep;
  if(NewMemoryRep == &MemoryRep::Null_Ptr) return 0;

  if(Keep) { // Copy old data into new space.
    if(NewDimLen < Len) Len = NewDimLen;
    memmove(NewMemoryRep->Data, BufferPtr, Len);
  }
  else { // Do not keep old data
    Len = 0;
  }

  UnBind();                // Unbind from old data
  DataPtr = NewMemoryRep;    // Bind to new Data
  BufferPtr = DataPtr->Data; // Point to starting text
  DimLen = NewDimLen;      // Record new allocated length
  return 1;
}

int MemoryBuffer::resize(unsigned bytes, int Keep)
// Resize the logical length of the buffer. If the
// "keep" variable is true the old data will be
// copied into the new space. By default the old
// data will be deleted.
{
  if(GrowBy == 0) return 0;

  MemoryRep *NewMemoryRep = new(bytes) MemoryRep;
  if(NewMemoryRep == &MemoryRep::Null_Ptr) return 0;

  if(Keep) { // Copy old data into new space.
    if(bytes < Len) Len = bytes;
    memmove(NewMemoryRep->Data, BufferPtr, Len);
  }

  UnBind();                  // Unbind from old data
  DataPtr = NewMemoryRep;    // Bind to new Data
  BufferPtr = DataPtr->Data; // Point to starting text
  DimLen = Len = bytes;      // Record new allocated length
  return 1;
}

int MemoryBuffer::EnsureUnique()
{
  if(!IsUnique()) {
    // Create a unique clone of this buffer
    MemoryBuffer buf(DimLen, GrowBy);
    buf.CopyN(BufferPtr, Len);
    if(buf.IsNull()) return 0; // Could not copy
    NewBinding(buf);           // Bind to copy
    BufferPtr = buf.BufferPtr;     // Initialize starting text element
  }
  return 1;
}

void MemoryBuffer::SetLength(unsigned Bytes)
{
  if(Bytes > DimLen) Bytes = DimLen;
  Len = Bytes;
}

void MemoryBuffer::CopyN(const void *buf, unsigned Bytes)
{
  Len = 0; InsReplAt(0, buf, Bytes);
}

void MemoryBuffer::Copy(const MemoryBuffer &buf)
{
  CopyN(buf.BufferPtr, buf.Len);
}

int MemoryBuffer::Find(void *buf, unsigned Offset)
// Returns index of first occurrence of pattern void *buf   
// Returns 0xffff if pattern not found
{
  unsigned char *Start = BufferPtr + Offset;     // Start of buffer data
  unsigned char *Next = Start;                   // Next buffer element
  unsigned char *Pattern = (unsigned char *)buf; // Next pattern element
  unsigned i = Offset;                           // Next buffer element index
  
  while(i < Len && *Pattern) {
    if (*Next == *Pattern) {
      Pattern++;
      if(*Pattern == 0) return i; // Pattern was found
      Next++;
    }
    else {
      i++;
      Start++;
      Next = Start;
      Pattern = (unsigned char *)buf;
    }
  }
  return NoMatch; // No match was found
}

int MemoryBuffer::Find(void *buf, unsigned Bytes, unsigned Offset) const
// Returns index of first occurrence of pattern void *buf   
// Returns 0xffff if pattern not found
{
  char *Start = (char *)BufferPtr + Offset; // Start of string data
  char *Next = Start;          // Next string element
  char *Pattern = (char *)buf; // Next pattern element
  unsigned i = Offset, j = 1;  // String and pattern indexes
  
  while(i < Len && j <= Bytes) {
    if (*Next == *Pattern) { // Matching character
      if(j == Bytes) return i; // Entire match was found
      Next++; Pattern++; j++;
    }
    else { // Try next spot in string
      i++;
      Start++;
      Next = Start;
      Pattern = (char *)buf; j = 1;
    }
  }
  return NoMatch; // No match was found
}

unsigned MemoryBuffer::DeleteAt(unsigned Pos, unsigned Bytes)
{
  unsigned long buf; // long is used to prevent overflows
  unsigned End;

  if(Pos < Len && Bytes !=0) {
    if(!EnsureUnique()) return 0; // Can't update
    buf = Pos + Bytes;
    if(buf > Len) buf = Len;
    End = unsigned(buf);
    Bytes = End - Pos;
    // Move the elements up to take their place
    memmove(BufferPtr+Pos, BufferPtr+End, Len-End);
    Len -= Bytes;
  }
  else
    Bytes = 0;

  return Bytes;
}

unsigned MemoryBuffer::InsReplAt(unsigned Pos, const void *buf, unsigned Bytes,
				 int Ins)
{
  unsigned Room, Needed, AddVal;
  
  if(Ins) {
    Room = DimLen - Len;
    if(Bytes > Room && GrowBy != 0) {
      Needed = (Bytes - Room);
      AddVal = (Needed % GrowBy) ? GrowBy : 0;
      Needed = (Needed / GrowBy) * GrowBy + AddVal;
      Grow_By(Needed);
    }
  }

  if (!EnsureUnique()) return 0; // Can't update
  if (Pos >= Len) Pos = Len; // Don't allow gaps

  if(Ins) { // Keep things in range. May have to truncate
    if(Bytes > DimLen - Len) Bytes = DimLen - Len;
    if(Pos < Len) {
      // Make room in the middle for inserted data
      memmove(BufferPtr+Pos+Bytes, BufferPtr+Pos, Len-Pos);
    }
  }
  else {
    if(Bytes > DimLen - Pos) Bytes = DimLen -Pos;
  }

  // Copy in source, compute final length
  memmove(BufferPtr+Pos, (unsigned char *)buf, Bytes);
  if(Ins) {
    Len += Bytes;
  }
  else {
    if((Pos+Bytes) > Len) Len = Pos+Bytes;
  }
  return Bytes;
}

int BufferCompare(const MemoryBuffer &a, const MemoryBuffer &b)
// Returns -1 if a < b, 0 if a == b, and 1 if a > b
{

  unsigned an = a.Len;
  unsigned bn = b.Len;
  unsigned sn = (an < bn) ? an : bn;
  unsigned char *ap = (unsigned char *)a.BufferPtr;
  unsigned char *bp = (unsigned char *)b.BufferPtr;

  for(unsigned i = 0; i < sn; i++) {
    if(*ap < *bp) return -1;
    if(*ap++ > *bp++) return 1;
  }

  if(an == bn) return 0;
  if(an < bn) return -1;
  return 1;
}

void MemoryBuffer::Load(const void *buf, unsigned Bytes) 
// Load the buffer
{   
    Clear(); InsReplAt(0, buf, Bytes);
}

void MemoryBuffer::Clear() 
// Clear the buffer
{ 
  if(Len > 0) DeleteAt(0, Len); 
} 

MemoryBuffer operator+(const MemoryBuffer &a, const MemoryBuffer &b)
{
  MemoryBuffer buf(a.Len + b.Len);
  buf += a;
  buf += b;
  return buf;
}

unsigned char& MemoryBuffer::operator[](unsigned i) 
{
  if(EnsureUnique()) { // Ensure unique before allowing modifications
    return BufferPtr[CheckIndex(i)];
  }
  else { 
    return MemoryRep::Null_Ptr.Data[0]; // Point byte to null string
  }
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
