// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: vbconfig.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer  
// File Creation Date: 02/19/1996 
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbConfig class is a character string based singly linked list
that is used to load program parameters from a ASCII text based
configuration file.
*/
// ----------------------------------------------------------- // 
#include <stdlib.h>
#include <stdio.h>
#include <fstream.h>
#include <string.h>
#include <ctype.h>
#include "vbconfig.h"

vbConfig::~vbConfig()
{
  UnLoad();
}

void vbConfig::UnLoad()
{
  Clear();
}         

int vbConfig::Load(char *fname)
{
  FileName = fname;
  return Load();
}

int vbConfig::Load(const char *fname)
{
  FileName = fname;
  return Load();
}

int vbConfig::Load(const vbString &fname)
{
  FileName = fname;
  return Load();
}

int vbConfig::ReLoad()
{
  UnLoad();
  return Load();
}

int vbConfig::ReLoad(char *fname)
{
  UnLoad();
  return Load(fname);
}

int vbConfig::ReLoad(const char *fname)
{
  UnLoad();
  return Load(fname);
}

int vbConfig::ReLoad(const vbString &fname)
{
  UnLoad();
  return Load(fname);
}

void vbConfig::FilterLineFeed(char &c)
// Added to filter the end of line sequence from a line of text.
{
  switch(c) {
    case '\r':
      c = '\0';
      break;
    case '\n':
      c = '\0';
      break;
    default:
      break;
  }
}

int vbConfig::Load()
{
  const int MaxLine = 1024;
  char LineBuffer[MaxLine];
  vbString buf;
  int Status, i;
  
  // Open vbConfig file
#ifdef __BCC32__
  // The BCC 32 ios class does not have an enumeration for "nocreate"
  ifstream InFile(FileName.c_str(), ios::in);
#else
  ifstream InFile(FileName.c_str(), ios::in|ios::nocreate);
#endif
  if(!InFile) return 0; 
        
  while(!InFile.eof()) {
    // Clear the line buffer before each read
    for(i = 0; i < MaxLine; i++) LineBuffer[i] = '\0';
    
    // Read in config file line by line
    InFile.getline(LineBuffer, MaxLine);
    
    // Get rid of the end of line sequence
    for(i = 0; i < MaxLine; i++) FilterLineFeed(LineBuffer[i]);
    
    // Copy each line to temporary holding buffer
    buf = LineBuffer;
    
    // Load config file data into singly-linked list
    Status = StoreCfgData(buf);
    if(!Status) return 0;
  }

  InFile.close();
  return 1; // Indicates success
}

int vbConfig::StoreCfgData(const vbString &str)
{
  vbConfigListNode *chsptr;
  vbString buf(str);
  
  // Remove any leading space 
  unsigned offset = buf.Find(" ");
  if(offset == 0) buf.DeleteAt(offset, 1);

  if(filter_comments) { // Ignore all lines starting with a comment ID
    if(buf[0] == comment_char)
      return 1; 
  }
  
  // Look for lines containing a parm ID string
  offset = buf.Find(parm_ID.c_str());
  if(offset != vbString::NoMatch) {
    vbString Name(buf);
    vbString Value(buf);

    // Separate the parameter name and value
    Name.DeleteAt(offset, (Name.length() - offset));
    offset = Name.Find(" "); // Remove any trailing spaces
    if(offset != vbString::NoMatch) Name.DeleteAt(offset, 1);
    chsptr = Store(Name); // Store the parameter name
    if(!chsptr) return 0;

    // Store the config file parameter value 
    offset = Value.Find(parm_ID.c_str());
    Value.DeleteAt(0, (offset+parm_ID.length()));
    offset = Value.Find(" "); // Remove any leading spaces
    if(offset == 0) Value.DeleteAt(offset, 1);
    offset = Value.Find(" "); // Look for any trailing spaces or other chars
    if(offset != vbString::NoMatch) {
      // Remove any comments following a space but leave long
      // character strings for values such as file names with
      // white spaces or multiple config values on one line
      if(filter_comments) {
      if(Value[offset+1] == comment_char) 
	Value.DeleteAt(offset, (Value.length() - offset));
      }
    }
    chsptr = Store(Value);
    if(!chsptr) return 0;
  }

  return 1; // Indicate success
}

char* vbConfig::GetStrValue(const vbString &Name, int fn)
// Search for string matching the "Name" variable. Will search
// using the full name unless the "fn" variable is false.
{
  vbConfigListNode *ptr;

  if(fn)
    ptr = Find(Name);
  else
    ptr = FindAny(Name);
  
  if(ptr) return ptr->GetNext()->Data.c_str();  // Return config value

  // Return NULL if config value is not found
  return 0;
}

char* vbConfig::GetStrValue(char *Name, int fn)
{
  vbString buf(Name);
  return GetStrValue(buf, fn);
}

char* vbConfig::GetStrValue(const char *Name, int fn)
{
  vbString buf(Name);
  return GetStrValue(buf, fn);
}

int vbConfig::GetIntValue(const vbString &Name, int fn)
// Search for string matching the "Name" variable. Will search
// using the full name unless the "fn" variable is false.
{
  vbConfigListNode *ptr;

  if(fn)
    ptr = Find(Name);
  else
    ptr = FindAny(Name);

  if(ptr) return atoi(ptr->GetNext()->Data.c_str()); // Return config value

  //Return NULL if config value is not found
  return 0;
}

int vbConfig::GetIntValue(char *Name, int fn)
{
  vbString buf(Name);
  return GetIntValue(buf, fn);
}


int vbConfig::GetIntValue(const char *Name, int fn)
{
  vbString buf(Name);
  return GetIntValue(buf, fn);
}

double vbConfig::GetFloatValue(const vbString &Name, int fn)
// Search for string matching the "Name" variable. Will search
// using the full name unless the "fn" variable is false.
{
  vbConfigListNode *ptr;

  if(fn)
    ptr = Find(Name);
  else
    ptr = FindAny(Name);

  if(ptr) return atof(ptr->GetNext()->Data.c_str()); // Return config value

  //Return NULL if config value is not found
  return 0;
}

double vbConfig::GetFloatValue(char *Name, int fn)
{
  vbString buf(Name);
  return GetFloatValue(buf, fn);
}

double vbConfig::GetFloatValue(const char *Name, int fn)
{
  vbString buf(Name);
  return GetFloatValue(buf, fn);
}

double vbConfig::GetDFPValue(const vbString &Name, int fn)
{
  return GetFloatValue(Name, fn);
}

double vbConfig::GetDFPValue(char *Name, int fn)
{
  vbString buf(Name);
  return GetFloatValue(buf, fn);
}

double vbConfig::GetDFPValue(const char *Name, int fn)
{
  vbString buf(Name);
  return GetFloatValue(buf, fn);
}

long vbConfig::GetLongValue(const vbString &Name, int fn)
// Search for string matching the "Name" variable. Will search
// using the full name unless the "fn" variable is false.
{
  vbConfigListNode *ptr;

  if(fn)
    ptr = Find(Name);
  else
    ptr = FindAny(Name);

  if(ptr) return atol(ptr->GetNext()->Data.c_str()); // Return config value

  //Return NULL if config value is not found
  return 0;
}

long vbConfig::GetLongValue(char *Name, int fn)
{
  vbString buf(Name);
  return GetLongValue(buf, fn);
}

long vbConfig::GetLongValue(const char *Name, int fn)
{
  vbString buf(Name);
  return GetLongValue(buf, fn);
}

int vbConfig::WriteConfigLine(const vbString &parm, const vbString &value)
// Write a line to the config file.  NOTE: All parameter values should
// not contain a parameter ID label at the end of the string. The 
// parameter ID label will be added to mark it as a config file parameter.
// By default an equal sign will be used as a parameter ID.
{
  ofstream outfile(FileName.c_str(), ios::app);

  if(!outfile) {
    outfile.close();
    return 0;
  }

  outfile << parm << parm_ID.c_str() << value << endl;
  outfile.close();
  return 1;
}  

int vbConfig::WriteConfigLine(const char *parm, const char *value)
{
  vbString p(parm); vbString v(value);
  return WriteConfigLine(p, v);
}

int vbConfig::WriteConfigLine(char *parm, char *value)
{
  vbString p(parm); vbString v(value);
  return WriteConfigLine(p, v);
}

int vbConfig::WriteCommentLine(const vbString &s)
// Write a comment line to the config file.  NOTE: Do not include the
// comment ID character. The comment ID will by automatically inserted.
{
  ofstream outfile(FileName.c_str(), ios::app);

  if(!outfile) {
    outfile.close();
    return 0;
  }

  outfile << comment_char << ' ' << s.c_str() << endl;
  outfile.close();
  return 1;
}  

int vbConfig::WriteCommentLine(const char *s)
{
  vbString cm(s);
  return WriteCommentLine(cm);
}

int vbConfig::WriteCommentLine(char *s)
{
  vbString cm(s);
  return WriteCommentLine(cm);
}

int vbConfig::WriteLine(const vbString &s)
// Write a line of text to the config file.  
{
  ofstream outfile(FileName.c_str(), ios::app);

  if(!outfile) {
    outfile.close();
    return 0;
  }

  outfile << s.c_str() << endl;
  outfile.close();
  return 1;
}  

int vbConfig::WriteLine(const char *s)
{
  vbString line(s);
  return WriteLine(line);
}

int vbConfig::WriteLine(char *s)
{
  vbString line(s);
  return WriteLine(line);
}

int vbConfig::ChangeConfigLine(const vbString &string_to_replace,
			     const vbString &string_to_insert,
			     int check_case, int append_lines)
// This function is used to replace strings in the config file.
// Returns the number of strings processed or zero if an error occurs.
{
  const int MAX_LINE = 1024;   // Maximum characters per line
  char LineBuffer[MAX_LINE];
  char rawLineBuffer[MAX_LINE];
  vbString LineData;
  vbString appendLineBuffer;
  vbConfigList list;
  int i, j;
  unsigned offset = 0;
  unsigned long line_number = 0;
  int num_processed = 0;      
  
#if defined (__WIN32__) || defined (__DOS__) 
  // In MS-DOS there are two file types, text and binary
#ifdef __BCC32__
  // The BCC 32 ios class does not have an enumeration for "nocreate"
  fstream iofile(FileName.c_str(), ios::in|ios::binary);
#else
  fstream iofile(FileName.c_str(), ios::in|ios::nocreate|ios::binary);
#endif
  
#elif defined (__UNIX__) 
  // In UNIX there is only one file type
  fstream iofile(FileName.c_str(), ios::in|ios::nocreate);
#else
#error "You must define a file system type: __WIN32__ __UNIX__ or __DOS__"
#endif

  if(!iofile) return 0;
	
  while(!iofile.eof()) { // Read in the file line by line

    // Clear the buffers
    for(i = 0; i < MAX_LINE; i++) LineBuffer[i] = '\0';
    for(i = 0; i < MAX_LINE; i++) rawLineBuffer[i] = '\0';
    LineData.DeleteAt(0, LineData.length());
    offset = 0; // Reset string offset
    
    iofile.getline(rawLineBuffer, MAX_LINE);
    line_number++;
        
    // Filter each line of text
    for(i = 0, j = 0; i < MAX_LINE; i++) {
      if((rawLineBuffer[i] == '\r') || (rawLineBuffer[i] == '\n')) break;
      LineBuffer[j++] = rawLineBuffer[i];
    }

    // Detect backslashes used to mark the continuation of a line
    if((LineBuffer[strlen(LineBuffer)-1] == '\\') && (append_lines == 1)) {
      appendLineBuffer += LineBuffer;
      appendLineBuffer += '\n'; // Insert LF after the backslash 
      continue;  // Goto the top of loop and append next line
    }
    if(appendLineBuffer.length() > 0) { // Append the next line
      appendLineBuffer += LineBuffer;
      LineData = appendLineBuffer;
      appendLineBuffer.DeleteAt(0, appendLineBuffer.length());
    }
    else
      LineData = LineBuffer;
    
    // Get rid of EOF marker
    if((strcmp(LineData.c_str(), "\0") == 0) && (iofile.eof())) break;

    // Routine to replace a line
    // ----------------------------------------------------------- 
    if(check_case) // Find entire match checking the case of the string
      offset = LineData.Find((char *)string_to_replace.c_str(),
			     string_to_replace.length(), 0);
    else
      offset = LineData.IFind((char *)string_to_replace.c_str(),
			      string_to_replace.length(), 0);

    if(offset != vbString::NoMatch) {
      offset += string_to_replace.length();
      LineData.DeleteAt(offset, (LineData.length() - offset));
      LineData.InsertAt(offset, parm_ID.c_str()); // Insert parameter ID string
      LineData.InsertAt(++offset, string_to_insert);
      num_processed++; // Record number of string processed for this file
    }
    list.Store(LineData); // Store all the lines 
  }
  
  // If any strings were replaced in the file, rewrite the file
  // ----------------------------------------------------------- 
  if(num_processed > 0) {
    char *EndOfLineSequence;
    iofile.close(); 
#if defined (__WIN32__) || defined(__DOS__)
    // In DOS/Windows there are two file types, text and binary
    fstream iofile(FileName.c_str(), ios::out|ios::trunc|ios::binary);
    EndOfLineSequence = "\r\n"; 
#else 
    // In UNIX there is only one file type
    fstream iofile(FileName.c_str(), ios::out|ios::in|ios::trunc);
    EndOfLineSequence = "\n"; 
#endif
    
    if(!iofile) return 0;

    vbConfigListNode *list_ptr = list.GetFront();
    while(!list.IsHeader(list_ptr)) {
      iofile.write(list_ptr->Data.c_str(), list_ptr->Data.length());
      iofile.write(EndOfLineSequence, strlen(EndOfLineSequence));
      list_ptr = list_ptr->GetNext(); 
    }

    iofile.close();

    // Make the changes to the config list in already in memory
    ReLoad();
  }
  else
    iofile.close();
  
  list.Clear();
  return num_processed;
}

int vbConfig::ChangeConfigValue(const vbString &parm, const vbString &value)
// Modify the config file. NOTE: All parameter values should not contain 
// a parameter ID label at the end of the string. The parameter ID label
// will be added to mark it as a config file parameter. By default an
// equal sign will be used as a parameter ID.
{
  int rv = ChangeConfigLine(parm, value, 1, 1);

  // Write the parameter if it does not exist
  if(!rv) return WriteConfigLine(parm, value);

  return rv;
}


int vbConfig::ChangeConfigValue(const char *parm, const vbString &value)
{
  vbString p(parm); vbString v(value);
  return ChangeConfigValue(p, v);
}

int vbConfig::ChangeConfigValue(const vbString &parm, const char *value)
{
  vbString p(parm); vbString v(value);
  return ChangeConfigValue(p, v);
}

int vbConfig::ChangeConfigValue(char *parm, const vbString &value)
{
  vbString p(parm); vbString v(value);
  return ChangeConfigValue(p, v);
}

int vbConfig::ChangeConfigValue(const vbString &parm, char *value)
{
  vbString p(parm); vbString v(value);
  return ChangeConfigValue(p, v);
}

int vbConfig::ChangeConfigValue(const char *parm, const char *value)
{
  vbString p(parm); vbString v(value);
  return ChangeConfigValue(p, v);
}

int vbConfig::ChangeConfigValue(char *parm, char *value)
{
  vbString p(parm); vbString v(value);
  return ChangeConfigValue(p, v);
}

int vbConfig::ChangeConfigValue(const vbString &parm, int value)
{
  char buf[255];
  sprintf(buf, "%d", value);
  vbString val(buf);
  return ChangeConfigValue(parm, val);
}

int vbConfig::ChangeConfigValue(const char *parm, int value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}

int vbConfig::ChangeConfigValue(char *parm, int value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}

int vbConfig::ChangeConfigValue(const vbString &parm, unsigned value)
{
  char buf[255];
  sprintf(buf, "%u", value);
  vbString val(buf);
  return ChangeConfigValue(parm, val);
}

int vbConfig::ChangeConfigValue(const char *parm, unsigned value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}

int vbConfig::ChangeConfigValue(char *parm, unsigned value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}

int vbConfig::ChangeConfigValue(const vbString &parm, long value)
{
  char buf[255];
  sprintf(buf, "%d", value);
  vbString val(buf);
  return ChangeConfigValue(parm, val);
}

int vbConfig::ChangeConfigValue(const char *parm, long value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}

int vbConfig::ChangeConfigValue(char *parm, long value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}

int vbConfig::ChangeConfigValue(const vbString &parm, float value)
{
  char buf[255];
  sprintf(buf, "%g", value);
  vbString val(buf);
  return ChangeConfigValue(parm, val);
}

int vbConfig::ChangeConfigValue(const char *parm, float value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}
  
int vbConfig::ChangeConfigValue(char *parm, float value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}

int vbConfig::ChangeConfigValue(const vbString &parm, double value)
{
  char buf[255];
  sprintf(buf, "%g", value);
  vbString val(buf);
  return ChangeConfigValue(parm, val);
}

int vbConfig::ChangeConfigValue(const char *parm, double value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}

int vbConfig::ChangeConfigValue(char *parm, double value)
{
  vbString p(parm);
  return ChangeConfigValue(p, value);
}
  
vbConfigList::~vbConfigList()
{
  Clear();
}

int vbConfigList::Copy(const vbConfigList &List)
{
  return vbDLListBase::Copy(List);
}

vbConfigListNode *vbConfigList::AllocNode(const vbString &X)
{
  return new vbConfigListNode(X);
}

vbDNodeBase *vbConfigList::DupNode(const vbDNodeBase *Node)
{
  return AllocNode(((vbConfigListNode *)Node)->Data);
}

void vbConfigList::FreeNode(vbDNodeBase *Node)
{
  delete((vbConfigListNode *)Node);
}

const vbConfigListNode *vbConfigList::Find(const vbString &X,
					 const vbConfigListNode *ptr) const
// Returns the first node having an element that matched X
{
  if(ptr == 0) ptr = GetFront();

  while(!IsHeader(ptr)) { // Scan until end of list
    if(ptr->Data == X) return ptr; // Match found
    ptr = ptr->GetNext();
  }
  return 0; // No match
}

vbConfigListNode *vbConfigList::Find(const vbString &X,
				       vbConfigListNode *ptr)
// Returns the first node having an element that matched X
{
  if(ptr == 0) ptr = GetFront();

  while(!IsHeader(ptr)) { // Scan until end of list
    if(ptr->Data == X) return ptr; // Match found
    ptr = ptr->GetNext();
  }
  return 0; // No match
}

vbConfigListNode *vbConfigList::FindAny(const vbString &X,
					  vbConfigListNode *ptr)
// Returns the first node having any element that matched X
{
  if(ptr == 0) ptr = GetFront();

  unsigned offset;
  
  while(!IsHeader(ptr)) { // Scan until end of list
    offset = 0;
    offset = ptr->Data.Find(X.c_str());
    if(offset != vbString::NoMatch) return ptr; // Match found
    ptr = ptr->GetNext();
  }
  return 0; // No match
}

vbConfigListNode *vbConfigList::Store(const vbString &X)
{
  vbConfigListNode *ptr = AllocNode(X);
  if(ptr) AttachToBack(ptr);
  return ptr; // Return a pointer to the node added
}
  
vbConfigListNode *vbConfigList::GetFront()
{
  return(vbConfigListNode *)vbDLListBase::GetFront();
}
  
const vbConfigListNode *vbConfigList::GetFront() const
{ 
  return(vbConfigListNode *)vbDLListBase::GetFront();
}

int vbConfigList::IsHeader(const vbConfigListNode *Node) const
{
  return vbDLListBase::IsHeader(Node);
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
