// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: vbshtml.cpp
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 01/25/2000
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbsHTML class is a base class used to parse html documents.
*/
// ----------------------------------------------------------- // 
#include <ctype.h>
#include "vbshtml.h"
#include "vbstring.h"

// The following array of HTML tags is a combination of HTML
// 2.0, 3.0, 3.2 tags supported by Netscape's Navigator
// web browser, Microsoft's Internet Explorer web browser, 
// and standards defined by the World Wide Web Consortium.
// This list was taken from the Willcam's Comprehensive HTML 
// Cross Reference at: 
// http://www.willcam.com/cmat/html/crossref.html 
const char *vbs_SUPPROTED_TAGS[vbsMAX_SUPPORTED_TAGS] = {
  "vbs_invalid_tag", // Invalid tag specified
  "vbs_unknown_tag", // Unknown tag specified
  "vbs_special_tag", // Unknown special tags starting with an 
                     // ampersand ending in a semicolon &xxxx;

  // Tags and format specifiers with special meaning
  "!--",         // Comment
  "&lt;",        // Less than sign "<" 
  "&gt;",        // Greater then sign ">" 
  "&amp;",       // Ampersand "&" 
  "&nbsp;",      // Non-breaking space 
  "&quot;",      // Quotation mark
  "&#",          // Extended ASCII character set
  
  // HTML tag strings
  "A",           // Anchor
  "ABBREV",      // Abbreviation
  "ACRONYM",     // Acronym
  "ADDRESS",     // Address
  "APPLET",      // Java Applet
  "AREA",        // Area
  "AU",          // Author
  "AUTHOR",      // Author
  "B",           // Bold
  "BANNER",      // Banner
  "BASE",        // Base
  "BASEFONT",    // Base Font
  "BGSOUND",     // Background Sound
  "BIG",         // Big Text
  "BLINK",       // Blink
  "BLOCKQUOTE",  // Block Quote
  "BQ",          // Block Quote
  "BODY",        // Body
  "BR",          // Line Break
  "CAPTION",     // Caption
  "CENTER",      // Center
  "CITE",        // Citation
  "CODE",        // Code
  "COL",         // Table Column
  "COLGROUP",    // Table Column Group
  "CREDIT",      // Credit
  "DEL",         // Deleted Text
  "DFN",         // Definition
  "DIR",         // Directory List
  "DIV",         // Division
  "DL",          // Definition List
  "DT",          // Definition Term
  "DD",          // Definition Definition
  "EM",          // Emphasized
  "EMBED",       // Embed
  "FIG",         // Figure
  "FN",          // Footnote
  "FONT",        // Font
  "FORM",        // Form
  "FRAME",       // Frame
  "FRAMESET",    // Frame Set
  "H1",          // Heading 1
  "H2",          // Heading 2
  "H3",          // Heading 3
  "H4",          // Heading 4
  "H5",          // Heading 5
  "H6",          // Heading 6
  "HEAD",        // Head
  "HR",          // Horizontal Rule
  "HTML",        // HTML
  "I",           // Italic
  "IFRAME",      // Frame - Floating
  "IMG",         // Inline Image
  "INPUT",       // Form Input
  "INS",         // Inserted Text
  "ISINDEX",     // Is Index
  "KBD",         // Keyboard
  "LANG",        // Language
  "LH",          // List Heading
  "LI",          // List Item
  "LINK",        // Link
  "LISTING",     // Listing
  "MAP",         // Map
  "MARQUEE",     // Marquee
  "MATH",        // Math
  "MENU",        // Menu List
  "META",        // Meta
  "MULTICOL",    // Multi Column Text
  "NOBR",        // No Break
  "NOFRAMES",    // No Frames
  "NOTE",        // Note
  "OL",          // Ordered List
  "OVERLAY",     // Overlay
  "P",           // Paragraph
  "PARAM",       // Parameters
  "PERSON",      // Person
  "PLAINTEXT",   // Plain Text
  "PRE",         // Preformatted Text
  "Q",           // Quote
  "RANGE",       // Range
  "SAMP",        // Sample
  "SCRIPT",      // Script
  "SELECT",      // Form Select
  "SMALL",       // Small Text
  "SPACER",      // White Space
  "SPOT",        // Spot
  "STRIKE",      // Strikethrough
  "STRONG",      // Strong
  "SUB",         // Subscript
  "SUP",         // Superscript
  "TAB",         // Horizontal Tab
  "TABLE",       // Table
  "TBODY",       // Table Body
  "TD",          // Table Data
  "TEXTAREA",    // Form Text Area
  "TEXTFLOW",    // Java Applet Textflow
  "TFOOT",       // Table Footer
  "TH",          // Table Header
  "THEAD",       // Table Head
  "TITLE",       // Title
  "TR",          // Table Row
  "TT",          // Teletype
  "U",           // Underlined
  "UL",          // Unordered List
  "VAR",         // Variable
  "WBR",         // Word Break
  "XMP"          // Example
};

vbsHTMLTagInfo::vbsHTMLTagInfo() 
{
  start_tag = end_tag = -1;
  tag_length = 0;
  tag_id = 0; 
  tag = attr = tag_info = "\0";
  start_instruction = end_instruction = 0; 
  has_attributes = 0;
}

vbsHTMLTagInfo::~vbsHTMLTagInfo() 
{ 

}

int operator==(const vbsHTMLTagInfo &a, const vbsHTMLTagInfo &b)
// Overloaded == operator added to work with vbs linked list class.
{
  if(a.start_tag != b.start_tag) return 0; 
  if(a.end_tag != b.end_tag) return 0;
  if(a.tag_length != b.tag_length) return 0;
  if(a.tag_id != b.tag_id) return 0;
  if(a.tag != b.tag) return 0;
  if(a.attr != b.attr) return 0;
  if(a.tag_info != b.tag_info) return 0;
  if(a.start_instruction != b.start_instruction) return 0;
  if(a.end_instruction != b.end_instruction) return 0; 
  if(a.has_attributes != b.has_attributes) return 0;
  return 1;
}

void vbsHTMLTagInfo::Copy(const vbsHTMLTagInfo &ob)
// Function used during copying and assignment.
{
  start_tag = ob.start_tag; 
  end_tag = ob.end_tag;
  tag_length = ob.tag_length;
  tag_id = ob.tag_id;
  tag = ob.tag;
  attr = ob.attr;
  tag_info = ob.tag_info;
  start_instruction = ob.start_instruction;
  end_instruction = ob.end_instruction; 
  has_attributes = ob.has_attributes;
}

vbsHTML::vbsHTML() 
{ 
  tag_list.Clear();
}

vbsHTML::~vbsHTML() 
{ 
  df_Close();
  tag_list.Clear();
}

void vbsHTML::ClearTagList()
// Clears the tag list.
{
  tag_list.Clear();
}

char *vbsHTML::GetTag(int tag_id)
// Returns a null terminated string 
// based on the value of the tag id 
// number.
{
  if(tag_id > vbsMAX_SUPPORTED_TAGS) tag_id = vbsHTML::vbs_invalid_tag;
  return (char *)vbs_SUPPROTED_TAGS[tag_id];
}

const char *vbsHTML::GetTag(int tag_id) const
// Returns a null terminated string 
// based on the value of the tag id 
// number.
{
  if(tag_id > vbsMAX_SUPPORTED_TAGS) tag_id = vbsHTML::vbs_invalid_tag;
  return (char *)vbs_SUPPROTED_TAGS[tag_id];
}

int vbsHTML::GetTagID(const vbString &tag)
// Returns an numerical value that represents the specified tag.
{
  int i; // Index of supported tags

  for(i = 0; i < vbsMAX_SUPPORTED_TAGS; i++) {
    if(CaseICmp(tag, vbs_SUPPROTED_TAGS[i]) == 0)
      return i; // Corresponds to the supported tags enumeration
  }
  return vbsHTML::vbs_unknown_tag;
}

int vbsHTML::LoadHTMLFile(const char *fname)
// Open the specified HTML file and collect the tags in a 
// doubly linked list. Returns a DiskFileB error code if 
// an error occurs.
{
  // Open the specified file
  int rv = df_Open(fname , DiskFileB::df_READONLY, DiskFileB::df_NO_CREATE);
  if(rv != DiskFileB::df_NO_ERROR) return rv;

  // Obtain the file location of all the tags in the document
  rv = CollectHTMLTags();
  if(rv != DiskFileB::df_NO_ERROR) return rv;

  // Read the tags
  rv = ProcessHTMLTags();
  if(rv != DiskFileB::df_NO_ERROR) return rv;

  return  DiskFileB::df_NO_ERROR;
}

void vbsHTML::CloseFile()
// Close the open HTML file after a load operation.
{
  df_Close();
}

int vbsHTML::CollectHTMLTags()
// Collect all the HTML tags in an previously opened file.
// Returns a DiskFileB error code if an error occurs.
{
  int rv, i;
  num_tags = 0;
  int found_open_tag = 0;
  int looking_for_tag_end = 0;
  char prev_tag;
  
  tag_list.Clear(); // Clear the tag list
  rv = df_Rewind(); // Rewind the open html file
  if(rv != DiskFileB::df_NO_ERROR) return rv;

  vbsHTMLTagInfo tag;
  char c;
  while(!df_EOF()) {
    rv = df_Get(c);
    if(rv != DiskFileB::df_NO_ERROR) return rv;

    switch(c) {
      case '<' : case '&' :
	if(c == '<') prev_tag = '<'; else prev_tag = '&';
	if(!looking_for_tag_end) { 
	  found_open_tag = 1;
	  looking_for_tag_end = 1;
	  tag.start_tag = df_FilePosition();
	  if(tag.start_tag > (df_StreamPos)0) tag.start_tag--;
	}
	break;

      case '#' : // Look for extended ASCII characters denoted by: "&#xxx;"
	if((prev_tag == '&') && (tag.start_tag = (df_FilePosition() - 1))) {
	  if(found_open_tag) { // Look for the end of a tag
	    // Get the next three numeric characters following a "&#" tag
	    // plus the terminating semicolon or space.
	    for(i = 0; i < 4; i++) { 
	      df_Get(c);
	      if((c == ';') || (c == ' ')) break;
	    }
	    tag.start_tag--; // Account for the ampersand
	    tag.end_tag = df_FilePosition();
	    num_tags++;
	    
	    // Calculate the length of the tag
	    tag.tag_length = tag.end_tag - tag.start_tag;
	    
	    tag_list.StoreNode(tag); // Add the tag to the list
	    found_open_tag = 0;
	    looking_for_tag_end = 0;
	  }
	} 
	break;

      case '>' : case  ';' :
	if(c == '>') prev_tag = '>'; else prev_tag = ';';
	if(found_open_tag) { // Look for the end of a tag
	  tag.end_tag = df_FilePosition();
	  num_tags++;
      
	  // Calculate the length of the tag including the angle brackets
	  tag.tag_length = tag.end_tag - tag.start_tag;
      
	  tag_list.StoreNode(tag); // Add the tag to the list
	  found_open_tag = 0;
	  looking_for_tag_end = 0;
	}
	break;

      default:
	break;
    }
  }
  
  return DiskFileB::df_NO_ERROR;
}

int vbsHTML::ProcessHTMLTags()
// Read and process all the tags in a open file. 
// Returns a DiskFileB error code if an error occurs.
{
  num_processed = 0;

  int rv = df_Rewind(); // Start at the beginning of the file
  if(rv != DiskFileB::df_NO_ERROR) return rv;

  MemoryBuffer mbuf;
  list_ptr = tag_list.GetFront();

  while((!tag_list.IsHeader(list_ptr)) && (!df_EOF())) {
    mbuf.Clear(); // Clear the buffer before each read
    num_processed++;
    mbuf.resize(list_ptr->Data.tag_length);

    // Read the tag starting at the opening angle bracket
    df_StreamPos pos = list_ptr->Data.start_tag;
    rv = df_Read((char *)mbuf.m_buf(), mbuf.length(), pos);
    if(rv != DiskFileB::df_NO_ERROR) return rv;

    // Store the tag and indentify it
    mbuf[mbuf.length()] = '\0'; // Ensure null termination
    list_ptr->Data.tag_info = (char *)mbuf.m_buf();

    // Parse the tag type and attributes
    ParseHTMLTagInfo(list_ptr->Data);

    // Excute the derived class version of the tag handler
    HandleHTMLTag(list_ptr->Data.tag_id);
    
    list_ptr = list_ptr->GetNext();  
  }

  return DiskFileB::df_NO_ERROR;
}

void vbsHTML::ParseHTMLTagInfo(vbsHTMLTagInfo &t)
// Parse the specific tag infomation based on the
// string contained in the vbsHTMLTagInfo::tag_info
// member.
{
  // Replace all carriage returns and line feed with spaces
  // if the tag occupies one or more lines in the file.
  t.tag_info.ReplaceChar('\r', ' ');
  t.tag_info.ReplaceChar('\n', ' ');

  vbString sbuf(t.tag_info);

  // Remove the opening and closing angle brackets
  unsigned offset = sbuf.Find("<");
  if(offset != vbString::NoMatch) sbuf.DeleteAt(offset, 1);
  offset = sbuf.Find(">");
  if(offset != vbString::NoMatch) sbuf.DeleteAt(offset, 1);

  // Filter all leading and trailing spaces
  sbuf.TrimLeadingSpaces();
  sbuf.TrimTrailingSpaces();

  vbString tag(sbuf); vbString attr(sbuf);

  // Check to see if this the start or the end of a tag instruction
  if(tag[0] == '/') {
    tag.DeleteAt(0, 1); // Remove the foward slash
    t.start_instruction = 0;
    t.end_instruction = 1;
  }
  else {
    t.start_instruction = 1;
    t.end_instruction = 0;
  }

  // Check to see if this is comment
  offset = tag.Find(vbs_SUPPROTED_TAGS[vbs_comment_tag]);
  if(offset != vbString::NoMatch) { // This is comment string
    t.has_attributes = 0;
    t.tag_id = vbs_comment_tag;
    t.tag = vbs_SUPPROTED_TAGS[vbs_comment_tag];
    return;
  }
  
  // Check to see if this a special meaning tag
  offset = tag.Find("&");
  if(offset != vbString::NoMatch) {
    t.has_attributes = 0;
    t.tag = tag;

    // This is an unknown special meaning tag
    offset = tag.Find(";");
    if(offset != vbString::NoMatch) {
      t.tag_id = vbs_special_tag;
    }

    // Identify the most common special meaning tags
    offset = tag.Find(vbs_SUPPROTED_TAGS[vbs_less_then]);
    if(offset != vbString::NoMatch) {
      t.tag_id = vbs_less_then;
      t.tag = vbs_SUPPROTED_TAGS[vbs_less_then];
    }
    offset = tag.Find(vbs_SUPPROTED_TAGS[vbs_greater_then]);
    if(offset != vbString::NoMatch) {
      t.tag_id = vbs_greater_then;
      t.tag = vbs_SUPPROTED_TAGS[vbs_greater_then];
    }
    offset = tag.Find(vbs_SUPPROTED_TAGS[vbs_ampersand]);
    if(offset != vbString::NoMatch) {
      t.tag_id = vbs_ampersand;
      t.tag = vbs_SUPPROTED_TAGS[vbs_ampersand];
    }
    offset = tag.Find(vbs_SUPPROTED_TAGS[vbs_nb_space]);
    if(offset != vbString::NoMatch) {
      t.tag_id =  vbs_nb_space;
      t.tag = vbs_SUPPROTED_TAGS[vbs_nb_space];
    }
    offset = tag.Find(vbs_SUPPROTED_TAGS[vbs_quote]);
    if(offset != vbString::NoMatch) {
      t.tag_id = vbs_quote;
      t.tag = vbs_SUPPROTED_TAGS[vbs_quote];
    }
    
    // Indentify extended ASCII character tags
    offset = tag.Find(vbs_SUPPROTED_TAGS[vbs_ex_acsii_set]);
    if(offset != vbString::NoMatch) {
      t.tag_id = vbs_ex_acsii_set;
      t.tag = tag;
    }
    return;
  }

  // Separate the tag and its attributes
  offset = tag.Find(" "); 
  if(offset == vbString::NoMatch) { // This tag has no attributes
    t.tag = tag;
    t.tag_id = GetTagID(t.tag); // Identify the tag
    t.has_attributes = 0;

    // Covert the tag to upper case so that it matches
    // the strings in the supported tags array. 
    tag.ToUpper(); 
    return;
  }
  else {
    // Remove everything after the first space
    tag.DeleteAt(offset, (tag.length() - offset));
    t.tag = tag;
    t.tag_id = GetTagID(t.tag); // Identify the tag

    // Covert the tag to upper case so that it matches
    // the strings in the supported tags array. 
    tag.ToUpper(); 
  }

  // Look for the tag's attribute
  offset = attr.Find(" ");
  if(offset != vbString::NoMatch) {
    // Remove everything before the first space
    attr.DeleteAt(0, offset+1);
    t.attr = attr;

    // Trim any spaces that may have been inserted over linefeeds
    t.attr.TrimLeadingSpaces();
    t.has_attributes = 1;
  }
}

void vbsHTML::Copy(const vbsHTML &ob)
// Member function used during copying and assignment.
{
  tag_list = ob.tag_list;
  num_tags = ob.num_tags;
  num_processed = ob.num_processed;
  list_ptr = ob.list_ptr;
}

// Derived class interface used to process tags
void vbsHTML::Handle_INVALID_Tag()
{
  // Override to handle INVALID tags
}

void vbsHTML::Handle_UNKNOWN_Tag()
{
  // Override to handle UNKNOWN tags
}

void vbsHTML::Handle_UNKNOWN_SPECIAL_Tag()
{
  // Override to handle unknown special tags starting 
  // with an ampersand ending in a semicolon &xxxx;
}

void vbsHTML::Handle_COMMENT_Tag()
{
  // Override to handle COMMENT tags
}


void vbsHTML::Handle_LESS_THEN_Tag()
{
  // Override to handle a less than sign "&lt;" 
}

void vbsHTML::Handle_GREATER_THEN_Tag() 
{
  // Override to handle a greater than sign "&gt;"
}

void vbsHTML::Handle_AMPERSAND_Tag()
{
  // Override to handle an ampersand "&amp;" 
}

void vbsHTML::Handle_NB_SPACE_Tag()
{
  // Override to handle a non-breaking space "&nbsp;"
}

void vbsHTML::Handle_QUOTE_Tag() 
{
  // Override to handle a quotation mark "&quot;" 
}

void vbsHTML::Handle_EX_ASCII_Tag()
{
  // Override to handle the extended ASCII character set "&#"
}

void vbsHTML::Handle_A_Tag()
{
  // Override to handle ANCHOR tags
}

void vbsHTML::Handle_ABBREV_Tag()
{
  // Override to handle ABBREVIATION tags
}

void vbsHTML::Handle_ACRONYM_Tag()
{
  // Override to handle ACRONYM tags
}

void vbsHTML::Handle_ADDRESS_Tag()
{
  // Override to handle ADDRESS tags
}

void vbsHTML::Handle_APPLET_Tag()
{
  // Override to handle JAVA APPLET tag
}

void vbsHTML::Handle_AREA_Tag()
{
  // Override to handle AREA tags
}

void vbsHTML::Handle_AU_Tag()
{
  // Override to handle AUTHOR tags
}

void vbsHTML::Handle_AUTHOR_Tag()
{
  // Override to handle AUTHOR tags
}

void vbsHTML::Handle_B_Tag()
{
  // Override to handle BOLD tags
}

void vbsHTML::Handle_BANNER_Tag()
{
  // Override to handle BANNER tags
}

void vbsHTML::Handle_BASE_Tag()
{
  // Override to handle BASE tags
}

void vbsHTML::Handle_BASEFONT_Tag()
{
  // Override to handle BASE FONT
}

void vbsHTML::Handle_BGSOUND_Tag()
{
  // Override to handle BACKGROUND SOUND
}

void vbsHTML::Handle_BIG_Tag()
{
  // Override to handle BIG text
}

void vbsHTML::Handle_BLINK_Tag()
{
  // Override to handle BLINK tags
}

void vbsHTML::Handle_BLOCKQUOTE_Tag()
{
  // Override to handle BLOCK QUOTE tags
}

void vbsHTML::Handle_BQ_Tag()
{
  // Override to handle BLOCK QUOTE tags
}

void vbsHTML::Handle_BODY_Tag()
{
  // Override to handle BODY tags
}

void vbsHTML::Handle_BR_Tag()
{
  // Override to handle LINE BREAK tags
}

void vbsHTML::Handle_CAPTION_Tag()
{
  // Override to handle CAPTION tags
}

void vbsHTML::Handle_CENTER_Tag()
{
  // Override to handle CENTER tags
}

void vbsHTML::Handle_CITE_Tag()
{
  // Override to handle CITATION tags
}

void vbsHTML::Handle_CODE_Tag()
{
  // Override to handle CODE tags
}

void vbsHTML::Handle_COL_Tag()
{
  // Override to handle TABLE Cols tags
}

void vbsHTML::Handle_COLGROUP_Tag()
{
  // Override to handle TABLE Cols tags
}

void vbsHTML::Handle_CREDIT_Tag()
{
  // Override to handle CREDIT tags
}

void vbsHTML::Handle_DEL_Tag()
{
  // Override to handle DELETED text tags
}

void vbsHTML::Handle_DFN_Tag()
{
  // Override to handle DEFINITION tags
}

void vbsHTML::Handle_DIR_Tag()
{
  // Override to handle DIRECTORY list tags
}

void vbsHTML::Handle_DIV_Tag()
{
  // Override to handle DIVISION tags
}

void vbsHTML::Handle_DL_Tag()
{
  // Override to handle DEFINITION list tags
}

void vbsHTML::Handle_DT_Tag()
{
  // Override to handle DEFINITION term tags
}

void vbsHTML::Handle_DD_Tag()
{
  // Override to handle DEFINITION tags
}

void vbsHTML::Handle_EM_Tag()
{
  // Override to handle EMPHASIZED tags
}

void vbsHTML::Handle_EMBED_Tag()
{
  // Override to handle EMBED tags
}

void vbsHTML::Handle_FIG_Tag()
{
  // Override to handle FIGURE tags
}

void vbsHTML::Handle_FN_Tag()
{
  // Override to handle FOOTNOTE tags
}

void vbsHTML::Handle_FONT_Tag()
{
  // Override to handle FONT tags
}

void vbsHTML::Handle_FORM_Tag()
{
  // Override to handle FORM tags
}

void vbsHTML::Handle_FRAME_Tag()
{
  // Override to handle FRAME tags
}

void vbsHTML::Handle_FRAMESET_Tag()
{
  // Override to handle FRAME sets
}

void vbsHTML::Handle_H1_Tag()
{
  // Override to handle HEADING 1 tags
}

void vbsHTML::Handle_H2_Tag()
{
  // Override to handle HEADING 2 tags
}

void vbsHTML::Handle_H3_Tag()
{
  // Override to handle HEADING 3 tags
}

void vbsHTML::Handle_H4_Tag()
{
  // Override to handle HEADING 4 tags
}

void vbsHTML::Handle_H5_Tag()
{
  // Override to handle HEADING 5 tags
}

void vbsHTML::Handle_H6_Tag()
{
  // Override to handle HEADING 6 tags
}

void vbsHTML::Handle_HEAD_Tag()
{
  // Override to handle HEAD tags
}

void vbsHTML::Handle_HR_Tag()
{
  // Override to handle HORIZONTAL rules
}

void vbsHTML::Handle_HTML_Tag()
{
  // Override to handle HTML tags
}

void vbsHTML::Handle_I_Tag()
{
  // Override to handle ITALIC tags
}

void vbsHTML::Handle_IFRAME_Tag()
{
  // Override to handle FRAME - Floating tag
}

void vbsHTML::Handle_IMG_Tag()
{
  // Override to handle INLINE images
}

void vbsHTML::Handle_INPUT_Tag()
{
  // Override to handle FORM input tags
}

void vbsHTML::Handle_INS_Tag()
{
  // Override to handle INSERTED text
}

void vbsHTML::Handle_ISINDEX_Tag()
{
  // Override to handle ISINDEX tag
}

void vbsHTML::Handle_KBD_Tag()
{
  // Override to handle KEYBOARD tags
}

void vbsHTML::Handle_LANG_Tag()
{
  // Override to handle LANGUAGE tags
}

void vbsHTML::Handle_LH_Tag()
{
  // Override to handle LIST header tags
}

void vbsHTML::Handle_LI_Tag()
{
  // Override to handle LIST item tags
}

void vbsHTML::Handle_LINK_Tag()
{
  // Override to handle LINK tags
}

void vbsHTML::Handle_LISTING_Tag()
{
  // Override to handle LISTING tags
}

void vbsHTML::Handle_MAP_Tag()
{
  // Override to handle MAP tags
}

void vbsHTML::Handle_MARQUEE_Tag()
{
  // Override to handle MARQUEE tags
}

void vbsHTML::Handle_MATH_Tag()
{
  // Override to handle MATH tags
}

void vbsHTML::Handle_MENU_Tag()
{
  // Override to handle MENU list tags
}

void vbsHTML::Handle_META_Tag()
{
  // Override to handle META tags
}

void vbsHTML::Handle_MULTICOL_Tag()
{
  // Override to handle MULTI COLUMN tags
}

void vbsHTML::Handle_NOBR_Tag()
{
  // Override to handle NO BREAK tags
}

void vbsHTML::Handle_NOFRAMES_Tag()
{
  // Override to handle NO FRAMES tags
}

void vbsHTML::Handle_NOTE_Tag()
{
  // Override to handle NOTE tags
}

void vbsHTML::Handle_OL_Tag()
{
  // Override to handle ORDERED list tags
}

void vbsHTML::Handle_OVERLAY_Tag()
{
  // Override to handle OVERLAY tags
}

void vbsHTML::Handle_P_Tag()
{
  // Override to handle PARAGRAPH tags
}

void vbsHTML::Handle_PARAM_Tag()
{
  // Override to handle PARAMETERS tags
}

void vbsHTML::Handle_PERSON_Tag()
{
  // Override to handle PERSON tags
}

void vbsHTML::Handle_PLAINTEXT_Tag()
{
  // Override to handle PLAIN text tags
}

void vbsHTML::Handle_PRE_Tag()
{
  // Override to handle PREFORMATTED text tags
}

void vbsHTML::Handle_Q_Tag()
{
  // Override to handle QUOTE tags
}

void vbsHTML::Handle_RANGE_Tag()
{
  // Override to handle RANGE tags
}

void vbsHTML::Handle_SAMP_Tag()
{
  // Override to handle SAMPLE tags
}

void vbsHTML::Handle_SCRIPT_Tag()
{
  // Override to handle SCRIPT tags
}

void vbsHTML::Handle_SELECT_Tag()
{
  // Override to handle FORM SELECT tags
}

void vbsHTML::Handle_SMALL_Tag()
{
  // Override to handle SMALL text tags
}

void vbsHTML::Handle_SPACER_Tag()
{
  // Override to handle WHITE SPACE tags
}

void vbsHTML::Handle_SPOT_Tag()
{
  // Override to handle SPOT tags
}

void vbsHTML::Handle_STRIKE_Tag()
{
  // Override to handle STRIKETHROUGH tags
}

void vbsHTML::Handle_STRONG_Tag()
{
  // Override to handle STRONG tags
}

void vbsHTML::Handle_SUB_Tag()
{
  // Override to handle SUBSCRIPT tags
}

void vbsHTML::Handle_SUP_Tag()
{
  // Override to handle SUPERSCRIPT tags
}

void vbsHTML::Handle_TAB_Tag()
{
  // Override to handle HORIZONTAL TABS tags
}

void vbsHTML::Handle_TABLE_Tag()
{
  // Override to handle TABLE tags
}

void vbsHTML::Handle_TBODY_Tag()
{
  // Override to handle TABLE body tags
}

void vbsHTML::Handle_TD_Tag()
{
  // Override to handle TABLE data tags
}

void vbsHTML::Handle_TEXTAREA_Tag()
{
  // Override to handle FORM form tags
}

void vbsHTML::Handle_TEXTFLOW_Tag()
{
  // Override to handle JAVA applet textflow
}

void vbsHTML::Handle_TFOOT_Tag()
{
  // Override to handle TABLE footer tags
}

void vbsHTML::Handle_TH_Tag()
{
  // Override to handle TABLE head
}

void vbsHTML::Handle_THEAD_Tag()
{
  // Override to handle TABLE head tag
}

void vbsHTML::Handle_TITLE_Tag()
{
  // Override to handle TITLE tags
}

void vbsHTML::Handle_TR_Tag()
{
  // Override to handle TABLE row tags 
}

void vbsHTML::Handle_TT_Tag()
{
  // Override to handle TELETYPE tags
}

void vbsHTML::Handle_U_Tag()
{
  // Override to handle UNDERLINED tags
}

void vbsHTML::Handle_UL_Tag()
{
  // Override to handle UNORDERED list tags
}

void vbsHTML::Handle_VAR_Tag()
{
  // Override to handle VARIABLE tags 
}

void vbsHTML::Handle_WBR_Tag()
{
  // Override to handle WORD BREAK tags
}

void vbsHTML::Handle_XMP_Tag()
{
  // Override to handle EXAMPLE tags
}

void vbsHTML::HandleHTMLTag(int tag_id)
// Routine used to execute the derived class version
// of a specific tag handler.
{
  switch(tag_id) {
    case vbs_invalid_tag :    // Invalid tag specified
      Handle_INVALID_Tag();
      break;
      
    case vbs_unknown_tag :    // Unknown tag specified
      Handle_UNKNOWN_Tag();
      break;

    case vbs_special_tag :    // Unknown special meaning tag
      Handle_UNKNOWN_SPECIAL_Tag();
      break;
      
    case vbs_comment_tag :     // Comments
      Handle_COMMENT_Tag();
      break;
      
    case vbs_less_then :       // Less than sign "&lt;" 
      Handle_LESS_THEN_Tag();
      break;

    case vbs_greater_then :    // Greater then sign "&gt;"
      Handle_GREATER_THEN_Tag();
      break;

    case vbs_ampersand :       // Ampersand "&amp;"
      Handle_AMPERSAND_Tag();
      break;

    case vbs_nb_space :        // Non-breaking space "&nbsp;"
      Handle_NB_SPACE_Tag();
      break;

    case vbs_quote :           // Quotation mark "&quot;"
      Handle_QUOTE_Tag();
      break;

    case vbs_ex_acsii_set :    // Extended ASCII character set
      Handle_EX_ASCII_Tag();
      break;

    case vbs_a_tag :           // anchor
      Handle_A_Tag();
      break;
      
    case vbs_abbrev_tag :      // abbreviation
      Handle_ABBREV_Tag();
      break;
      
    case vbs_acronym_tag :     // acronym
      Handle_ACRONYM_Tag();
      break;
      
    case vbs_address_tag :     // address
      Handle_ADDRESS_Tag();
      break;
      
    case vbs_applet_tag :      // java applet
      Handle_APPLET_Tag();
      break;
      
    case vbs_area_tag :        // area
      Handle_AREA_Tag();
      break;
      
    case vbs_au_tag :          // author
      Handle_AU_Tag();
      break;
      
    case vbs_author_tag :      // author
      Handle_AUTHOR_Tag();
      break;
      
    case vbs_b_tag :           // bold
      Handle_B_Tag();
      break;
      
    case vbs_banner_tag :      // banner
      Handle_BANNER_Tag();
      break;
      
    case vbs_base_tag :        // base
      Handle_BASE_Tag();
      break;
      
    case vbs_basefont_tag :    // base font
      Handle_BASEFONT_Tag();
      break;
      
    case vbs_bgsound_tag :     // background sound
      Handle_BGSOUND_Tag();
      break;
      
    case vbs_big_tag :         // big text
      Handle_BIG_Tag();
      break;
      
    case vbs_blink_tag :       // blink
      Handle_BLINK_Tag();
      break;
      
    case vbs_blockquote_tag :  // block quote
      Handle_BLOCKQUOTE_Tag();
      break;
      
    case vbs_bq_tag :          // block quote
      Handle_BQ_Tag();
      break;
      
    case vbs_body_tag :        // body
      Handle_BODY_Tag();
      break;
      
    case vbs_br_tag :          // line break
      Handle_BR_Tag();
      break;
      
    case vbs_caption_tag :     // caption
      Handle_CAPTION_Tag();
      break;
      
    case vbs_center_tag :      // center
      Handle_CENTER_Tag();
      break;
      
    case vbs_cite_tag :        // citation
      Handle_CITE_Tag();
      break;
      
    case vbs_code_tag :        // code
      Handle_CODE_Tag();
      break;
      
    case vbs_col_tag :         // table column
      Handle_COL_Tag();
      break;
      
    case vbs_colgroup_tag :    // table column group
      Handle_COLGROUP_Tag();
      break;
      
    case vbs_credit_tag :      // credit
      Handle_CREDIT_Tag();
      break;
      
    case vbs_del_tag :         // deleted text
      Handle_DEL_Tag();
      break;
      
    case vbs_dfn_tag :         // definition
      Handle_DFN_Tag();
      break;
      
    case vbs_dir_tag :         // directory list
      Handle_DIR_Tag();
      break;
      
    case vbs_div_tag :         // division
      Handle_DIV_Tag();
      break;
      
    case vbs_dl_tag :          // definition list
      Handle_DL_Tag();
      break;
      
    case vbs_dt_tag :          // definition term
      Handle_DT_Tag();
      break;
      
    case vbs_dd_tag :          // definition definition
      Handle_DD_Tag();
      break;
      
    case vbs_em_tag :          // emphasized
      Handle_EM_Tag();
      break;
      
    case vbs_embed_tag :       // embed
      Handle_EMBED_Tag();
      break;
      
    case vbs_fig_tag :         // figure
      Handle_FIG_Tag();
      break;
      
    case vbs_fn_tag :          // footnote
      Handle_FN_Tag();
      break;
      
    case vbs_font_tag :        // font
      Handle_FONT_Tag();
      break;
      
    case vbs_form_tag :        // form
      Handle_FORM_Tag();
      break;
      
    case vbs_frame_tag :       // frame
      Handle_FRAME_Tag();
      break;
      
    case vbs_frameset_tag :    // frame set
      Handle_FRAMESET_Tag();
      break;
      
    case vbs_h1_tag :          // heading 1
      Handle_H1_Tag();
      break;
      
    case vbs_h2_tag :          // heading 2
      Handle_H2_Tag();
      break;
      
    case vbs_h3_tag :          // heading 3
      Handle_H3_Tag();
      break;
      
    case vbs_h4_tag :          // heading 4
      Handle_H4_Tag();
      break;
      
    case vbs_h5_tag :          // heading 5
      Handle_H5_Tag();
      break;
      
    case vbs_h6_tag :          // heading 6
      Handle_H6_Tag();
      break;
      
    case vbs_head_tag :        // head
      Handle_HEAD_Tag();
      break;
      
    case vbs_hr_tag :          // horizontal rule
      Handle_HR_Tag();
      break;
      
    case vbs_html_tag :        // html
      Handle_HTML_Tag();
      break;
      
    case vbs_i_tag :           // italic
      Handle_I_Tag();
      break;
      
    case vbs_iframe_tag :      // frame - floating
      Handle_IFRAME_Tag();
      break;
      
    case vbs_img_tag :         // inline image
      Handle_IMG_Tag();
      break;
      
    case vbs_input_tag :       // form input
      Handle_INPUT_Tag();
      break;
      
    case vbs_ins_tag :         // inserted text
      Handle_INS_Tag();
      break;
      
    case vbs_isindex_tag :     // is index
      Handle_ISINDEX_Tag();
      break;
      
    case vbs_kbd_tag :         // keyboard
      Handle_KBD_Tag();
      break;
      
    case vbs_lang_tag :        // language
      Handle_LANG_Tag();
      break;
      
    case vbs_lh_tag :          // list heading
      Handle_LH_Tag();
      break;
      
    case vbs_li_tag :          // list item
      Handle_LI_Tag();
      break;
      
    case vbs_link_tag :        // link
      Handle_LINK_Tag();
      break;
      
    case vbs_listing_tag :     // listing
      Handle_LISTING_Tag();
      break;
      
    case vbs_map_tag :         // map
      Handle_MAP_Tag();
      break;
      
    case vbs_marquee_tag :     // marquee
      Handle_MARQUEE_Tag();
      break;
      
    case vbs_math_tag :        // math
      Handle_MATH_Tag();
      break;
      
    case vbs_menu_tag :        // menu list
      Handle_MENU_Tag();
      break;
      
    case vbs_meta_tag :        // meta
      Handle_META_Tag();
      break;
      
    case vbs_multicol_tag :    // multi column text
      Handle_MULTICOL_Tag();
      break;
      
    case vbs_nobr_tag :        // no break
      Handle_NOBR_Tag();
      break;
      
    case vbs_noframes_tag :    // no frames
      Handle_NOFRAMES_Tag();
      break;
      
    case vbs_note_tag :        // note
      Handle_NOTE_Tag();
      break;
      
    case vbs_ol_tag :          // ordered list
      Handle_OL_Tag();
      break;
      
    case vbs_overlay_tag :     // overlay
      Handle_OVERLAY_Tag();
      break;
      
    case vbs_p_tag :           // paragraph
      Handle_P_Tag();
      break;
      
    case vbs_param_tag :       // parameters
      Handle_PARAM_Tag();
      break;
      
    case vbs_person_tag :      // person
      Handle_PERSON_Tag();
      break;
      
    case vbs_plaintext_tag :   // plain text
      Handle_PLAINTEXT_Tag();
      break;
      
    case vbs_pre_tag :         // preformatted text
      Handle_PRE_Tag();
      break;
      
    case vbs_q_tag :           // quote
      Handle_Q_Tag();
      break;
      
    case vbs_range_tag :       // range
      Handle_RANGE_Tag();
      break;
      
    case vbs_samp_tag :        // sample
      Handle_SAMP_Tag();
      break;
      
    case vbs_script_tag :      // script
      Handle_SCRIPT_Tag();
      break;
      
    case vbs_select_tag :      // form select
      Handle_SELECT_Tag();
      break;
      
    case vbs_small_tag :       // small text
      Handle_SMALL_Tag();
      break;
      
    case vbs_spacer_tag :      // white space
      Handle_SPACER_Tag();
      break;
      
    case vbs_spot_tag :        // spot
      Handle_SPOT_Tag();
      break;
      
    case vbs_strike_tag :      // strikethrough
      Handle_STRIKE_Tag();
      break;
      
    case vbs_strong_tag :      // strong
      Handle_STRONG_Tag();
      break;
      
    case vbs_sub_tag :         // subscript
      Handle_SUB_Tag();
      break;
      
    case vbs_sup_tag :         // superscript
      Handle_SUP_Tag();
      break;
      
    case vbs_tab_tag :         // horizontal tab
      Handle_TAB_Tag();
      break;
      
    case vbs_table_tag :       // table
      Handle_TABLE_Tag();
      break;
      
    case vbs_tbody_tag :       // table body
      Handle_TBODY_Tag();
      break;
      
    case vbs_td_tag :          // table data
      Handle_TD_Tag();
      break;
      
    case vbs_textarea_tag :    // form text area
      Handle_TEXTAREA_Tag();
      break;
      
    case vbs_textflow_tag :    // java applet textflow
      Handle_TEXTFLOW_Tag();
      break;
      
    case vbs_tfoot_tag :       // table footer
      Handle_TFOOT_Tag();
      break;
      
    case vbs_th_tag :          // table header
      Handle_TH_Tag();
      break;
      
    case vbs_thead_tag :       // table head
      Handle_THEAD_Tag();
      break;
      
    case vbs_title_tag :       // title
      Handle_TITLE_Tag();
      break;
      
    case vbs_tr_tag :          // table row
      Handle_TR_Tag();
      break;
      
    case vbs_tt_tag :          // teletype
      Handle_TT_Tag();
      break;
      
    case vbs_u_tag :           // underlined
      Handle_U_Tag();
      break;
      
    case vbs_ul_tag :          // unordered list
      Handle_UL_Tag();
      break;
      
    case vbs_var_tag :         // variable
      Handle_VAR_Tag();
      break;
      
    case vbs_wbr_tag :         // word break
      Handle_WBR_Tag();
      break;
      
    case vbs_xmp_tag :         // example
      Handle_XMP_Tag();
      break;

    default :
      Handle_INVALID_Tag();
      break;
  }
}

int vbsHTML::LoadMemoryBuffer(const MemoryBuffer &membuf)
{
  int rv = CollectHTMLTags(membuf);
  if(rv) return rv;

  rv = ProcessHTMLTags(membuf);
  if(rv) return rv;

  return 0;
}

int vbsHTML::CollectHTMLTags(const MemoryBuffer &membuf)
{
  int i;
  num_tags = 0;
  int found_open_tag = 0;
  int looking_for_tag_end = 0;
  char prev_tag;
  
  tag_list.Clear(); // Clear the tag list
  char *start = (char *)membuf.m_buf();
  unsigned len = membuf.length();

  vbsHTMLTagInfo tag;
  char c; unsigned pos;
  for(pos = 0; pos < len; pos++, start++) {
    c = *start;
    switch(c) {
      case '<' : case '&' :
	if(c == '<') prev_tag = '<'; else prev_tag = '&';
	if(!looking_for_tag_end) { 
	  found_open_tag = 1;
	  looking_for_tag_end = 1;
	  tag.start_tag = pos;
	  if(tag.start_tag > (df_StreamPos)0) tag.start_tag--;
	}
	break;

      case '#' : // Look for extended ASCII characters denoted by: "&#xxx"
	if((prev_tag == '&') && (tag.start_tag = (pos - 1))) {
	  if(found_open_tag) { // Look for the end of a tag
	    // Get the next three numeric characters following a "&#" tag
	    // plus the terminating semicolon or space.
	    for(i = 0; i < 4; i++, start++, pos++) { 
	      c = *start;
	      if((c == ';') || (c == ' ')) break;
	    }
	    tag.start_tag--; // Account for the ampersand
	    tag.end_tag = pos;
	    num_tags++;
	    
	    // Calculate the length of the tag including the angle brackets
	    tag.tag_length = tag.end_tag - tag.start_tag;
	    
	    tag_list.StoreNode(tag); // Add the tag to the list
	    found_open_tag = 0;
	    looking_for_tag_end = 0;
	  }
	} 
	break;

      case '>' : case  ';' :
	if(c == '>') prev_tag = '>'; else prev_tag = ';';
	if(found_open_tag) { // Look for the end of a tag
	  tag.end_tag = pos;
	  num_tags++;
      
	  // Calculate the length of the tag including the angle brackets
	  tag.tag_length = tag.end_tag - tag.start_tag;
      
	  tag_list.StoreNode(tag); // Add the tag to the list
	  found_open_tag = 0;
	  looking_for_tag_end = 0;
	}
	break;

      default:
	break;
    }
  }
  
  return 0;
}

int vbsHTML::ProcessHTMLTags(const MemoryBuffer &membuf)
// Read and process all the tags in a open file. 
{
  num_processed = 0;
  
  MemoryBuffer mbuf;
  list_ptr = tag_list.GetFront();
  
  char *start = (char *)membuf.m_buf();
  char *tag_pos;
  
  while(!tag_list.IsHeader(list_ptr)) {
    mbuf.Clear(); // Clear the buffer before each read
    mbuf.resize(list_ptr->Data.tag_length);

    // Read the tag starting at the opening angle bracket
    unsigned pos = list_ptr->Data.start_tag;
    tag_pos = start + pos;
    memmove((char *)mbuf.m_buf(), tag_pos, mbuf.length());  
    
    // Store the tag and indentify it
    mbuf[mbuf.length()] = '\0'; // Ensure null termination
    list_ptr->Data.tag_info = (char *)mbuf.m_buf();

    // Parse the tag type and attributes
    ParseHTMLTagInfo(list_ptr->Data);

    // Excute the derived class version of the tag handler
    HandleHTMLTag(list_ptr->Data.tag_id);

    num_processed++;
    list_ptr = list_ptr->GetNext();  
  }

  return 0;
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
