// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: vbshttp.cpp
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 01/25/2000
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

HTTP client/server constants and data structures shared between
HTTP client and server applications.
*/
// ----------------------------------------------------------- // 
#include "vbshttp.h"


// HTTP connection methods
const char *vbsHTTP_METHOD[vbsMAX_HTTP_METHODS] = {
  "UNKNOWN",    // String used for internal processing
  "GET",        // Retrieve whatever data is identified by the URI.
  "HEAD",       // Returns only HTTP headers and no document body. 
  "PUT",        // Store the data under the supplied URL.
  "DELETE",     // Delete the information corresponding to the given URL.
  "POST",       // Creates a new object linked to the specified object. 
  "LINK",       // Links an existing object to the specified object. 
  "UNLINK"      // Removes link (or other meta-) information from an object. 
};

// HTTP status messages
const char *vbsHTTP_STATUS_CODES[vbsMAX_HTTP_STATUS_CODES] = {
  "Unknown HTTP status code",              // Used for internal processing
  "200 HTTP status: Document OK",                       // OK
  "201 HTTP status: Document created",                  // CREATED
  "202 HTTP status: Document accepted",                 // ACCEPTED
  "204 HTTP status: Document has no content",           // NOCONTENT
  "300 HTTP status: Multiple choices",                  // MULTIPLE_CHOICES
  "301 HTTP status: Document has moved permanently",    // MOVED_PERMANENTLY
  "302 HTTP status: Document has moved temporarily",    // MOVED_TEMPORARILY
  "304 HTTP status: Document has not been modified",    // NOT_MODIFIED
  "400 HTTP client error: Bad client request",          // BAD_REQUEST
  "401 HTTP client error: Unauthorized client request", // UNAUTHORIZED
  "403 HTTP client error: Forbidden client request",    // FORBIDDEN
  "404 HTTP client error: Document not found",          // NOT_FOUND
  "500 HTTP server error: Internal server error",       // INTERNAL
  "501 HTTP server error: Service not implemented",     // NOT_IMPLEMENTED
  "502 HTTP server error: Bad gateway or proxy",        // BAD_GATEWAY
  "503 HTTP server error: Status unavailable"           // UNAVAILABLE
};

const char *vbsHTTPMethodMessage(int method)
{
  if(method > vbsMAX_HTTP_METHODS) method = vbHTTP_UNKNOWN;
  return vbsHTTP_METHOD[method];
}

const char *vbsHTTPStatusCodeMessage(int status)
{
  switch(status) {
    
    case vbsHTTP_STATUS_OK:                 // 200
      return vbsHTTP_STATUS_CODES[1];
      
    case vbsHTTP_STATUS_CREATED :           // 201
      return vbsHTTP_STATUS_CODES[2];
      
    case vbsHTTP_STATUS_ACCEPTED :          // 202
      return vbsHTTP_STATUS_CODES[3];
      
    case vbsHTTP_STATUS_NO_CONTENT :        // 204
      return vbsHTTP_STATUS_CODES[4];
      
    case vbsHTTP_STATUS_MULTIPLE_CHOICES :  // 300
      return vbsHTTP_STATUS_CODES[5];
      
    case vbsHTTP_STATUS_MOVED_PERMANENTLY : // 301
      return vbsHTTP_STATUS_CODES[6];
      
    case vbsHTTP_STATUS_MOVED_TEMPORARILY : // 302
      return vbsHTTP_STATUS_CODES[7];
      
    case vbsHTTP_STATUS_NOT_MODIFIED :      // 304
      return vbsHTTP_STATUS_CODES[8];
      
    case vbsHTTP_STATUS_BAD_REQUEST :       // 400
      return vbsHTTP_STATUS_CODES[9];
      
    case vbsHTTP_STATUS_UNAUTHORIZED :      // 401
      return vbsHTTP_STATUS_CODES[10];
      
    case vbsHTTP_STATUS_FORBIDDEN :         // 403
      return vbsHTTP_STATUS_CODES[11];
      
    case vbsHTTP_STATUS_NOT_FOUND :         // 404
      return vbsHTTP_STATUS_CODES[12];
      
    case vbsHTTP_STATUS_INTERNAL :          // 500
      return vbsHTTP_STATUS_CODES[13];
      
    case vbsHTTP_STATUS_NOT_IMPLEMENTED :   // 501
      return vbsHTTP_STATUS_CODES[14];
      
    case vbsHTTP_STATUS_BAD_GATEWAY :       // 502
      return vbsHTTP_STATUS_CODES[15];
      
    case vbsHTTP_STATUS_UNAVAILABLE :       // 503
      return vbsHTTP_STATUS_CODES[16];
      
    default:
      break;
  }
  return vbsHTTP_STATUS_CODES[0];
}

vbsNetscapeCookie::vbsNetscapeCookie()
{
  host = name = value = expires = domain = path = "\0";
  secure = 0;
}

vbsNetscapeCookie::~vbsNetscapeCookie()
{

}

void vbsNetscapeCookie::Copy(const vbsNetscapeCookie &ob)
{
  expires = ob.expires;
  domain = ob.domain;
  path = ob.path;
  host = ob.host;
  name = ob.name;
  value = ob.value;
  secure = ob.secure;
}

int operator==(const vbsNetscapeCookie &a, const vbsNetscapeCookie &b)
// Overloaded == operator added to work with vbs linked list class.
{
  if(a.expires != b.expires) return 0;
  if(a.domain != b.domain) return 0;
  if(a.path != b.path) return 0;
  if(a.host != b.host) return 0;
  if(a.name != b.name) return 0;
  if(a.value != b.value) return 0;
  if(a.secure != b.secure) return 0;
  return 1;
}

vbsHTTPHeader::vbsHTTPHeader() {
  http_header = authentication_scheme = realm = auth_cookie = \
    current_server = location = http_last_modified =  date = \
    http_expires = file_extension = content_type = mime_type = \
    etag = content_encoding = pragma = cache_control = "\0";
  
  // Default settings
  http_version = 1.0F;
  http_status = vbsHTTP_STATUS_OK;
  authentication_needed = 0;
  not_found = 0;
  keep_alive = 0;
  timeout = -1;   // -1 indicates that this value has not been set
  max_conns = -1; // -1 indicates that this value has not been set
  length = -1;    // -1 indicates that this value has not been set
  no_cache = 0;
  accept_ranges = 0;
  use_cookies = 1;
}

vbsHTTPHeader::~vbsHTTPHeader()
{

}

void vbsHTTPHeader::Copy(const vbsHTTPHeader &ob)
{
  http_header = ob.http_header;
  authentication_scheme = ob.authentication_scheme;
  realm = ob.realm;
  current_server = ob.current_server;
  location = ob.location;
  http_last_modified = ob.http_last_modified;
  date = ob.date;
  http_expires = ob.http_expires;
  file_extension = ob.file_extension;
  content_type = ob.content_type;
  mime_type = ob.mime_type;
  etag = ob.etag;
  pragma = ob.pragma;
  cache_control = ob.cache_control;
  content_encoding = ob.content_encoding;
  http_version = ob.http_version;
  http_status = ob.http_status;
  authentication_needed = ob.authentication_needed;
  not_found = ob.not_found;
  keep_alive = ob.keep_alive;
  timeout = ob.timeout;
  max_conns = ob.max_conns;
  length = ob.length;
  no_cache = ob.no_cache;
  accept_ranges = ob.accept_ranges;
  auth_cookie = ob.auth_cookie;
  use_cookies = ob.use_cookies;
  netscape_cookies = ob.netscape_cookies;
}

int operator==(const vbsHTTPHeader &a, const vbsHTTPHeader &b)
{
  if(a.http_header != b.http_header) return 0;
  return 1;
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
