// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: vbstream.cpp
// C++ Compiler Used: MSVC40, HP CPP 10.24, egcs-2.90.29 
// Produced By: Doug Gaer   
// File Creation Date: 09/20/1999
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbStream class is used to stream variable blocks from a 
client to a server over a TCP/IP connection.
*/
// ----------------------------------------------------------- // 
#include <string.h>
#include "vbstream.h"

// TCP/IP Stream functions
// --------------------------------------------------------------
int vbStream::StreamServer(int port, int max_connections)
// Initialize a stream server. Please note all ports below 1024 are
// reserved. Returns a non-zero value if any errors occur during
// initialization. The "max_connections" variable determines how many
// pending connections the queue will hold.
{
  if(InitSocketLibrary() == 0) {
    if(InitSocket(SOCK_STREAM, port) < 0) return socket_error;
  }
  else {
    return socket_error;
  }


  // Bind the name to the socket
  if(Bind() < 0) {
    Close();
    return socket_error;
  }

  // Listen for connections with a specified backlog
  if(Listen(max_connections) < 0) {
    Close();
    return socket_error;
  }

  return socket_error = vbSOCKET_NO_ERROR;
}

int vbStream::StreamClient(int port, char *host)
// Connect a stream client. Please note all ports below 1024 are
// reserved. Returns zero if no errors occur during initialization.
{
  if(InitSocketLibrary() == 0) {
    if(InitSocket(SOCK_STREAM, port, host) < 0) return socket_error;
  }
  else {
    return socket_error;
  }

  // Connect to the server
  if(Connect() < 0) {
    return socket_error;
  }

  return socket_error = vbSOCKET_NO_ERROR;  
}

void vbStream::SetBlockStatus(vbBlockHeader &vb, __SBYTE__ net_status,
			      __SBYTE__ file_status)
// Set the variable block header status member control byte.
// NOTE: The block's next deleted variable must be set, if
// needed, after a call to this function. The block length
// will not be set here.
{
  __ULWORD__ block_status = net_status;
  block_status = (block_status<<8) & 0xff00;
  block_status += file_status;
  vb.Status = block_status;
  vb.CkWord = vbCheckWord;
  vb.NextDeletedVB = (FAU)0;
}

int vbStream::WriteBlock(vbsSocket_t s, const void *buf, int bytes)
// Write a block of raw data to the socket. NOTE: All block read and write
// operations must operate in blocking mode to maintain synchronization
// between the transmitter and receiver. Returns zero if no errors
// occur.
{
  vbBlockHeader vb;
  SetBlockStatus(vb, vbSendBlock);
  vb.Length = bytes;

  // Write the block header
  if(WriteBlock(s, buf, vb) != vbSOCKET_NO_ERROR) return socket_error;
  
  return socket_error = vbSOCKET_NO_ERROR;  
}

int vbStream::WriteBlock(vbsSocket_t s, const void *buf, vbBlockHeader &vb)
// Write a variable block to the specified socket. Returns zero if no
// errors occur. NOTE: This function assumes that the block header has
// already been initialized.
{
  // Write the block header
  if(WriteHeader(s, vb) != vbSOCKET_NO_ERROR) return socket_error;

  // Write the block
  if(Send(s, (char *)buf, vb.Length) < 0)
    return socket_error;
  
  if(bytes_moved != vb.Length) 
      return socket_error = vbSOCKET_BLOCKSIZE_ERROR;

  return socket_error = vbSOCKET_NO_ERROR;  
}

int vbStream::WriteHeader(vbsSocket_t s, vbBlockHeader &vb)
// Write a variable block header to the specified socket.
// returns zero if no errors occur.
{
  // Write the block header
  if(Send(s, (char *)&vb, sizeof(vbBlockHeader)) < 0)
    return socket_error;

  // If the bytes sent do not equal the bytes moved then
  // the transmitter and receiver are no longer synchronized.
  if(bytes_moved != sizeof(vbBlockHeader)) 
    return socket_error = vbSOCKET_BLOCKHEADER_ERROR;

  return socket_error = vbSOCKET_NO_ERROR;
}

int vbStream::ReadBlock(vbsSocket_t s, void *buf, vbBlockHeader &vb)
// Read variable block from the specified socket. Returns zero
// if no errors occur.
{
  if(Recv(s, buf, vb.Length) < 0)
    return socket_error;

  // If the bytes read do not equal the bytes received then
  // the transmitter and receiver are no longer synchronized.
  if(bytes_read != vb.Length) 
    return socket_error = vbSOCKET_BLOCKSIZE_ERROR;

  return socket_error = vbSOCKET_NO_ERROR;
}

int vbStream::ReadHeader(vbsSocket_t s, vbBlockHeader &vb)
// Read a variable block header from the specified socket.
// Returns zero if no errors occur or an error number
// corresponding to a value defined in the vbsSocketError
// enumeration.
{
  if(Recv(s, (char *)&vb, sizeof(vbBlockHeader)) < 0)
    return socket_error;

  // If the bytes read do not equal the bytes received then
  // the transmitter and receiver are no longer synchronized.
  if(bytes_read != sizeof(vbBlockHeader)) 
    return socket_error = vbSOCKET_BLOCKHEADER_ERROR;
      
  if(vb.CkWord != vbCheckWord) 
    return socket_error = vbSOCKET_BLOCKSYNC_ERROR;
  	
  return socket_error = vbSOCKET_NO_ERROR;    
}

void *vbStream::RequestBlock(vbsSocket_t s, const void *request,
			   vbBlockHeader &request_header, 
			   vbBlockHeader &requested_block_header)
// Request a block from the specified socket. Returns the requested block
// or a null value if an error occurs.
{
  SetBlockStatus(request_header, vbRequestBlock);

  // Write the request header and the request itself
  if(WriteBlock(s, request, request_header) != vbSOCKET_NO_ERROR)
    return 0;
  
  // Read the returned request header 
  if(ReadHeader(s, requested_block_header) != vbSOCKET_NO_ERROR) 0;

  // Read the returned requested block
  char *buf = new char[(__ULWORD__)requested_block_header.Length];
  if(!buf) return 0;
  
  if(Recv(s, buf, requested_block_header.Length) < 0)
    return 0;

  // Check the byte count to ensure that all blocks were received.
  if(bytes_read != requested_block_header.Length) return 0; 

  return (void *)(buf); // Return the requested block
}

int vbStream::DeleteBlock(vbsSocket_t s, const void *request,
			  vbBlockHeader &request_header)
// Request the a block be deleted on the remote end. Returns
// zero if no error occur.
{
  SetBlockStatus(request_header, vbDeleteRemoteBlock);

  // Write the delete request and the request itself
  if(WriteBlock(s, request, request_header) != vbSOCKET_NO_ERROR)
    return socket_error;

  return socket_error = vbSOCKET_NO_ERROR;    
}

int vbStream::AddBlock(vbsSocket_t s, const void *block,
		       vbBlockHeader &block_header)
// Request that this block be added. Returns zero if no error 
// occur.
{
  SetBlockStatus(block_header, vbAddRemoteBlock);

  // Write the add request and then the block to add
  if(WriteBlock(s, block, block_header) != vbSOCKET_NO_ERROR)
    return socket_error;

  return socket_error = vbSOCKET_NO_ERROR;    
}

int vbStream::ChangeBlock(vbsSocket_t s, const void *request, 
			  const void *block, vbBlockHeader &request_header,
			  vbBlockHeader &block_header)
// Request that this block be changed. Returns zero if no error 
// occur.
{
  // Write the change request and the request itself
  SetBlockStatus(request_header, vbChangeRemoteBlock);
  if(WriteBlock(s, request, request_header) != vbSOCKET_NO_ERROR)
    return socket_error;

  // Write the changes
  SetBlockStatus(block_header, vbSendBlock);
  if(WriteBlock(s, block, block_header) != vbSOCKET_NO_ERROR)
    return socket_error;

  return socket_error = vbSOCKET_NO_ERROR;    
}

int vbStream::WriteAckBlock(vbsSocket_t s)
// Send an acknowledgment header to the specified socket.
// Returns zero if no errors occur.
{
  vbBlockHeader ack;
  SetBlockStatus(ack, vbAcknowledgeBlock);
  
  if(WriteHeader(s, ack) != vbSOCKET_NO_ERROR)
    return socket_error = vbSOCKET_BLOCKACK_ERROR;
  
  return socket_error = vbSOCKET_NO_ERROR;    
}

int vbStream::ReadAckBlock(vbsSocket_t s)
// Read an acknowledgment block from the specified socket.
// Returns zero if no errors occur.
{
  // Wait for an acknowledgment
  vbBlockHeader ack;
  if(ReadHeader(s, ack) != vbSOCKET_NO_ERROR)
    return socket_error = vbSOCKET_BLOCKACK_ERROR;

  // Read the status byte to determine what to do with this block
  __ULWORD__ block_status = ack.Status;
  __SBYTE__ status = (__SBYTE__)((block_status & 0xFF00)>>8);
      
  if(status != vbAcknowledgeBlock)
    return socket_error = vbSOCKET_BLOCKACK_ERROR;

  return socket_error = vbSOCKET_NO_ERROR; 
}

int vbStream::TerminateConnection(vbsSocket_t s)
// Block command used to shutdown a server or client.
// Returns zero if no errors occur.
{
  vbBlockHeader vb;
  SetBlockStatus(vb, vbKillServer);

  if(WriteHeader(s, vb) != vbSOCKET_NO_ERROR) return socket_error;

  return socket_error = vbSOCKET_NO_ERROR;  
}

int vbStream::CloseConnection(vbsSocket_t s)
// Block command used to close a persistent client or
// server connection. Returns zero if no errors occur.
{
  vbBlockHeader vb;
  SetBlockStatus(vb, vbCloseConnection);

  if(WriteHeader(s, vb) != vbSOCKET_NO_ERROR) return socket_error;

  return socket_error = vbSOCKET_NO_ERROR;  
}

int vbStream::PortNumber(int &port)
// Pass back the port number actually set by the system
// in the "port" variable.
{
  if(GetSockName() < 0) {
    return socket_error;
  }
  port = GetPortNumber();
  return socket_error = vbSOCKET_NO_ERROR;
}

int vbStream::RemotePortNumber(int &port)
// Pass back the port number actually set by the system
// in the "port" variable.
{
  port = GetRemotePortNumber();
  return socket_error = vbSOCKET_NO_ERROR;
}

int vbStream::HostName(char *hs)
{
  if(GetHostName(hs) < 0) {
    return socket_error;
  }
  return socket_error = vbSOCKET_NO_ERROR;
}

int vbStream::RemoteHostName(char *hs)
{
  if(GetRemoteHostName(hs) < 0) {
    return socket_error;
  }
  return socket_error = vbSOCKET_NO_ERROR;
}

int vbStream::WriteBlock(const void *buf, int bytes)
{
  return WriteBlock(vbsocket, buf, bytes);
}

int vbStream::TerminateConnection()
{
  return TerminateConnection(vbsocket);
}

int vbStream::CloseConnection()
{
  return CloseConnection(vbsocket);
}

int vbStream::ReadRemoteBlock(void *buf, vbBlockHeader &vb)
{
  return ReadBlock(remote_socket, buf, vb);
}

int vbStream::ReadClientHeader(vbBlockHeader &vb)
{
  return ReadHeader(remote_socket, vb);
}

int vbStream::WriteRemoteBlock(void *buf, int bytes)
{
  return WriteBlock(remote_socket, buf, bytes);
}

void *vbStream::RequestBlock(const void *request,
			     vbBlockHeader &request_header, 
			     vbBlockHeader &requested_block_header)
{
  return RequestBlock(vbsocket, request, request_header,
		      requested_block_header);
}

int vbStream::DeleteBlock(const void *request, vbBlockHeader &request_header)
{
  return DeleteBlock(vbsocket, request, request_header);
}

int vbStream::WriteAckBlock()
{
  return WriteAckBlock(vbsocket);
}

int vbStream::WriteRemoteAckBlock()
{
  return WriteAckBlock(remote_socket);
}

int vbStream::ReadAckBlock()
{
  return ReadAckBlock(vbsocket);
}

int vbStream::ReadRemoteAckBlock()
{
  return ReadAckBlock(remote_socket);
}

int vbStream::AddBlock(const void *block, vbBlockHeader &block_header)
{
  return AddBlock(vbsocket, block, block_header);
}

int vbStream::ChangeBlock(const void *request, const void *block, 
			  vbBlockHeader &request_header, 
			  vbBlockHeader &block_header)
{
  return ChangeBlock(vbsocket, request, block, request_header, block_header);
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
