// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: vbs_b64.h
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 01/25/2000
// Date Last Modified: 07/17/2000
// Copyright (c) 2000 Dougas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

Standalone functions used for base 64 password encryption and
decoding.
*/
// ----------------------------------------------------------- // 
#include <stdio.h>
#include "vbs_b64.h"

// Base 64 conversion table
const char vbsBase64table[64] = {
  'A','B','C','D','E','F','G','H',
  'I','J','K','L','M','N','O','P',
  'Q','R','S','T','U','V','W','X',
  'Y','Z','a','b','c','d','e','f',
  'g','h','i','j','k','l','m','n',
  'o','p','q','r','s','t','u','v',
  'w','x','y','z','0','1','2','3',
  '4','5','6','7','8','9','+','/'
};

int vbsCalculateBase64Length(int len)
// Calculate the number of bytes it will take to store a
// base 64 encrypted string.
{
  return 4 * (((len) + 2) / 3);
}

int vbsBase64ConvertToNumber(unsigned char c)
// Convert the encoded base 64 byte to a number.
{
  if(c >= 'A' && c <= 'Z') return (c - 'A');
  if(c >= 'a' && c <= 'z') return (c - 'a' + 26);
  if(c >= '0' && c <= '9') return (c - '0' + 52);
  if(c == '+') return 62;
  if(c == '/') return 63;
  return -1;
}

void vbsBase64Decode(const char *s, char *strbuf)
// Decode the encoded string "s" from a base64 format and pass
// back the decoded string in the "strbuf" variable.
{
  int i, ii;
  unsigned char sbuf[255]; 
  unsigned char *p = (unsigned char *)strbuf;
  int sbufSize = 0;
  int length = strlen(s);
  unsigned char out;

  for(ii = 0; ii < length; ii++) {
    i = vbsBase64ConvertToNumber(*s);

    if(i >= 0)
      sbuf[sbufSize++]=(unsigned char) i;
    if(sbufSize == 4) {
      out = (unsigned char)((sbuf[0] << 2) | (sbuf[1] >> 4));
      *p++ = out;
      
      out = (unsigned char)((sbuf[1] << 4) | (sbuf[2] >> 2));
      if(sbufSize > 2) *p++ = out;
      
      out = (unsigned char)((sbuf[2] << 6) | sbuf[3]);
      if(sbufSize > 3) *p++ = out;
      
      sbufSize=0;
    }
    s++;
  }

  // Decode the remaining bytes
  if(sbufSize) {
    out = (unsigned char)((sbuf[0] << 2) | (sbuf[1] >> 4));
    *p++ = out;
    
    out = (unsigned char)((sbuf[1] << 4) | (sbuf[2] >> 2));
    if(sbufSize > 2) *p++ = out;
    
    out = (unsigned char)((sbuf[2] << 6) | sbuf[3]);
    if(sbufSize > 3) *p++ = out;
  }
  
  *p = '\0'; // Ensure null termination
}

void vbsBase64Encode(const char *s, char *strbuf, int length)
// Encode the specified string "s" to a base64 format and pass
// back the encrypted string in the "strbuf" variable.
{
  int i;
  unsigned char *p = (unsigned char *)strbuf;

  // Transform the 3x8 bits to 4x6 bits, as required by base64 encoding
  for(i = 0; i < length; i += 3) {
    *p++ = vbsBase64table[s[0] >> 2];
    *p++ = vbsBase64table[((s[0] & 3) << 4) + (s[1] >> 4)];
    *p++ = vbsBase64table[((s[1] & 0xf) << 2) + (s[2] >> 6)];
    *p++ = vbsBase64table[s[2] & 0x3f];
    s += 3;
  }

  // Pad the result if necessary
  if(i == length + 1)
    *(p - 1) = '=';
  else if(i == length + 2)
    *(p - 1) = *(p - 2) = '=';

  *p = '\0'; // Null terminate the string
}

void vbsBasicAuthenticationEncode(const char *user, const char *passwd,
				  const char *header, vbString &wwwauth)
// Write an authentication header for the basic authentication scheme using
// base 64 encoding. The "basic" authentication scheme is based on the model
// that the user agent must authenticate itself with a user-ID and a password
// for each realm. The realm value should be considered an opaque string which
// can only be compared for equality with other realms on that server. The
// server will authorize the request only if it can validate the user-ID and
// password for the protection space of the Request-URI.
{
  char *t1, *t2, *res;
  int len1 = strlen(user) + 1 + strlen(passwd);
  int len2 = vbsCalculateBase64Length(len1);
  t1 = new char[len1 + 1];
  sprintf(t1, "%s:%s", user, passwd);
  t2 = new char[1 + len2];
  vbsBase64Encode(t1, t2, len1);
  res = new char[len2 + 11 + strlen(header)];
  sprintf(res, "%s: Basic %s\r\n", header, t2);
  wwwauth = res;
}

void vbsBasicAuthenticationDecode(const vbString &wwwauth, vbString &user,
				  vbString &passwd)
// Decode a basic authentication line and pass back the username and
// password in the "user" and "passwd" variables.
{
  vbString sbuf(wwwauth);

  // Remove the header information
  sbuf.TrimTrailingSpaces();
  sbuf.DeleteBeforeLastIncluding(" ");

  char str[1024];
  for(int i = 0; i< 1024; i++) str[i] = 0;
  vbsBase64Decode(sbuf.c_str(), str);
  user = str;
  user.DeleteAfterIncluding(":");
  passwd = str;
  passwd.DeleteBeforeIncluding(":");
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
